; $Id: adn_ccd_sat.pro,v 6.1 last revision 2013/04/19 andrea la camera $
;+
; NAME:
;       adn_ccd_sat
;
; PURPOSE:
;    CCD saturation and full-well capacity parameters added in order
;    to simulate the saturation of the CCD
;
;
; CATEGORY:
;       ...
;
; CALLING SEQUENCE:
;
;       image=adn_ccd_sat(image, par.full_well)
;     
; INPUTS:
;
;   image        = Input map [FLOAT] = Array [nb_pix, nb_pix]
;   par.full_well = number that describe the maximum capacity of each CDD pixel  
;
;
; OPTIONAL INPUTS:
;       none.
;
; KEYWORD PARAMETERS:
;
; OUTPUT:
;
;   image      = noise_free saturated map 
;
; OPTIONAL OUTPUTS:
;       none.
;
; COMMON BLOCKS:
;       none.
;
; SIDE EFFECTS:
;       none.
;
; RESTRICTIONS:
;       none.
;
; PROCEDURE:
;       none.
;
; EXAMPLE:
;       ...
;
; MODIFICATION HISTORY:
;   April 2006: Andrea La Camera (DISI),
;                  [lacamera@disi.unige.it].
;
;-
;
function adn_ccd_sat, image, value

c=where(image GE value, count)
if count GT 0 then begin
	message, 'Number of saturated pixels = '+STRTRIM(count,1)+'/'+$
		strtrim((size(image))[1]*(size(image))[2],1)+$
		'  ('+$
		strtrim(100*count/((size(image))[1]*(size(image))[2]),1)+$
		' %)', /info

	image(c)=value
endif
return, image


end
