; $Id: anb_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    anb_gen_default
;
; PURPOSE:
;    anb_gen_default generates the default parameter structure
;    for the ANB module and save it in the right location.
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    anb_gen_default
; 
; ROUTINE MODIFICATION HISTORY:
;    routine written: oct 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : march 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -add convergence plots filename browser
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : march 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                     Added verbosity choice. Disable if you want a 
;                     [default = no]
;-
;
pro anb_gen_default

; obtain module infos
info = anb_info()

; generate module description structure.
module = gen_def_module(info.mod_name, info.ver)

path = !caos_env.work+!caos_env.delim     
;Difference of magnitude convergence plot structure
   deltaM_cvplot = {deltaM_cvplot,                        $
                    enable       : 1B,                    $
                    filename     : path + 'DeltaM_bin.ps' }

;Separation convergence plot structure
   separation_cvplot = {separation_cvplot,                        $
                        enable       : 1B,                        $
                        filename     : path + 'Separation_bin.ps' }

;Position angle convergence plot structure
   PA_cvplot = {PA_cvplot,                        $
                enable       : 1B,                $
                filename     : path + 'PA_bin.ps' }

;Main component magnitude convergence plot structure
   Main_mag_cvplot = {Main_mag_cvplot,                        $
                      enable       : 1B,                      $
                      filename     : path + 'Main_mag_bin.ps' }

;Companion magnitude convergence plot structure
   Comp_mag_cvplot = {Comp_mag_cvplot,                        $
                      enable       : 1B,                      $
                      filename     : path + 'Comp_mag_bin.ps' }

;Results parameters structure saved in .sav
   cvparam = {cvparam,                                $
              enable       : 1B,                      $
              filename     : path + 'result_bin.sav'  }

par =   $
   {    $
   anb,                                      $ ; structure named anb
   module           : module,                $ ; module description structure
   verbosity        : 0B,                    $ ; verbosity (i.e. print results every iteration)
                                               ; [0B=no, 1B=yes]
   comparison       : 0B,                    $ ; comparison with binary model parameters (1=yes, 0=no)
   d_detect         : 9.,                    $ ; diameter of the detected star patch [pixel]
   tresh_detect     : 50.,                   $ ; treshold of the detection [rms background value]
   centr_box_size   : 5.,                    $ ; centroid box size [pixel]
   cflux_dir        : 1B,                    $ ; Direct calibration of the flux (1=yes, 0=no)
   cal_flux         : 100.  ,                $ ;    Calibrated flux [counts]
   cal_mag          : 10.  ,                 $ ;    Calibrated magnitude in the considered band 
   band_tab         : ['R','I','J','H','K'], $ ; observation band 
   band_index       : 0L                   , $ ; observation band index
   efficiency       : .3,                    $ ; overall efficiency (atm.+optics+detector)
   tel_surf         : 104.,                  $ ; telescope collecting surface [/m^2]
   int_time         : 1.,                    $ ; integration time [s]
   aper_phot        : [3,5,7],               $ ; Aperture photometry set [pixel]
   bin_separation   : 0.12,                  $ ; queried binary separation [arcsec]
   bin_pa           : 36.  ,                 $ ; queried binary position angle (PA) [degree]
   bin_mag          : 20.  ,                 $ ; queried binary main component magnitude
   bin_delt_m       : 2.,                    $ ; queried binary difference of magnitude
   deltaM_cvplot    : deltaM_cvplot,         $ ; Difference of magnitude convergence plot structure
   separation_cvplot: separation_cvplot,     $ ; Separation convergence plot structure
   PA_cvplot        : PA_cvplot,             $ ; Position angle convergence plot structure
   Main_mag_cvplot  : Main_mag_cvplot,       $ ; Main component magnitude convergence plot structure
   Comp_mag_cvplot  : Comp_mag_cvplot,       $ ; Companion magnitude convergence plot structure
   cvparam          : cvparam                $ ; Convergence parameters .sav structure
}

; save the default parameter structure in the default file
save, par, FILENAME=info.def_file

end
