; $Id: anb_info.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    anb_info
;
; PURPOSE:
;    anb_info is the routine that returns the basic information about
;    the module ANB.
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    info_structure = anb_info()
; 
; OUTPUTS:
;    info_structure: structure containing the module's basic information.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: october 2000,
;                     Serge Correia (OAA) [marcel@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -useless variable "calib" eliminated.
;                    -variable info.help added (instead of !caos_env.help).
;
;-
;
function anb_info

pack_name = 'AIRY_Software_Package_7.2'
                                 ; Software Package name
help_file = 'airy_help.html'
help      = !caos_env.modules+pack_name+!caos_env.delim+'help'+!caos_env.delim+help_file
                                 ; help-file address for Soft.Pack. AIRY
ver       = fix(7)               ; version number of the Software Package used

mod_name = 'anb'                ; module name
descr    = 'ANalysis Binary'    ; short module description
def_file = mk_par_name(mod_name, PACK_NAME=pack_name, /DEFAULT)
                                ; default parameter file

inp_type = 'img_t'              ; input is of image type (img_t)
out_type = ''                   ; no output
inp_opt  = 0B                   ; the input cannot be optional

init     = 1B                   ; initialization struc. needed
time     = 0B                   ; time struc. not needed

; convert in low case format and eliminate all the blanks
mod_name = strlowcase(strcompress(mod_name, /REMOVE_ALL))
inp_type = strlowcase(strcompress(inp_type, /REMOVE_ALL))
out_type = strlowcase(strcompress(out_type, /REMOVE_ALL))

info = {                    $
       pack_name: pack_name,$
       help     : help,     $
       ver      : ver,      $

       mod_name : mod_name, $
       descr    : descr,    $
       def_file : def_file, $

       inp_type : inp_type, $
       out_type : out_type, $
       inp_opt  : inp_opt,  $

       init     : init,     $
       time     : time      $
       }

return, info
end
