pro detect_binary_star, res, d, tresh, flag, out, verbosity

;+
; Detect the positions of the wanted two stars in the array "res" given the minimum detection level
; (expressed here in background rms, including the background itself units by variable "tresh") 
; and the diameter "d" of the supposed star in pixels. 
; Result is put into the structure out[0 or 1] = {x, y, pixel value}
; Originally developped by Serge Correia, OAA-INAF (Arcetri), 1998-2000?.
; Modifications: -19/01/2016: replicate call simplified,
;                 Marcel Carbillet, marcel.carbillet@unice.fr,
;                 Lagrange (UNS/OCA/CNRS).
;                -16/03/2016: added verbosity keyword,
;                 Andrea La Camera, andrea.lacamera@unige.it,
;                 DIBRIS, Univ. of Genoa, Italy.
;-
 
  dim 	  = size(res)
  nb_pix	  = long(dim[1])
  xb	  = 5
  stat 	  = moment(res[0:xb-1, 0:nb_pix-1])
  back 	  = stat[0]
  back_rms  = sqrt(stat[1])
  mask 	  = res
  mask_2    = fltarr(nb_pix, nb_pix)
  mask_out  = fltarr(nb_pix, nb_pix)
;out	  = replicate({x:0., y:0., value:0.}, fix(nb_pix^2/(d^2/4)))
  out	  = replicate({x:0., y:0., value:0.}, 2)
  flag      = 1B            ; flag non-detect binary (main component, companion or both) 
  max = max(mask, pos_max)
  if verbosity then begin
     message, 'estimated background value =' + string(back), /informational
     message, 'estimated background rms   =' + string(back_rms), /informational
     message, 'Treshold value             =' + string(tresh*back_rms+back), /informational
  endif  
  
  for n = 0, 1 do begin
     if verbosity then message, 'maximum value              =' + string(max), /informational
     if max ge tresh*back_rms+back then begin   
        mask_2 = res	
        x_max = pos_max mod nb_pix
        y_max = pos_max/nb_pix
	out[n].x 	= x_max + 0.5 ;center pixel location = pixel value + 0.5
	out[n].y 	= y_max + 0.5
	out[n].value 	= max
        for i = 0, nb_pix-1 do begin
           for j=0, nb_pix-1 do begin
              if ((i-x_max)^2 + (j-y_max)^2) lt (d^2/4) then mask[i,j] = 0.
              if (((i-x_max)^2 + (j-y_max)^2) ge (d^2/4)) or 		$
                 (mask_out[i,j] ne 0.) then mask_2[i,j] = 0.			
           endfor
        endfor
        mask_out = mask_out + mask_2
        max = max(mask, pos_max)        
     endif else begin 
        if (n eq 0) and verbosity then message, 'Main component non detected', /informational
        if (n eq 1) and verbosity then message, 'Second component non detected', /informational
        flag=0B
        return
     endelse 
  endfor
  
return
end
