pro make_graph_bin_comp, file, ap_phot_d, first, pix_size, lsty, thickk, astr_lsty, astr_tck, $
                         enable, file_out   


; ROUTINE MODIFICATION HISTORY: 
;    modifications  : 10/11/2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    - structure name result_a change into result_bin 
;                      in order to match the name given in anb_prog_code.pro.
;


restore, file

dim         = size(ap_phot_d)
ap_phot_nb  = dim[1]
legend_phot = string(ap_phot_d[first:ap_phot_nb-1], '(f3.1)')

x_ps     = 15.  ; .ps file parameters
y_ps     = 10.
x_off_ps = 2.
y_off_ps = x_ps + 3. ;landscape mode

; sym routine number correspondent to apert. phot. diameter, e.g. Psym=sym(symb[7])
;pre_defined symbols 
symb     = intarr(ap_phot_nb) 
if ap_phot_nb ge 1 then symb[0]  = 11
if ap_phot_nb ge 2 then symb[1]  = 10
if ap_phot_nb ge 3 then symb[2]  = 9
if ap_phot_nb ge 4 then symb[3]  = 6

;pre_defined line type
;lsty         = [2,1,0,3,4,5]
;thickk       = [4,4,4,4,4,4,4]
legend_lsty  = lsty[first:ap_phot_nb-1]
legend_tck   = thickk[first:ap_phot_nb-1]

!p.font = 0     
;----------------------------------------------------------------------------------
if enable[0] eq 1B then begin

set_plot, 'ps'
device, xsize=x_ps, ysize=y_ps, xoffset=x_off_ps, yoffset=y_off_ps, /PORTRAIT,	$
filename = file_out[0], /TIMES, /BOLD

        plot, result_bin.Mag_diff_int[*,first], ticklen=1,  thick = thickk[first], 	$
	xgridstyle=1, ygridstyle=1, charsize=1.2, linestyle=lsty[first],		$
	xtitle='Iteration number',  				                $
	ytitle='(!9D!XM original - !9D!XM detected)',			        $
	yrange=[floor(min(result_bin.Mag_diff_int[*, first:ap_phot_nb-1], /nan)),                        $
                ceil(max(result_bin.Mag_diff_int[*, first:ap_phot_nb-1], /nan)) ]
      
        if ap_phot_nb gt first+1 then begin
            for i=first+1, ap_phot_nb-1 do begin
                oplot, result_bin.Mag_diff_int[*,i], linestyle=lsty[i], thick = thickk[i]
            endfor
        endif

legend_windt_m, '  '+ legend_phot,                                     $
TITLE='Aperture photometry diameter [pixels]', /NOBOX,                 $
LINESTYLE=legend_lsty, POSITION=11, charsize=1.2, thick = [4,4,4]


device, /close
set_plot, 'x'

endif
;----------------------------------------------------------------------------------
if enable[1] eq 1B then begin

set_plot, 'ps'
device,  xsize=x_ps, ysize=y_ps, xoffset=x_off_ps, yoffset=y_off_ps, /PORTRAIT,  $
filename = file_out[1], /TIMES, /BOLD

        plot, result_bin.sep_all_rel*1e3, linestyle=astr_lsty, thick = astr_tck,    $
        ticklen=1, xgridstyle=1, ygridstyle=1, charsize=1.2,		         $
	xtitle='Iteration number',                      	                 $
	ytitle='(original - detected) separation [mas]'

device, /close
set_plot, 'x'

endif
;----------------------------------------------------------------------------------
if enable[2] eq 1B then begin

set_plot, 'ps'
device,  xsize=x_ps, ysize=y_ps, xoffset=x_off_ps, yoffset=y_off_ps, /PORTRAIT,   $
filename = file_out[2], /TIMES, /BOLD

        plot, result_bin.pa_all_rel, linestyle=astr_lsty, thick = astr_tck,          $
        ticklen=1, xgridstyle=1, ygridstyle=1, charsize=1.2,                      $
	xtitle='Iteration number',                      	                  $
	ytitle='(original - detected) P.A. [deg.]'

device, /close
set_plot, 'x'

endif
;----------------------------------------------------------------------------------
if enable[3] eq 1B then begin

set_plot, 'ps'
device,  xsize=x_ps, ysize=y_ps, xoffset=x_off_ps, yoffset=y_off_ps, /PORTRAIT,   $
filename = file_out[3], /TIMES, /BOLD

        plot, result_bin.delta_M[0,*,first], linestyle=lsty[first], ticklen=1, 	          $
	xgridstyle=1, ygridstyle=1, charsize=1.2, thick = thickk[first],		  $
	xtitle='Iteration number',                       	                  $
	ytitle='(M original - M detected) Main',			          $
	yrange=[floor(min(result_bin.delta_M[0,*,first:ap_phot_nb-1])),                              $
                ceil(max(result_bin.delta_M[0,*,first:ap_phot_nb-1])) ]

        if ap_phot_nb gt first+1 then begin
            for i=first+1, ap_phot_nb-1 do begin
                oplot, result_bin.delta_M[0,*,i], linestyle=lsty[i], thick = thickk[i]
            endfor
        endif

legend_windt_m, '  '+ legend_phot,                                     $
TITLE='Aperture photometry diameter [pixels]', /NOBOX,                 $
LINESTYLE=legend_lsty, POSITION=5, charsize=1.2, thick = legend_tck


;legend_windt, '  '+['1', '3', '5', '7'], PSYM=[8, 8, 8, 8],                $
;TITLE='Aperture photometry diameter [pixels]', /SYM_ONLY, /NOBOX,          $
;SYMBOL=[symb[1],symb[3],symb[5],symb[7]], POSITION=11, charsize=1.2

device, /close
set_plot, 'x'

endif
;----------------------------------------------------------------------------------
if enable[4] eq 1B then begin

set_plot, 'ps'
device,  xsize=x_ps, ysize=y_ps, xoffset=x_off_ps, yoffset=y_off_ps, /PORTRAIT,   $
filename = file_out[4], /TIMES, /BOLD

        plot, result_bin.delta_M[1,*,first], linestyle=lsty[first], ticklen=1, 	          $
	xgridstyle=1, ygridstyle=1, charsize=1.2, thick = thickk[first],		  $
	xtitle='Iteration number',                       	                  $
	ytitle='(M original - M detected) Comp.',			          $
	yrange=[floor(min(result_bin.delta_M[1,*,first:ap_phot_nb-1])),                              $
                ceil(max(result_bin.delta_M[1,*,first:ap_phot_nb-1])) ]

        if ap_phot_nb gt first+1 then begin
            for i=first+1, ap_phot_nb-1 do begin
                oplot, result_bin.delta_M[1,*,i], linestyle=lsty[i], thick = thickk[i]
            endfor
        endif

legend_windt_m, '  '+ legend_phot,                                     $
TITLE='Aperture photometry diameter [pixels]', /NOBOX,                 $
LINESTYLE=legend_lsty, POSITION=5, charsize=1.2, thick = legend_tck


device, /close
set_plot, 'x'

endif

return
end
