; $Id: anb_prog.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    anb_prog
;
; PURPOSE:
;    anb_prog represents the scientific algorithm for the
;    ANalysis Binary (ANB) module.
;
; CATEGORY:
;    module's program routine
;
; CALLING SEQUENCE:
;    error = anb_prog(inp_img_t, $ ; img_t input structure
;                     par,       $ ; parameters structure
;                     INIT=init  ) ; initialization structure
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;       see anb.pro for a detailed description. 
;
; ROUTINE MODIFICATION HISTORY: 
;    routine written: october 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : march 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]
;                    -add convergence plots filename browser
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -use of variable "calibration" eliminited for version 4.0
;                     of the whole system CAOS.
;                   : March 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]:
;                     Added verebosity choice [default = no]
;
;-
;
function anb_prog, inp_img_t, $
                   par,       $
                   INIT=init

; CAOS global common block
common caos_block, tot_iter, this_iter

; initialization of the error code: no error as default
error = !caos_error.ok

; program itself
;test if the input image is 3D (use after rft, in a single project
;dedicated to the analysis of already saved data) 
;or in 2D (tipically in the data flow of a deconvolution project)
if (size(inp_img_t.image))[0] eq 3 then begin 
   for i=0, (size(inp_img_t.image))[3]-1 do begin 
      anb_prog_code, inp_img_t.image[*,*,i], par, init, inp_img_t, i+1, (size(inp_img_t.image))[3]
   endfor
endif else begin 
   anb_prog_code, inp_img_t.image, par, init, inp_img_t, this_iter, tot_iter
endelse

; back to calling program
return, error
end
