; $Id: cbd_gui.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    cbd_gui
;
; PURPOSE:
;    cbd_gui generates the Graphical User Interface (GUI) for setting the
;    parameters of the CBD module. A parameter file
;    called cbd_yyyyy.sav is created, where yyyyy is the number n_module
;    associated to the the module instance.  The file is stored in the
;    project directory proj_name located in the working directory.
;    (see cbd.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's Graphical User Interface routine
;
; CALLING SEQUENCE:
;    error = cbd_gui(n_module, proj_name)
;
; INPUTS:
;    n_module  : number associated to the instance of the CBD module.
;                [integer scalar -- n_module > 0].
;    proj_name : name of the current project [string].
;
; OUTPUTS:
;    error: error code [long scalar].
;
; COMMON BLOCKS:
;    none.
;
; CALLED NON-IDL FUNCTIONS:
;    none.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: december 2008,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]
;    modifications  : december 2010,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -package version control eliminated.
;                   : may 2011,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                    -GUI clarified (e.g. confusion between image and object).
;                   : February 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -Output's order exchanged & text in the GUI modified
;                    -New way to call AIRY_HELP. By using the "online_help" 
;                     routine, we resolved a known-issue of the Soft.Pack.
;                   : February 2014, 
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -GUI rewritten with TAB, several optional outputs
;                     added.  
;                   : January 2015,
;                    -Add another method of Blind Deconvolution, based on the 
;                     paper cited in the file "AIRYSoftPack6.1_From6.0to6.1.txt"
;                    -Some changes on the GUI (internal tabs added and
;                     several input fields moved here and there).
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -RON and GAIN have been moved to PRE (and the
;                     values are read into the header)
;                    -GUI debugged (small changes on text and
;                     displacement of the widgets)
;                    -added optional output save: objective function
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; status setting provedure ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
pro cbd_gui_set, state
  COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band
  
;controls for the button of CBD method 
;(Desidera et al = 0, Prato et al = 1)
  if (state.par.cbd_method EQ 1B) then begin
     state.par.ok_filter = 0
     widget_control, state.id.ok_filter, SET_VALUE = 0
     widget_control, state.id.ok_filter, SENSITIVE = 0
     state.par.ok_strehl=1
     widget_control, state.id.ok_strehl, SET_VALUE = 1
     widget_control, state.id.basetab_methods, SET_TAB_CURRENT = 1
  endif else begin
     widget_control, state.id.ok_filter, /SENSITIVE
     widget_control, state.id.basetab_methods, SET_TAB_CURRENT = 0
  endelse
  
;controls for the button of initial image
  if (state.par.ok_image EQ 1B ) then begin
     widget_control, state.id.img_filename, /SENSITIVE
     widget_control, state.id.ok_image, SET_VALUE=1
  endif else begin
     widget_control, state.id.img_filename, SENSITIVE=0
    widget_control, state.id.ok_image, SET_VALUE=0
 endelse

;controls for the button of initial psf
  if (state.par.ok_psf EQ 1B ) then begin
     widget_control, state.id.psf_filename, /SENSITIVE
     widget_control, state.id.ok_psf, SET_VALUE=1
  endif else begin
     widget_control, state.id.psf_filename, SENSITIVE=0
     widget_control, state.id.ok_psf, SET_VALUE=0
  endelse
  
  
;controls for the button of filter for PSFs 
  if (state.par.ok_filter EQ 1B ) then begin
     widget_control, state.id.ok_filter, SET_VALUE=1
     widget_control, state.id.edit_filter, /SENSITIVE
  endif 
  if (state.par.ok_filter EQ 0B ) then begin
     widget_control, state.id.ok_filter, SET_VALUE=0
     widget_control, state.id.edit_filter, SENSITIVE=0
  endif 
  
 ;controls for the button of Strehl Constraint for PSFs
  if (state.par.ok_strehl EQ 1B ) then begin
     widget_control, state.id.ok_strehl, SET_VALUE=1
     widget_control, state.id.edit_strehl, /SENSITIVE
  endif else begin
     widget_control, state.id.ok_strehl, SET_VALUE=0
     widget_control, state.id.edit_strehl, SENSITIVE=0
  endelse
  
; controls for CBD B part
  if (state.par.psf_init LE 2) then begin
     widget_control, state.id.psf_id_filename, /SENSITIVE
  endif else begin
     widget_control, state.id.psf_id_filename, SENSITIVE=0
  endelse
  if (state.par.psf_init EQ 4) then begin
     widget_control, state.id.psf_userdefined, /SENSITIVE
  endif else begin
     widget_control, state.id.psf_userdefined, SENSITIVE=0
  endelse


  
;OPTIONAL OUTPUTS ---------------------------------------------
  if (state.par.save_discrepancy) then begin
     widget_control, state.id.save_discrepancy_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_discrepancy_file, SENSITIVE=0
  endelse   
  
  if (state.par.save_obj_fun) then begin
     widget_control, state.id.save_obj_fun_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_obj_fun_file, SENSITIVE=0
  endelse   

  if (state.par.save_rho1) then begin
     widget_control, state.id.save_rho1_file, /SENSITIVE
     widget_control, state.id.load_psf_file, /SENSITIVE
  endif else begin
     widget_control, state.id.save_rho1_file, SENSITIVE=0
     widget_control, state.id.load_psf_file, SENSITIVE=0
  endelse   

  if (state.par.save_rho2) then begin
     widget_control, state.id.save_rho2_file, /SENSITIVE
     widget_control, state.id.load_obj_file, /SENSITIVE
     widget_control, state.id.err_type, /SENSITIVE
     if (state.par.err_type) then begin
        widget_control, state.id.err_time, /SENSITIVE
        widget_control, state.id.err_surf, /SENSITIVE
        widget_control, state.id.err_eff, /SENSITIVE
        widget_control, state.id.err_lambda, /SENSITIVE
        widget_control, state.id.err_width, /SENSITIVE
        widget_control, state.id.err_reset, /SENSITIVE
        widget_control, state.id.err_band, /SENSITIVE
     endif else begin     
        widget_control, state.id.err_time, SENSITIVE=0
        widget_control, state.id.err_surf, SENSITIVE=0
        widget_control, state.id.err_eff, SENSITIVE=0
        widget_control, state.id.err_lambda, SENSITIVE=0
        widget_control, state.id.err_width, SENSITIVE=0
        widget_control, state.id.err_reset, SENSITIVE=0
        widget_control, state.id.err_band, SENSITIVE=0
     endelse
  endif else begin
     widget_control, state.id.save_rho2_file, SENSITIVE=0
     widget_control, state.id.load_obj_file, SENSITIVE=0
     widget_control, state.id.err_type, SENSITIVE=0
     widget_control, state.id.err_time, SENSITIVE=0
     widget_control, state.id.err_surf, SENSITIVE=0
     widget_control, state.id.err_eff, SENSITIVE=0
     widget_control, state.id.err_lambda, SENSITIVE=0
     widget_control, state.id.err_width, SENSITIVE=0
     widget_control, state.id.err_reset, SENSITIVE=0
     widget_control, state.id.err_band, SENSITIVE=0
  endelse   

;--------------------------------------------------------------
  
;;Indicating in GUI which bands have been selected

  dummy  = CLOSEST(state.par.err_lambda,lambda_tab)
  value  = INTARR(N_ELEMENTS(band_tab))
  n_band = N_ELEMENTS(band_tab)
  f_band = FLTARR(n_band)
  
  IF ((state.par.err_lambda*state.par.err_width NE 0) AND (dummy GT -1)) THEN BEGIN
     IF (((state.par.err_lambda - lambda_tab[dummy]) LT 1e-12)  AND        $
         ((state.par.err_width  - width_tab[dummy ]) LT 1e-12)) THEN BEGIN
        flag_band     = 0B      ;Indicates a single band!!
        value[dummy]  = 1
        f_band[dummy] = 1.
     ENDIF ELSE BEGIN
        flag_band = 1B          ;Indicates wavelength range spans over more than 1 band
        lambda1   = state.par.err_lambda - state.par.err_width/2
        lambda2   = state.par.err_lambda + state.par.err_width/2
        FOR i=0,n_band-1 DO BEGIN
           band1 = lambda_tab[i]-width_tab[i]/2.
           band2 = lambda_tab[i]+width_tab[i]/2.
           f_band[i] = INTERVAL2(lambda1, lambda2, band1, band2, d1, d2)
           lambda1 = d1
           lambda2 = d2
        ENDFOR
        r1 = WHERE(f_band GT 0., c1)
      IF (c1 GT 0) THEN $
         value[r1] = 1.  $
      ELSE              $
         MESSAGE,'Bandwidth selected not within the standard bands. Check!!'
   ENDELSE
     
  ENDIF
  
  WIDGET_CONTROL,state.id.err_band  ,SET_VALUE= value
;;------------------------------------------------
  
end


PRO cbd_gui_event, event
COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band

  COMMON error_block, error
  
  WIDGET_CONTROL, event.top, GET_UVALUE=state
  
   IF TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_KILL_REQUEST' THEN BEGIN
      error = !caos_error.cancel
       WIDGET_CONTROL, event.top, /DESTROY
    ENDIF
   
                                ; events
   WIDGET_CONTROL, event.id, GET_UVALUE = uvalue
   
   
   CASE uvalue OF
      
                                ; standard help button
      'help':begin
         online_help, book=(cbd_info()).help, /FULL_PATH
      end
      
                                ; standard restore button:
      'restore' : begin
         
                                ; restore the desired parameter file
         par = 0
         title = "parameter file to restore"
         restore, filename_gui(state.def_file,                          $
                               title,                                   $
                               GROUP_LEADER=event.top,                  $
                               FILTER=state.par.module.mod_name+'*sav', $
                               /NOEDIT,                                 $
                               /MUST_EXIST,                             $
                               /ALL_EVENTS                              )
         
         widget_control, state.id.ok_filter,        SET_VALUE=par.ok_filter
         widget_control, state.id.ok_strehl,        SET_VALUE=par.ok_strehl
;         widget_control, state.id.method,        SET_VALUE=par.method
         widget_control, state.id.img_filename,  SET_VALUE=par.filename
         widget_control, state.id.psf_filename,  SET_VALUE=par.filename2
         widget_control, state.id.img_filename,  SENSITIVE=0
         widget_control, state.id.psf_filename,  SENSITIVE=0
         widget_control, state.id.ok_image,      SET_VALUE=0
         widget_control, state.id.ok_psf,      SET_VALUE=0
         widget_control, state.id.iter_obj, sensitive=1
         widget_control, state.id.iter_psf, sensitive=1
         widget_control, state.id.save_discrepancy, SET_VALUE=par.save_discrepancy
         widget_control, state.id.save_discrepancy_file, SET_VALUE=par.save_discrepancy_file
         widget_control, state.id.save_obj_fun, SET_VALUE=par.save_obj_fun
         widget_control, state.id.save_obj_fun_file, SET_VALUE=par.save_obj_fun_file
         widget_control, state.id.save_rho1, SET_VALUE=par.save_rho1
         widget_control, state.id.save_rho1_file, SET_VALUE=par.save_rho1_file
         widget_control, state.id.load_psf_file,  SET_VALUE=par.load_psf_file
         widget_control, state.id.save_rho2, SET_VALUE=par.save_rho2
         widget_control, state.id.save_rho2_file, SET_VALUE=par.save_rho2_file
         widget_control, state.id.load_obj_file,  SET_VALUE=par.load_obj_file
         
         widget_control, state.id.err_type,  SET_VALUE=par.err_type
         widget_control, state.id.err_time,  SET_VALUE=par.err_time
         widget_control, state.id.err_surf,  SET_VALUE=par.err_surf
         widget_control, state.id.err_eff,  SET_VALUE=par.err_eff
         widget_control, state.id.err_lambda,  SET_VALUE=par.err_lambda
         widget_control, state.id.err_width,  SET_VALUE=par.err_width
                                ; update the current module number
         par.module.n_module = state.par.module.n_module
         
                                ; update the state structure
         state.par = par
         cbd_gui_set,state
      end
      
                                ; standard save button:
      'save' : BEGIN
         
                                ; check before saving the parameter file if filename already exists
         check_file = findfile(state.sav_file)
         if check_file[0] ne "" then begin
            answ = dialog_message(['file '+state.sav_file+' already exists.', $
                                   'do you want to overwrite it ?'],          $
                                  DIALOG_PARENT=event.top,                    $
                                  TITLE='CBD warning',                        $
                                  /QUEST)
                                ; return without saving if the user doesn't want to overwrite the
                                ; existing parameter file
            if strlowcase(answ) eq "no" then return
         endif else begin
            answ = dialog_message(['file '+state.sav_file+' will be saved.'], $
                                  DIALOG_PARENT=event.top,                    $
                                  TITLE='CBD information',                    $
                                  /INFO                                       )
                                ; inform were the parameters will be saved
         endelse
         
         par = state.par
         save, par, FILENAME = state.sav_file
         error = !caos_error.ok
         widget_control, event.top, /DESTROY
         return
         
      end
      
                                ; standard cancel button (exit without saving)
      'cancel': begin
         error = !caos_error.cancel
         WIDGET_CONTROL, event.top, /DESTROY
         return
      end
      
      'method': begin
         state.par.method=event.value
         cbd_gui_set, state
      end
      'cbd_method' : begin
         widget_control, state.id.cbd_method, GET_VALUE=dummy
         state.par.cbd_method=dummy
         cbd_gui_set,state
         end
      'ok_filter':begin
         widget_control, state.id.ok_filter, GET_VALUE=dummy
         state.par.ok_filter=dummy
         cbd_gui_set,state
      end
      'edit_filter' : begin
         if (state.par.ok_filter eq 1) then begin
            title = "Filter application GUI"
            dummy = cbd_filter_gui(title, state, GROUP_LEADER=event.top)
            state.par.ok_manage_filter=dummy.ok_manage_filter
            state.par.filtername=dummy.filtername
            state.par.d=dummy.d
            state.par.pix=dummy.pix
            state.par.baseline=dummy.baseline
            state.par.lambda=dummy.lambda
            state.par.width=dummy.width
            state.par.angles=dummy.angles
            state.par.npixel=dummy.npixel
            state.par.filtertype=dummy.filtertype                      
         endif
      end
      'ok_strehl':begin
         widget_control, state.id.ok_strehl, GET_VALUE=dummy
         state.par.ok_strehl=dummy
         cbd_gui_set,state
      end          
      'edit_strehl' : begin
         if state.par.ok_strehl eq 1 then begin
            title = "Strehl Constraint GUI"
            dummy = cbd_strehl_gui(title, state, GROUP_LEADER=event.top)
            state.par.strehl=dummy.strehl
            state.par.max_ide=dummy.max_ide
                                ;state.par.gaussian=create_gaussian()
         endif        
      end
      
      'obj_iter': begin
         widget_control, state.id.iter_obj, GET_VALUE=dummy
         state.par.iter_obj=dummy
      end
      
      'psf_iter': begin
         widget_control, state.id.iter_psf, GET_VALUE=dummy
         state.par.iter_psf=dummy
      end
      
      
      'ok_image': begin
         widget_control, state.id.ok_image, GET_VALUE=dummy
         state.par.ok_image=dummy
         cbd_gui_set, state
      end
      
      'ok_psf': begin
         widget_control, state.id.ok_psf, GET_VALUE=dummy
         state.par.ok_psf=dummy
         cbd_gui_set, state
      end
      
      'ok_psf_start': begin
         widget_control, state.id.ok_psf_start, GET_VALUE=dummy
         state.par.ok_psf_start=dummy
         cbd_gui_set, state
      end
      
      
      'obj_filename': begin
         widget_control, state.id.img_filename , GET_VALUE=dummy
         state.par.filename=dummy
      end
      
      'obj_filename2': begin
         widget_control, state.id.psf_filename , GET_VALUE=dummy
         state.par.filename2=dummy
      end

      'psf_init' : begin
         widget_control, state.id.psf_init, GET_VALUE=dummy
         state.par.psf_init = dummy
         cbd_gui_set, state
         a = ["I","C","B","A","P"]
         ;print, "Scelta PSF = ", a[dummy]
      end
      'psf_id_filename' : begin
         state.par.psf_id_filename = event.value
         ;print, "PSF ID FILENAME = ", state.par.psf_id_filename
      end
      'psf_userdefined': begin
         state.par.psf_userdefined = event.value
         ;print, "PSF user.defined = ", state.par.psf_userdefined
      end
      'sgp_verbosity' : begin
         state.par.sgp_verbosity = event.select
         ;print, "SGP VERBOSITY = ",  state.par.sgp_verbosity 
      end
      'psf_dl' : begin
         cbd_psf_dl
      end
      'save_discrepancy':begin
         state.par.save_discrepancy = event.select
         cbd_gui_set, state
      end
      'save_discrepancy_file':begin
         state.par.save_discrepancy_file=event.value
      end
      'save_obj_fun':begin
         state.par.save_obj_fun = event.select
         cbd_gui_set, state
      end
      'save_obj_fun_file':begin
         state.par.save_obj_fun_file=event.value
      end
      'save_rho1':begin
         state.par.save_rho1 = event.select
         cbd_gui_set, state
      end
      'save_rho1_file':begin
         state.par.save_rho1_file=event.value
      end 
      'load_psf_file': begin
         state.par.load_psf_file=event.value
      end
      'save_rho2':begin
         state.par.save_rho2 = event.select
         cbd_gui_set, state
      end
      'save_rho2_file':begin
         state.par.save_rho2_file=event.value
      end       
      'load_obj_file': begin
         state.par.load_obj_file=event.value
      end
      'err_type': begin
         state.par.err_type = event.value
         cbd_gui_set, state
      end
      'err_time': state.par.err_time = event.value
      'err_surf': state.par.err_surf = event.value
      'err_eff' : state.par.err_eff  = event.value
      'err_lambda': state.par.err_lambda  = event.value
      'err_width' : state.par.err_width  = event.value
      'err_band': BEGIN
         WIDGET_CONTROL, event.id , GET_VALUE=dummy
         r1 = WHERE(dummy GT 0, c1)
         IF (c1 GT 0) THEN BEGIN
            min_lambda = MIN(lambda_tab[r1]-width_tab[r1]/2.)
            max_lambda = MAX(lambda_tab[r1]+width_tab[r1]/2.)
            lambda = (max_lambda+min_lambda)/2
            width  = (max_lambda-min_lambda)
            state.par.err_lambda= lambda
            state.par.err_width = width
         ENDIF ELSE BEGIN
            state.par.err_lambda= 0.
            state.par.err_width = 0.
         ENDELSE
         WIDGET_CONTROL, state.id.err_lambda, SET_VALUE=state.par.err_lambda
         WIDGET_CONTROL, state.id.err_width,  SET_VALUE=state.par.err_width
         cbd_gui_set, state
      end
      'err_reset':BEGIN
         state.par.err_lambda = 0.
         state.par.err_width  = 0.
         WIDGET_CONTROL,state.id.err_lambda, SET_VALUE=state.par.err_lambda
         WIDGET_CONTROL,state.id.err_width , SET_VALUE=state.par.err_width
         cbd_gui_set, state
      END
      else :                    ; nothing to do ;)
   ENDCASE
   
   widget_control, event.top, SET_UVALUE=state
   
   RETURN
   
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
FUNCTION cbd_gui, n_module, proj_name, GROUP_LEADER=group
COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band
COMMON error_block, error

; retrieve the module information
info = cbd_info()

; check if a saved parameter file already exists for this module.
; if it exists it is restored, otherwise the default parameter file is restored.
sav_file = mk_par_name(info.mod_name, n_module, PROJ_NAME=proj_name)
def_file = mk_par_name(info.mod_name, PACK_NAME=info.pack_name, /DEFAULT)


par=0
check_file = findfile(sav_file)
if check_file[0] eq '' then begin
   restore, def_file
   par.module.n_module = n_module
   if (par.module.mod_name ne info.mod_name) then      $
      message, 'the default parameter file ('+def_file $
              +') is from another module: please take the right one'
endif else begin
   restore, sav_file
   if (par.module.mod_name ne info.mod_name) then $
      message, 'the parameter file '+sav_file     $
              +' is from another module: please generate a cbd one'
   if (par.module.ver ne info.ver) then       $
      message, 'the parameter file '+sav_file $
              +' is not compatible: please generate it again'
endelse

id = $
   { $
   root_base_id   : 0L, $
   method         : 0L, $
   cbd_method     : 0L, $
   ok_filter      : 0L, $
   ok_strehl      : 0L, $
   edit_filter    : 0L, $
   edit_strehl    : 0L, $
   basetab_methods : 0L, $
   psf_init : 0L, $
   psf_id_filename : 0L, $
   psf_userdefined : 0L, $
   sgp_verbosity : 0L, $
   img_filename   : 0L, $
   psf_filename   : 0L, $
   ok_image       : 0L, $
   ok_psf         : 0L, $
   ok_psf_start   : 0L, $
   iter_obj       : 0L, $
   iter_psf       : 0L, $
   save_discrepancy : 0L, $
   save_discrepancy_file : 0L, $
   save_obj_fun : 0L, $
   save_obj_fun_file : 0L, $
   save_rho1 : 0L, $
   save_rho1_file : 0L, $
   load_psf_file :0L, $
   save_rho2 : 0L, $
   save_rho2_file : 0L, $
   load_obj_file : 0L,$
   err_type : 0L, $
   err_time :0L, $
   err_surf :0L, $   
   err_eff :0L, $
   err_lambda :0L, $
   err_width :0L, $
   err_reset :0L, $   
   err_band :0L $ 
   }

state = $
  {     $
   sav_file : sav_file, $
  def_file : def_file, $
  id       : id      , $
  par      : par       $
  }



; root base definition
modal= N_ELEMENTS(group) NE 0
title= STRUPCASE(info.mod_name)+' parameter setting GUI'
root_base_id = WIDGET_BASE(TITLE=title,MODAL=modal,/COL,GROUP_LEADER=group)

; set the status structure
widget_control, root_base_id, SET_UVALUE=state

; parameters base
par_base = widget_base(root_base_id, FRAME=10, ROW=2)

par_base_id = widget_base(par_base, /COL)

base  = WIDGET_BASE(par_base_id, /FRAME, /COLUMN)
text  = WIDGET_LABEL(base,  VALUE='Constrained Blind Deconvolution based on one of the two methods (see the Help for complete references)',/align_left )
;text  = WIDGET_LABEL(base,  VALUE="CBD A - Desidera' et al. (2009) -- standard IBD, strehl constrained IBD",$
;                     /align_left )
;text  = WIDGET_LABEL(base,  VALUE="CBD B - Prato et al. (2013) -- strehl constrained IBD",$
;                     /align_left )
text  = WIDGET_LABEL(base,                                    $
                     VALUE="[N.B.: top output is for the REC'D OBJECT, "  $
                     +"while bottom output is for the REC'D PSF(s). ]" , /align_left)

basetab=WIDGET_TAB(par_base_id, location=0, uvalue='')
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
basetabA=WIDGET_BASE(basetab, /COL, TITLE='METHODS')

base_method = WIDGET_BASE(basetabA, /FRAME, /COLUMN)
base_cbd_method = WIDGET_BASE(base_method, /ROW)
state.id.cbd_method = cw_bgroup(base_cbd_method, $
                                LABEL_LEFT='Choose Blind method:',     $
                                ['CBD A) Desidera et al. (2009) - Standard IBD, Strehl Constrained IBD',$
                                 'CBD B) Prato et al. (2015)    - Strehl Constrained IBD'],$
                                ROW=2,                        $
                                SET_VALUE=state.par.cbd_method,$
                                UVALUE='cbd_method',           $
                                /EXCLUSIVE,                   $
                                /NO_RELEASE )

base_method1=widget_base(base_cbd_method,/column,/frame)
state.id.iter_obj = cw_field (base_method1, TITLE='Object iterations ', $
                              /LONG, VALUE=par.iter_obj, UVALUE='obj_iter', $
                              /ALL_EVENTS, XSIZE=10)
state.id.iter_psf = cw_field (base_method1, TITLE='PSF(s) iterations ', $
                              /LONG, VALUE=par.iter_psf, UVALUE='psf_iter', $
                             /ALL_EVENTS, XSIZE=10)

base_opt=widget_base(base_method,/row)
base_filter=widget_base(base_opt,/ROW, /FRAME)
state.id.ok_filter = cw_bgroup(base_filter, 'Apply a band filter to PSF(s)      ', $
                               COL=1, SET_VALUE=state.par.ok_filter, $
                               UVALUE='ok_filter', /NONEXCLUSIVE )
state.id.edit_filter = WIDGET_BUTTON(base_filter, VALUE='Edit...', UVALUE='edit_filter')

base_strehl=widget_base(base_opt,/row, /FRAME)
state.id.ok_strehl = cw_bgroup(base_strehl, 'Apply a Strehl Constraint to PSF(s)', $
                               COL=1, SET_VALUE=state.par.ok_strehl,$
                               UVALUE='ok_strehl', /NONEXCLUSIVE )
state.id.edit_strehl = WIDGET_BUTTON(base_strehl, VALUE='Edit...', UVALUE='edit_strehl')

;sub-tab in which the user can set the parameters of each method ....
state.id.basetab_methods = WIDGET_TAB(basetabA, location=1, uvalue='settings')
; first tab (default): DESIDERA ET AL. 
basetab_methodA = WIDGET_BASE(state.id.basetab_methods, /COL, TITLE="CBD A settings")
par_base_id2 = widget_base(basetab_methodA, /ROW)
base_image1 = WIDGET_BASE(par_base_id2, /FRAME, /COLUMN)
text  = WIDGET_LABEL(base_image1,                                        $
                     VALUE='Do you have a PSF(s) estimate?',/ALIGN_LEFT)
state.id.ok_psf = cw_bgroup(base_image1, LABEL_LEFT= '',   $
                            ['NO  [PSF(s) based on the'+string(10B)+$
                             '     autocorrelation of the observed image(s)]',$
                             'YES [PSF(s) provided by the user]'], $
                            ROW=2, SET_VALUE=state.par.ok_psf, UVALUE='ok_psf', $
                            /EXCLUSIVE)
state.id.psf_filename = cw_filename(base_image1, $
                                    TITLE='Insert the PSF(s) estimate file path',   $
                                    VALUE=state.par.filename2, UVALUE='obj_filename2', $
                                    xsize=44,/ALL_EVENTS)
base_image2 = WIDGET_BASE(par_base_id2, /FRAME, /COLUMN)
text=WIDGET_LABEL(base_image2,VALUE='What kind of initialization would you like'+string(10B)+$
                  '      to use for object reconstruction box?',/ALIGN_LEFT)
state.id.ok_image = cw_bgroup(base_image2, $
                              LABEL_LEFT= '',   $
                              ['Constant OBJECT','OBJECT provided by the user'], $
                              COLUMN=2,  SET_VALUE=0,  UVALUE='ok_image', /EXCLUSIVE)
state.id.img_filename = cw_filename(base_image2, $
                                    TITLE='Insert the object estimate file path', $
                                    VALUE=state.par.filename, UVALUE='obj_filename', $
                                    xsize=44, /ALL_EVENTS)
text = WIDGET_LABEL(base_image2,VALUE=' ')
text = WIDGET_LABEL(base_image2,VALUE='What kind of initialization would you like'+string(10B)+$
                    '      to use for PSF(s) reconstruction box?',/ALIGN_LEFT)
state.id.ok_psf_start = cw_bgroup(base_image2,                         $
                                  LABEL_LEFT= '',   $
                                  ['PSF(s) constant','Previous PSF(s)'], $
                                  COLUMN=2, SET_VALUE=state.par.ok_psf_start,  $
                                  UVALUE='ok_psf_start', /EXCLUSIVE)
; second tab: PRATO ET AL. 
basetab_methodB = WIDGET_BASE(state.id.basetab_methods, /ROW, TITLE="CBD B settings")
basetab_methodB_left = widget_base(basetab_methodB, /COL)
state.id.psf_init = cw_bgroup(basetab_methodB_left, $
                          LABEL_LEFT= 'PSF Initialization:',   $
                          ['Autocorrelation of the Ideal PSF', $
                           'Ideal PSF + a suitable constant', $
                           'Gaussian blurring of the ideal PSF', $
                           'Autocorrelation of the input image',$
                           'User-Defined PSF'], $
                          /COL,  SET_VALUE=state.par.psf_init,  $
                          UVALUE='psf_init', /EXCLUSIVE)
label = WIDGET_LABEL (basetab_methodB_left, value="")
state.id.sgp_verbosity = CW_BGROUP(basetab_methodB_left, $
                                   ['SGP Info Verbosity'], $
                                   /COL, set_value=state.par.sgp_verbosity, $
                                    UVALUE='sgp_verbosity',  /NONEXCLUSIVE)

basetab_methodB_right = widget_base(basetab_methodB, /COL)
state.id.psf_id_filename = cw_filename(basetab_methodB_right,       $
                                    TITLE='Enter the path of the Ideal PSF', $
                                    VALUE=state.par.psf_id_filename,      $
                                    UVALUE='psf_id_filename',             $
                                    /ALL_EVENTS )
state.id.psf_userdefined = cw_filename(basetab_methodB_right,       $
                                    TITLE='Enter the path of the User-Defined PSF', $
                                    VALUE=state.par.psf_userdefined,      $
                                    UVALUE='psf_userdefined',             $
                                    /ALL_EVENTS )
label = WIDGET_LABEL (basetab_methodB_right, value="Do you need an Ideal PSF? Compute it on the fly! ")
button = WIDGET_BUTTON (basetab_methodB_right, value="IDEAL PSF SIMULATOR", uvalue="psf_dl")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
basetabB=WIDGET_BASE(basetab, /COL, TITLE='OPTIONAL OUTPUTS')
base_optional = WIDGET_BASE(basetabB, /COL)
;------------------------------
base_optional2=widget_base(base_optional, /row) 
base_optional_top1=widget_base(base_optional2, /col, /frame) 
state.id.save_discrepancy=CW_BGROUP(base_optional_top1, $
                                    ['Save the discrepancy value at each iteration'], $
                                    /COL, set_value=state.par.save_discrepancy, $
                                    UVALUE='save_discrepancy',  /NONEXCLUSIVE)
state.id.save_discrepancy_file = cw_filename(base_optional_top1,       $
                                             TITLE='Enter the path of the output file', $
                                             VALUE=state.par.save_discrepancy_file,      $
                                             UVALUE='save_discrepancy_file',             $
                                             /ALL_EVENTS                         )

base_optional_top2=widget_base(base_optional2, /col, /frame) 
state.id.save_obj_fun=CW_BGROUP(base_optional_top2, $
                 ['Save the objective function value at each iteration'], $
                          /COL, set_value=state.par.save_obj_fun, $
                          UVALUE='save_obj_fun',  /NONEXCLUSIVE)
state.id.save_obj_fun_file = cw_filename(base_optional_top2,       $
                        TITLE='Enter the path of the output file', $
                        VALUE=state.par.save_obj_fun_file,      $
                        UVALUE='save_obj_fun_file',             $
                        /ALL_EVENTS                         )

;------------------------------
base_optional_bottom=widget_base(base_optional, /row, /frame) 
;------------------------------
base_optional_sx=widget_base(base_optional_bottom, /col, /frame) 
state.id.save_rho2=CW_BGROUP(base_optional_sx, $
                             ["[REC'd OBJ] Save the restoration error at each iteration"], $
                             /COL, set_value=state.par.save_rho2, $
                             UVALUE='save_rho2',  /NONEXCLUSIVE)
state.id.load_obj_file = cw_filename(base_optional_sx,       $
                                     TITLE='Enter the path of the OBJECT ground truth ', $
                                     VALUE=state.par.load_obj_file,      $
                                     UVALUE='load_obj_file',             $
                                     /ALL_EVENTS   )
state.id.save_rho2_file = cw_filename(base_optional_sx,       $
                                      TITLE='Enter the path of the output file', $
                                      VALUE=state.par.save_rho2_file,      $
                                      UVALUE='save_rho2_file',             $
                                      /ALL_EVENTS                         )
state.id.err_type=CW_BGROUP(base_optional_sx, $
                            ["[diffuse objects] Root mean square error (RMSE)", $
                             "[point-like objects] Magnitude average relative error (MARE)"], $
                            /COL, set_value=state.par.err_type, $
                            label_top='Choose the error type, based on which kind of object to be reconstructed:', $
                            UVALUE='err_type',  /EXCLUSIVE)
base_err = widget_base(base_optional_sx, /ROW) 
base_err_sx  = widget_base(base_err, /COL)
state.id.err_time= cw_field (base_err_sx, TITLE='Integration Time [sec] ', $
                              /float, VALUE=state.par.err_time, UVALUE='err_time', $
                              /ALL_EVENTS, XSIZE=10)
state.id.err_surf= cw_field (base_err_sx, TITLE='Telescope surface [m^2]', $
                              /float, VALUE=state.par.err_surf, UVALUE='err_surf', $
                              /ALL_EVENTS, XSIZE=10)
state.id.err_eff= cw_field (base_err_sx, TITLE= 'Overall efficiency     ', $
                              /float, VALUE=state.par.err_eff, UVALUE='err_eff', $
                              /ALL_EVENTS, XSIZE=10)
base_err_dx  = widget_base(base_err, /COL)
bandstuff = widget_base(base_err_dx, /COL)
; lambda
state.id.err_lambda = cw_field(bandstuff,TITLE='Center wavelength [m] ', $
                               /FLOATING, VALUE=state.par.err_lambda, UVALUE='err_lambda', $
                               /ALL_EVENTS, xsize=12)
; width
state.id.err_width = cw_field(bandstuff, TITLE='Band-width [m]        ', $
                              /FLOATING, VALUE=state.par.err_width, UVALUE='err_width',$
                              /ALL_EVENTS, xsize=12)
dummy     = N_PHOT(1.,BAND=dummy1,LAMBDA=dummy2,WIDTH=dummy3)
band_tab  = dummy1[3:9]
lambda_tab= dummy2[3:9]
width_tab = dummy3[3:9]
det_base2= WIDGET_BASE(base_err_dx, /ROW, /BASE_ALIGN_CENTER)
state.id.err_reset = WIDGET_BUTTON(det_base2,UVALUE='err_reset',VALUE='Reset Bands')
dummy  = CLOSEST(state.par.err_lambda,lambda_tab)
value  = INTARR(N_ELEMENTS(band_tab))
n_band = N_ELEMENTS(band_tab)
f_band = FLTARR(n_band)
IF (((state.par.err_lambda - lambda_tab[dummy]) LT 1e-12)  AND        $
    ((state.par.err_width  - width_tab[dummy ]) LT 1e-12)) THEN BEGIN
   flag_band     = 0B           ;Indicates a single band!!
   value[dummy]  = 1
   f_band[dummy] = 1.
   state.id.err_band = CW_BGROUP(det_base2, band_tab, /ROW, SET_VALUE=value, UVALUE='err_band', $
                                 BUTTON_UVALUE= INDGEN(n_band),/NONEXCL)
ENDIF ELSE BEGIN                ;The selected wavelength range
                                ;spans over two or more bands
   flag_band = 1B
   lambda1 = state.par.err_lambda - state.par.err_width/2
   lambda2 = state.par.err_lambda + state.par.err_width/2
   FOR i=0,n_band-1 DO BEGIN
      band1 = lambda_tab[i]-width_tab[i]/2.
      band2 = lambda_tab[i]+width_tab[i]/2.
      f_band[i] = INTERVAL2(lambda1, lambda2, band1, band2, d1, d2)
      lambda1 = d1
      lambda2 = d2
   ENDFOR
   r1 = WHERE(f_band GT 0., c1)
   IF (c1 GT 0) THEN BEGIN
      value[r1] = 1.
      state.id.err_band = CW_BGROUP(det_base2, band_tab, /ROW, SET_VALUE=value, UVALUE='err_band',$
                                    BUTTON_UVALUE= INDGEN(n_band),/NONEXCL)
   ENDIF ELSE BEGIN
      MESSAGE,'Bandwidth selected not within the standard bands. Check!!'
   ENDELSE
ENDELSE
;------------------------------
base_optional_dx=widget_base(base_optional_bottom, /col, /frame) 
state.id.save_rho1=CW_BGROUP(base_optional_dx, $
                             ["[REC'd PSF] Save the restoration error at each iteration"], $
                             /COL, set_value=state.par.save_rho1, $
                             UVALUE='save_rho1',  /NONEXCLUSIVE)
state.id.load_psf_file = cw_filename(base_optional_dx,       $
                                     TITLE='Enter the path of the PSF ground truth ', $
                                     VALUE=state.par.load_psf_file,      $
                                     UVALUE='load_psf_file',             $
                                     /ALL_EVENTS   )
state.id.save_rho1_file = cw_filename(base_optional_dx,       $
                                      TITLE='Enter the path of the output file', $
                                      VALUE=state.par.save_rho1_file,      $
                                      UVALUE='save_rho1_file',             $
                                      /ALL_EVENTS                         )
dummy = widget_label(base_optional_dx, $
                     value=" ")
dummy = widget_label(base_optional_dx, $
                     value="For the PSF, the restoration error is RMSE")
;------------------------------
;Standard buttons
;----------------
base2      = WIDGET_BASE(root_base_id, FRAME=10, /ROW)
help_id    = WIDGET_BUTTON(base2, VALUE='HELP'              ,UVALUE='help')
cancel_id  = WIDGET_BUTTON(base2, VALUE='CANCEL'            ,UVALUE='cancel')
restore_id = WIDGET_BUTTON(base2, VALUE='RESTORE PARAMETERS',UVALUE='restore')
save_id    = WIDGET_BUTTON(base2, VALUE='SAVE PARAMETERS'   ,UVALUE='save')

IF modal THEN WIDGET_CONTROL, cancel_id, /CANCEL_BUTTON
IF modal THEN WIDGET_CONTROL, save_id, /DEFAULT_BUTTON

cbd_gui_set, state

;Final stuff
;============
WIDGET_CONTROL, root_base_id, SET_UVALUE=state
WIDGET_CONTROL, root_base_id, /REALIZE


XMANAGER, 'cbd_gui', root_base_id, GROUP_LEADER=group

RETURN, error
END
