; $Id: cbd_filter_gui.pro, last revision: 2011/05/16 marcel.carbillet $
;
;+
; NAME:
;    cbd_filter_gui
;
; PURPOSE:
;    cbd_filter_gui generates a Graphical User Interface (GUI) 
;    for the choice or the manage of a band filter to apply to
;    reconstructed PSFs
;
; CATEGORY:
;    GUI
;
; CALLING SEQUENCE:
;    dummy = cbd_filter_gui (title,              $
;                         state    ,          $
;                         GROUP_LEADER=group  )
;
; INPUTS:
;    title     : the title choose for the sub-GUI.
;
; OPTIONAL INPUTS:
;    none.
;
; KEYWORD PARAMETERS:
;    none.
;
; OUTPUTS:
;    parameters for the use of a choose filter 
;
; OPTIONAL OUTPUTS:
;    none.
;
; COMMON BLOCKS:
;    common cbd_gui_block, id,par,band_tab, lambda_tab, width_tab, flag_band, f_band
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; PROCEDURE:
;    none.
;
; EXAMPLE:
;    ...
;
; PROGRAM MODIFICATION HISTORY:
;    program written: may 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it].
;    modifications  : may 2011,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr],
;                    -GUI clarified (e.g. confusion between image and object).
;
;-
;

pro cbd_filter_gui_set
COMMON cbd_gui_block1, band_tab, lambda_tab, width_tab, flag_band, f_band
COMMON cbd_gui_block2,id, par
case par.ok_manage_filter of
                  0 : begin
                       widget_control,id.filter_filename,/sensitive
                       widget_control,id.d,sensitive=0
                       widget_control,id.lambda,sensitive=0
                       widget_control,id.pix,sensitive=0
                       widget_control,id.baseline,sensitive=0
                       widget_control,id.filtertype,sensitive=0
                       widget_control,id.angles,sensitive=0
                       widget_control,id.npixel,sensitive=0
                       widget_control,id.width,sensitive=0
                       widget_control,id.text0,sensitive=0
                       widget_control,id.band,sensitive=0
                       widget_control,id.reset,sensitive=0
                       widget_control,id.text1,sensitive=0
                       ;widget_control,id.text2,sensitive=0
                       widget_control,id.text3,sensitive=0
                       end
                  
                  1 : begin
                       widget_control,id.filter_filename,sensitive=0
                       widget_control,id.d,/sensitive
                       widget_control,id.lambda,/sensitive
                       widget_control,id.pix,/sensitive
                       widget_control,id.baseline,/sensitive
                       widget_control,id.filtertype,/sensitive
                       widget_control,id.angles,/sensitive
                       widget_control,id.npixel,/sensitive
                       widget_control,id.width,/sensitive
                       widget_control,id.text0,/sensitive
                       widget_control,id.band,/sensitive
                       widget_control,id.reset,/sensitive
                       widget_control,id.text1,/sensitive
                       widget_control,id.text3,/sensitive
                      end
                  endcase

dummy  = CLOSEST(par.lambda,lambda_tab)
value  = INTARR(N_ELEMENTS(band_tab))
n_band = N_ELEMENTS(band_tab)
f_band = FLTARR(n_band)

IF ((par.lambda*par.width NE 0) AND (dummy GT -1)) THEN BEGIN
   IF (((par.lambda - lambda_tab[dummy]) LT 1e-12)  AND        $
       ((par.width  - width_tab[dummy ]) LT 1e-12)) THEN BEGIN
      flag_band     = 0B           ;Indicates a single band!!
      value[dummy]  = 1
      f_band[dummy] = 1.
   ENDIF ELSE BEGIN
      flag_band = 1B               ;Indicates wavelength range spans over more than 1 band
      lambda1   = par.lambda - par.width/2
      lambda2   = par.lambda + par.width/2
      FOR i=0,n_band-1 DO BEGIN
         band1 = lambda_tab[i]-width_tab[i]/2.
         band2 = lambda_tab[i]+width_tab[i]/2.
         f_band[i] = INTERVAL2(lambda1, lambda2, band1, band2, d1, d2)
         lambda1 = d1
         lambda2 = d2
      ENDFOR
      r1 = WHERE(f_band GT 0., c1)
      IF (c1 GT 0) THEN $
        value[r1] = 1.  $
      ELSE              $
        MESSAGE,'Bandwidth selected not within the standard bands. Check!!'
   ENDELSE

ENDIF

WIDGET_CONTROL,id.band  ,SET_VALUE= value



end




;;;;;;;;;;;;;;;;;;;;;;;
; GUI event loop
;;;;;;;;;;;;;;;;;;;;;;;

pro cbd_filter_gui_event, event

common cbd_gui_block1,band_tab, lambda_tab, width_tab, flag_band, f_band
common cbd_gui_block2, id, par

widget_control, event.id, GET_UVALUE=uvalue

case uvalue of

'filter_filename': begin
                  widget_control, id.filter_filename , GET_VALUE=dummy
		       par.filtername=dummy
                  end

'filtertype':begin
               widget_control, id.filtertype, GET_VALUE=dummy
               par.filtertype=dummy    
               end
'whofilter':begin
               widget_control, id.who_filter, GET_VALUE=dummy
               par.who_filter=dummy
             end

'pix_val':   begin
               par.pix=event.value   
               end

'dimension_val':  begin
               par.npixel=event.value    
                    end
'baseline_val':  begin
               par.baseline=event.value    
                    end

'lambda_val':  begin
               par.lambda=event.value 
                    end

'width_val':  begin
               par.width=event.value   
                    end

'diameter_val':  begin
               par.d=event.value   
                    end
    'ok_manage_filter':begin
                  widget_control, id.ok_manage_filter , GET_VALUE=dummy
                  par.ok_manage_filter=dummy
                  cbd_filter_gui_set
                  end

   'angles_val':begin
                  ;widget_control, id.angles, GET_VALUE=dummy    
                  par.angles=event.value
                       end

'menu_band': BEGIN
       WIDGET_CONTROL, event.id , GET_VALUE=dummy
       r1 = WHERE(dummy GT 0, c1)
       IF (c1 GT 0) THEN BEGIN
          min_lambda = MIN(lambda_tab[r1]-width_tab[r1]/2.)
          max_lambda = MAX(lambda_tab[r1]+width_tab[r1]/2.)
          lambda = (max_lambda+min_lambda)/2
          width  = (max_lambda-min_lambda)
          par.lambda= lambda
          par.width = width
       ENDIF ELSE BEGIN
          par.lambda= 0.
          par.width = 0.
       ENDELSE
       WIDGET_CONTROL, id.lambda, SET_VALUE=par.lambda
       WIDGET_CONTROL, id.width,  SET_VALUE=par.width
       cbd_filter_gui_set
    END

'reset':BEGIN
       par.lambda = 0.
       par.width  = 0.
       WIDGET_CONTROL,id.lambda, SET_VALUE=par.lambda
       WIDGET_CONTROL,id.width , SET_VALUE=par.width
       cbd_filter_gui_set
    END



'ok':  widget_control, event.top, /DESTROY

endcase

end

;;;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code
;;;;;;;;;;;;;;;;;;;;;;;;;

function cbd_filter_gui, title, state, GROUP_LEADER=group

common cbd_gui_block1, band_tab, lambda_tab, width_tab, flag_band, f_band
common cbd_gui_block2,id, par
error = !caos_error.ok



id = { $
         root_base_id0   : 0L, $
         ok_manage_filter:0L, $
         filter_filename:0L, $
         d:0L,$
         baseline: 0L,$
         lambda: 0L,$
         width:0L,$
         npixel:0L,$
         pix:0L, $
         filtertype:0L, $
         who_filter:0L,$
         angles:0L, $
         text0:'', $
         text1:'', $
         text2:'', $
         text3:'', $
         text4:'', $
         reset:0L, $
         band:0L $
           }

par ={$
          ok_manage_filter:state.par.ok_manage_filter, $
          filtername:state.par.filtername, $
          d:state.par.d,$
          baseline: state.par.baseline,	$ 
          lambda: state.par.lambda,$
          pix: state.par.pix,$
          npixel: state.par.npixel,$
          width: state.par.width, $
          filtertype: state.par.filtertype, $
          who_filter: state.par.who_filter,$
          angles: state.par.angles $  
}

modal = n_elements(group) ne 0
root_base_id0 = widget_base(TITLE=title,/COL,MODAL=modal, GROUP_LEADER=group)
root_base_id = widget_base(root_base_id0, FRAME=5,/COL)

title=widget_base(root_base_id,/FRAME,/row)
text = WIDGET_LABEL(title, VALUE='                 Selection of the filter(s) for PSF(s) reconstruction')

text = WIDGET_LABEL(root_base_id, VALUE=' ')

choice=["Filter(s) provided by the user","Filter(s) derived from the ideal PSF(s)"]

base_filter1=widget_base(root_base_id,/FRAME,/row)
id.ok_manage_filter = cw_bgroup(base_filter1,               $
                            LABEL_LEFT='', $
                            choice,               $
			                COL=2,			     $
                            SET_VALUE=state.par.ok_manage_filter,      $
                            UVALUE='ok_manage_filter',                 $
                            /EXCLUSIVE,                      $
		                    /NO_RELEASE                      $
			    )

base_filter2=widget_base(root_base_id,/FRAME,/row)
id.filter_filename = cw_filename(base_filter2,	                        $
                        TITLE='Insert the FILTER path',                $
                        VALUE=state.par.filtername,	                    $
                        UVALUE='filter_filename',                              $
                        /ALL_EVENTS                                         )

base_filter_lbt=widget_base(root_base_id,/FRAME,/column)
id.text0  = WIDGET_LABEL(base_filter_lbt, VALUE='Create filter from the ideal PSF(s)')

base_filter3=widget_base(base_filter_lbt,/row)
base_filter4=widget_base(base_filter3,/column)
id.d = cw_field (base_filter4,                  $
                        TITLE='Diameter  [m]       ',$
                         VALUE=state.par.d ,$
                         uvalue='diameter_val',$
                          /all_events )

id.pix = cw_field(base_filter4,                  $
                        TITLE='Pixel Size [arcsec] ',      $
                         /FLOATING, $
                        VALUE=state.par.pix, $
                         uvalue='pix_val',$
                         /all_events)

id.npixel= cw_field(base_filter4,                  $
                        TITLE='Dimension           ',      $
                         /LONG,$
                        VALUE=state.par.npixel,$
                         uvalue='dimension_val',$
                         /all_events)


id.angles = cw_field(base_filter4,$
                           title='Orientations        ',$
                                   /all_events,$
                                  /STRING, $
                                   VALUE=par.angles,$
                                   uvalue='angles_val')

base_filter5=widget_base(base_filter3,/column)
id.baseline = cw_field(base_filter5,$
                                   title='Baseline [m]',$
                                   value=state.par.baseline ,$
                                   uvalue='baseline_val',$
                                    /all_events )

id.lambda = cw_field(base_filter5,       $
                            title='Lambda   [m]', $
                            /FLOATING,$
                            VALUE=state.par.lambda, $
                             uvalue='lambda_val',$
                              /all_events)

id.width =   cw_field(base_filter5,       $
                            title='Width    [m]', $
                             /FLOATING,$
                            VALUE=state.par.width, $
                             uvalue='width_val',$
                              /all_events)


dummy     = N_PHOT(1.,BAND=dummy1,LAMBDA=dummy2,WIDTH=dummy3)
band_tab  = dummy1[3:7]
lambda_tab= dummy2[3:7]
width_tab = dummy3[3:7]

det_base2= WIDGET_BASE(base_filter5, /ROW, /BASE_ALIGN_CENTER)

id.reset = WIDGET_BUTTON(det_base2,UVALUE='reset',VALUE='Reset Bands')

dummy  = CLOSEST(state.par.lambda,lambda_tab)
value  = INTARR(N_ELEMENTS(band_tab))
n_band = N_ELEMENTS(band_tab)
f_band = FLTARR(n_band)

IF (((state.par.lambda - lambda_tab[dummy]) LT 1e-12)  AND        $
    ((state.par.width  - width_tab[dummy ]) LT 1e-12)) THEN BEGIN

   flag_band     = 0B                        ;Indicates a single band!!
   value[dummy]  = 1
   f_band[dummy] = 1.
   id.band =                                                             $
     CW_BGROUP(det_base2, band_tab, /ROW, SET_VALUE=value, UVALUE='menu_band', $
               BUTTON_UVALUE= INDGEN(n_band),/NONEXCL)

ENDIF ELSE BEGIN                             ;The selected wavelength range
                                             ;spans over two or more bands

   flag_band = 1B

   lambda1 = state.par.lambda - state.par.width/2
   lambda2 = state.par.lambda + state.par.width/2

   FOR i=0,n_band-1 DO BEGIN
      band1 = lambda_tab[i]-width_tab[i]/2.
      band2 = lambda_tab[i]+width_tab[i]/2.
      f_band[i] = INTERVAL2(lambda1, lambda2, band1, band2, d1, d2)
      lambda1 = d1
      lambda2 = d2
   ENDFOR

   r1 = WHERE(f_band GT 0., c1)

   IF (c1 GT 0) THEN BEGIN
      value[r1] = 1.
       id.band =                                                            $
        CW_BGROUP(det_base2, band_tab, /ROW, SET_VALUE=value, UVALUE='menu_band',$
                  BUTTON_UVALUE= INDGEN(n_band),/NONEXCL)
   ENDIF ELSE BEGIN
      MESSAGE,'Bandwidth selected not within the standard bands. Check!!'
   ENDELSE

ENDELSE
id.text1  = WIDGET_LABEL(base_filter_lbt, VALUE='[P.S.:Insert orientation angles separated by "-".  Ex.: 0-60-120... ]',/ALIGN_LEFT)


base_filter6=widget_base(root_base_id,/frame,/column)

id.text3  = WIDGET_LABEL(base_filter6, VALUE='                Filter type:')
id.filtertype = cw_bgroup(base_filter6,            $
                            label_left='                    ',	$
                            ['Step-like','Gaussian-like','Butterworth-like'],        $
			                COL=3,			     $
                            SET_VALUE=state.par.filtertype,      $
                            UVALUE='filtertype',                 $
                            /EXCLUSIVE,                      $
		            /NO_RELEASE                      )
		            
base_filter7=widget_base(root_base_id,/frame,/column)	
	            
id.text4  = WIDGET_LABEL(base_filter7, VALUE='             FILTER APPLIED TO:')		            
id.who_filter = cw_bgroup(base_filter7,            $
                            label_left='                    ',	$
                            ['ONLY PSF(s)','BOTH PSF(s) and OBJECT'],        $
			    COL=2,			     $
                            SET_VALUE=state.par.who_filter,      $
                            UVALUE='whofilter',                 $
                            /EXCLUSIVE,                      $
		            /NO_RELEASE                      )		            
		            
; standard buttons base

btn_base_id = widget_base(root_base_id0,FRAME=5, /ROW)

base = WIDGET_BASE (btn_base_id, /COLUMN)
text = WIDGET_LABEL(base,      $
                    VALUE=''   )

ok_id = widget_button(btn_base_id,                               $
                      VALUE="Back to cbd parameter setting GUI", $
                      UVALUE="ok"                                )

;if modal then widget_control, ok_id, /CANCEL_BUTTON
WIDGET_CONTROL, root_base_id0, SET_UVALUE=id
widget_control, root_base_id0, /REALIZE
cbd_filter_gui_set

xmanager, 'cbd_filter_gui', root_base_id0, GROUP_LEADER=group



return, par

end
