; $Id: cbd_strehl_gui.pro 2011/05/12 marcel.carbillet $
;
;+
; NAME:
;    cbd_filter_gui
;
; PURPOSE:
;    cbd_filter_gui generates a Graphical User Interface (GUI) 
;    for the choice or the manage of a band filter to apply to
;    reconstructed PSFs
;
; CATEGORY:
;    GUI
;
; CALLING SEQUENCE:
;    dummy = cbd_filter_gui (title,              $
;                         state    ,          $
;                         GROUP_LEADER=group  )
;
; INPUTS:
;    title     : the title choose for the sub-GUI.
;
; OPTIONAL INPUTS:
;    none.
;
; KEYWORD PARAMETERS:
;    none.
;
; OUTPUTS:
;    parameters for the use of a choose filter 
;
; OPTIONAL OUTPUTS:
;    none.
;
; COMMON BLOCKS:
;    common cbd_gui_block, id,par,band_tab, lambda_tab, width_tab, flag_band, f_band
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; PROCEDURE:
;    none.
;
; EXAMPLE:
;    ...
;
; PROGRAM MODIFICATION HISTORY:
;    program written: december 2008,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it].
;    modifications  : may 2011,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr]:
;                    -Strehl RATIO, not percentage.
;
;-
;

;pro cbd_strehl_gui_set
;COMMON cbd_gui_block2, id, par,state

;end
;;;;;;;;;;;;;;;;;;;;;;;
; GUI event loop
;;;;;;;;;;;;;;;;;;;;;;;

pro cbd_strehl_gui_event, event

common cbd_gui_block2, id,par


widget_control, event.id, GET_UVALUE=uvalue

case uvalue of

   'reset_strehl' : begin
                          par.strehl[*]=0.0
                          widget_control, id.strehl, SET_Value=transpose(par.strehl)
                       end

   'strehl'  : begin
                   dummyx = event.x
                   dummyy = event.y
                   widget_control, id.strehl, get_Value=dummy
                   par.strehl[dummyy]=dummy[dummyx,dummyy]
                  end
                  
    'max_ide': begin
                widget_control, id.max_ide, GET_VALUE=dummy
                par.max_ide=double(dummy)
               end
               
     'cmpt_max_ide': begin
                      title = "PSF parameters"
                      dummy=cbd_cmpt_max_ide(title, state, GROUP_LEADER=event.top)
                      widget_control, id.max_ide, set_value=double(dummy.max_ide)
                      par.max_ide=double(dummy.max_ide)
                     end

   'ok':  widget_control, event.top, /DESTROY

endcase

end

;;;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code
;;;;;;;;;;;;;;;;;;;;;;;;;

function cbd_strehl_gui, title, state, GROUP_LEADER=group

common cbd_gui_block2, id,par

error = !caos_error.ok


id = { $
         root_base_id0   : 0L, $
         strehl          : 0L, $
         max_ide         : 0L, $
         cmpt_max_ide    : 0L, $
         reset_strehl    : 0L  $
      }

par ={$
     max_ide: double(state.par.max_ide), $
     strehl : state.par.strehl   $
     }

modal = n_elements(group) ne 0
root_base_id0 = widget_base(TITLE=title,/COL,MODAL=modal, GROUP_LEADER=group)
root_base_id = widget_base(root_base_id0, FRAME=5,/COL)

title=widget_base(root_base_id,/FRAME,/row,xoffset=30)
text = WIDGET_LABEL(title, VALUE='Strehl Constraint over reconstructed PSFs')


strehl_table=widget_base(root_base_id,xsize=180,ysize=160)
rlabel=make_array(360,/string,value='Frame #')
num=indgen(360)
rlabel=rlabel+strcompress(string(num),/remove_all)
;button_strehl=widget_base(strehl_table,xoffset=11,yoffset=3,xsize=64,ysize=26)



id.strehl = widget_table(strehl_table,$
                       ALIGNMENT=2,$
                       xoffset=8,yoffset=10,$
                       value=transpose(state.par.strehl),$
                       uvalue="strehl",$
                       editable=1,$
                       COLUMN_LABELS=['Strehl Ratio [0<SR<1]'],$
                       ROW_LABELS=rlabel,$
                       COLUMN_WIDTHS=150,$
                       y_scroll_size=5   )
base_max_ide=widget_base(root_base_id,/row)
id.max_ide= cw_field(base_max_ide,           $
                     title='Peak of the ideal PSF',$
                     /floating,$
                     value=double(state.par.max_ide),$
                     uvalue="max_ide",$
                     xsize=16,$
                     /all_events $
                     )
;cmpt_max_ide=widget_button(base_max_ide,$
;                      VALUE="<- Compute it", $
;                      UVALUE="cmpt_max_ide" $
;                      )



; standard buttons base

btn_base_id = widget_base(root_base_id0,FRAME=5, /ROW)

id.reset_strehl=widget_button(btn_base_id,$
                      VALUE="Reset", $
                      UVALUE="reset_strehl" $
                      )

ok_id = widget_button(btn_base_id,                               $
                      VALUE="Back to cbd parameter setting GUI", $
                      UVALUE="ok"                                )

;if modal then widget_control, ok_id, /CANCEL_BUTTON
WIDGET_CONTROL, root_base_id0, SET_UVALUE=id
widget_control, root_base_id0, /REALIZE
;cbd_strehl_gui_set

xmanager, 'cbd_strehl_gui', root_base_id0, GROUP_LEADER=group

return, par

end
