; $Id: create_lbt_psf.pro,v 1.1 2005/05/01 10.44 gabriele Exp $
;
;+
; NAME:
;    create_lbt_psf.pro
;
; PURPOSE:
;     create_lbt_psf creates LBT psf(s)
;     
;
; CATEGORY:
;    FUNCTION
;
; CALLING SEQUENCE:
;    dummy = create_lbt_psf(par)
;
; INPUTS:
;    par     : the parameter structure (the same of the whole module cbd).
;
; OPTIONAL INPUTS:
;    none.
;
; KEYWORD PARAMETERS:
;    none.
;
; OUTPUTS:
;    LBT's PSFs 
;
; OPTIONAL OUTPUTS:
;    none.
;
; COMMON BLOCKS:
;    none.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; PROCEDURE:
;    psf_interf.pro
;
; EXAMPLE:
;    ...
;
; PROGRAM MODIFICATION HISTORY:
;
;-
;

function create_lbt_psf,par


b = par.baseline               ;Baseline of the interferometer [m]
d = par.d               ;Diameter of one pupil [m]
pix_size = par.pix      ;Detector pixel size [arcsec]
lambda = par.lambda        ;Central filter wavelenght [m]
nb_pix =par.npixel           ;Ingoing size array [pixels]


a=strcompress(par.angles,/remove_all)

gam=float(strsplit(a,'-',/extract))     ; Array containing the parallactic  angles (degrees)
dim_gam=size(gam)
nb_angl = dim_gam[1]            ;Number of equidistant parallactic angles

;************************** Make object **********************************

psf =make_array(nb_pix,nb_pix,nb_angl,/float,value=0.)

;************************* Make normalised PSF ****************************


For i=0,nb_angl-1 do begin
      psfpar=psf[*,*,i]
      psf_interf, b, d, lambda, gam[i], pix_size, nb_pix, psfpar
      psf[*,*,i]=psfpar
endfor

return,psf
end
