; $Id: cbd_info.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    cbd_info
;
; PURPOSE:
;    cbd_info is the routine that returns the basic information about
;    the module CBD.
;
; CATEGORY:
;    utility
;
; CALLING SEQUENCE:
;    info_structure = cbd_info()
; 
; OUTPUTS:
;    info_structure: structure containing the module's basic information.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: december 2008, 
;                     Gabriele Desidera' (DISI) [desidea@disi.unige.it].
;    modifications  : ...
;                      
;-
function cbd_info

pack_name = 'AIRY_Software_Package_7.2'
                                 ; Software Package name
help_file = 'airy_help.html'
help      = !caos_env.modules+pack_name+!caos_env.delim+'help'+!caos_env.delim+help_file
                                 ; help-file address for Soft.Pack. AIRY
ver       = fix(7)               ; version number of the Software Package used

mod_name = 'cbd'                ; Module name. Only LOWCASE string allowed
descr    = 'Constrained Blind Deconvolution'    ; Short module description
def_file = mk_par_name(mod_name, PACK_NAME=pack_name, /DEFAULT) ; default parameter file

init     = 1B                   ; initialization structure is needed
time     = 0B                   ; no time delay/integration management

inp_type = 'img_t'        ; input data types.
out_type = 'img_t,img_t'              ; output data types.
inp_opt  = [0B,0B]              ; inputs may not optional

; convert in low case format and eliminate all the blanks
mod_name = strlowcase(strcompress(mod_name, /REMOVE_ALL))
inp_type = strlowcase(strcompress(inp_type, /REMOVE_ALL))
out_type = strlowcase(strcompress(out_type, /REMOVE_ALL))

; control the module name length
if strlen(mod_name) ne !caos_env.module_len then $
   message, 'the module name must have '+strtrim(!caos_env.module_len)+' chars'

info = {                    $
       pack_name: pack_name,$
       help     : help,     $
       ver      : ver,      $

       mod_name : mod_name, $
       descr    : descr,    $

       def_file : def_file, $
       inp_type : inp_type, $
       out_type : out_type, $

       init     : init,     $
       time     : time      $
       }

return, info
end
