; $Id: adjust_strehl.pro, v. 6.1, last revision 2016/04/05 marcel.carbillet $
;
;+
; NAME:
;       adjust_strehl
;
; PURPOSE:
;       ...
;
; CATEGORY:
;       scientific program
;
; CALLING SEQUENCE:
;       ...
;
; INPUTS/OUTPUTS/KEYWORDS/ETC.:
;       ...
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: December 2008?,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it].
;                   : May 2014,
;                     Marcel Carbillet (Lagrange) [marcel.carbillet@unice.fr]:
;                    -some printing to know how/when the constraint is applied.
;                    -a tolerance of 0.01 for the Strehl is now considered.
;
;
;-
function adjust_strehl, psf, tf_g, strehl, max_ide, N, M

; fixed parameter
tol_strehl = 0.01

; CAOS global common block
common caos_block, tot_iter, this_iter
  
tmp=psf
s=max(tmp)/max_ide
print, '--- ITER #'+strtrim(this_iter,2)
print,'BEGIN STREHL CONSTRAINT'
print,'Strehl [given]=', strehl
print,'Strehl [calcd]=', s
while (s gt strehl+tol_strehl) do begin ; a tolerance is considered for the
                                        ; exact Strehl value not to go above,
                                        ; otherwise the risk is, on the other
                                        ; hand, to go much lower then the limit
                                        ; given in terms of Strehl ratio.
  tf=fft(tmp,-1,/double)*N*M
  tmp=double(fft(tf_g*tf,1,/double)/(N*M))
  s=max(tmp)/max_ide
  print,'Strehl [calcd]=', s
endwhile
print,'END STREHL CONSTRAINT'

;pos=where(double(tmp) lt 0.,nummin)
;print,nummin

return, tmp
end