function cbd_afunction_multi, X, TF

; CBD_* VERSION FOR AIRY'S CBD MODULE contains some small
; modifications for adapting existing code to CBD's code--

; afunction - Implements the blurring linear operator via FFT
;   This function applies the blurring operator A to the
;   reconstruction x of the unknown object in the Fourier space, by
;   componentwise multiplication of the two FFTs and then by transforming
;   back the result by an IFFT. The routine accounts for multiple psf and
;   images.
;
; SYNOPSIS
;   out = afunction(x, TF)
;
; INPUT
;   x          (double)        - reconstruction
;   TF         (dcomplex)      - Fourier transform of the blurring operator A
;
; OUTPUT
;   out        (double)        - vectorized blurred version of x
;
; ------------------------------------------------------------------------------

dimTF = size(TF,/dimensions)
if (n_elements(dimTF) eq 2) then dimTF = [dimTF,1]
dimX = size(X,/dimensions)
out = dblarr(dimTF)
if (n_elements(dimX) gt 2) then begin
	 for i = 0, dimTF[2]-1 do begin
		 out[*,*,i] = real_part(fft(reform(TF[*,*,i])*fft(reform(X[*,*,i])),/inverse))
	 endfor
endif else begin
     x_t = fft(X)
	 for i = 0, dimTF[2]-1 do begin
 		 out[*,*,i] = real_part(fft(reform(TF[*,*,i])*x_t,/inverse))
	 endfor
endelse
return, out

end

; ==========================
; End of Afunction.pro file
; ==========================
