; $Id: create_filter.pro,v 1.1 2005/05/01 10.44 gabriele desidera' Exp $
;
;+
; NAME:
;    create_filter.pro
;
; PURPOSE:
;    create_filter generates a filter with the parameters
;    choosen by the user
;    
;
; CATEGORY:
;    FUNCTION
;
; CALLING SEQUENCE:
;    dummy = create_filter(par)
;
; INPUTS:
;    par     : the parameter structure (the same of the whole module cbd).
;
; OPTIONAL INPUTS:
;    none.
;
; KEYWORD PARAMETERS:
;    none.
;
; OUTPUTS:
;    image filter to apply to the recontructed PSFs 
;
; OPTIONAL OUTPUTS:
;    none.
;
; COMMON BLOCKS:
;    none.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; PROCEDURE:
;    none.
;
; EXAMPLE:
;    ...
;
; PROGRAM MODIFICATION HISTORY:
;    program written: May 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it].
;    modifications  : 
;
;
;-
;


function create_filter,par

;case par.ok_manage_filter of 
;'1': begin
;            fits_read,par.filtername,psf
;     end
;'2': begin
;           psf=create_lbt_psf(par)
;     end
;endcase

a=strcompress(par.angles,/remove_all)
gam=float(strsplit(a,'-',/extract))

mask=create_id_filter(par.d,par.baseline,par.lambda,par.npixel,par.pix,gam)
dim=size(mask)

if dim[1] eq 3 then begin
case par.filtertype of

0: begin    ; step filter

    end

1: begin    ;gaussian filter

     sigma=3.
     gau=make_array(dim[1],dim[2],/double,value=0.)

     for i=0,dim[1]-1 do begin
     for j=0,dim[2]-1 do begin
        gau[i,j]=(1./(sqrt(2*!pi)*sigma))*exp(-0.5*(((sqrt((i-((dim[1])/2))^2+(j-((dim[2])/2))^2))/sigma)^2))
     endfor
     endfor
     m=min(gau[where(gau gt 0)])
     pos=where(finite(gau) eq 0,num)
     if num gt 0 then begin
       gau[pos]=m
     endif
     gau=gau/max(gau)

     for i=0,dim[3]-1 do begin
        mask[*,*,i]=shift(fft((fft(mask[*,*,i],-1,/double)*dim[1]*dim[2])*(fft(gau,-1,/double)*dim[1]*dim[2]),1,/double)/dim[1]/dim[2],(dim[1]/2),(dim[2]/2))
        mask[*,*,i]=mask[*,*,i]/max(mask[*,*,i])
     endfor
     end

2: begin  ; butterworth filter

     but=make_array(dim[1],dim[2],/double,value=0.)
     n=5.
     cf=5.
     for i=0,dim[1]-1 do begin
     for j=0,dim[2]-1 do begin
        but[i,j]=(1.)/(sqrt(1.+((sqrt((i-(dim[1]/2))^2+(j-(dim[2]/2))^2))/cf)^(2*n)))
     endfor
     endfor
     m=min(but[where(but gt 0)])
     pos=where(finite(but) eq 0,num)
     if num gt 0 then begin
        but[pos]=m
     endif
     but=but/max(but)

     for i=0,dim[3]-1 do begin
        mask[*,*,i]=shift(fft((fft(mask[*,*,i],-1,/double)*dim[1]*dim[2])*(fft(but,-1,/double)*dim[1]*dim[2]),1,/double)/dim[1]/dim[2],(dim[1]/2),(dim[2]/2))
        mask[*,*,i]=mask[*,*,i]/max(mask[*,*,i])
     endfor
     end

endcase
endif else begin

case par.filtertype of

0: begin    ; step filter

     end

1: begin    ;gaussian filter

     sigma=3.
     gau=make_array(dim[1],dim[2],/double,value=0.)

     for i=0,dim[1]-1 do begin
     for j=0,dim[2]-1 do begin
        gau[i,j]=(1./(sqrt(2*!pi)*sigma))*exp(-0.5*(((sqrt((i-((dim[1])/2))^2+(j-((dim[2])/2))^2))/sigma)^2))
     endfor
     endfor
     m=min(gau[where(gau gt 0)])
     pos=where(finite(gau) eq 0,num)
     if num gt 0 then begin
       gau[pos]=m
     endif
     gau=gau/max(gau)

     mask=shift(fft((fft(mask,-1,/double)*dim[1]*dim[2])*(fft(gau,-1,/double)*dim[1]*dim[2]),1,/double)/dim[1]/dim[2],(dim[1]/2),(dim[2]/2))
     mask=mask/max(mask)
     end

2: begin  ; butterworth filter

     but=make_array(dim[1],dim[2],/double,value=0.)
     n=5.
     cf=5.
     for i=0,dim[1]-1 do begin
     for j=0,dim[2]-1 do begin
        but[i,j]=(1.)/(sqrt(1.+((sqrt((i-(dim[1]/2))^2+(j-(dim[2]/2))^2))/cf)^(2*n)))
     endfor
     endfor
     m=min(but[where(but gt 0)])
     pos=where(finite(but) eq 0,num)
     if num gt 0 then begin
        but[pos]=m
     endif
     but=but/max(but)

     mask=shift(fft((fft(mask,-1,/double)*dim[1]*dim[2])*(fft(but,-1,/double)*dim[1]*dim[2]),1,/double)/dim[1]/dim[2],(dim[1]/2),(dim[2]/2))
     mask=mask/max(mask)
     
  end

endcase
endelse
help,mask
return,double(mask)


end
