; $Id: cnv_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    cnv_gen_default
;
; PURPOSE:
;    cnv_gen_default generates the default parameter structure for the CNV
;    module and save it in the right location.
;    (see cnv.pro's header --or file airy_help.html-- for details about the
;     module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    cnv_gen_default
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -parameters needed for image complete definition added
;                     (total efficiency, telescope collecting surface,
;                     integration time).
;                   : may 2001,
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -add possibility to handle different integration time by
;                     baseline (up to 6 baselines)
;                   : february 2003,
;                     Barbara Anconelli (DISI) [1997s019@educ.disi.unige.it].
;                    -real acquisition parameters added.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS").
;                   : june 2003,
;                     Marcel  Carbillet (OAA)  [marcel@arcetri.astro.it],
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -real acquisition parameters commented...
;                   : december 2005,
;                     Gabriele Desidera' (DISI) [desidera@disi.unige.it]:
;                    -time integration parameter activated for the output structure
;
;-
;
pro cnv_gen_default

; obtain module infos
info = cnv_info()

; generate module description structure
module = gen_def_module(info.mod_name, info.ver)

int_time = make_array(360,2,/double)
int_time[*,1]=1.

par = $
   {  $
   cnv,                 $ ; structure named cnv
   module    : module,  $ ; module description structure
   efficiency: .3,      $ ; overall efficiency (atm.+optics+detector)
   tel_surf  : 104.,    $ ; telescope collecting surface [/m^2]
;  real_acq  : 0B,      $ ; acquisition in real time?
   int_time  : int_time $ ; integration time [s] for up to 360 baselines
   }

; save the default parameters structure in the file def_file
save, par, FILENAME=info.def_file

end
