; $Id: cnv_info.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    cnv_info
;
; PURPOSE:
;    cnv_info is the routine that returns the basic information for the
;    module CNV (see cnv.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    info_structure = cnv_info()
; 
; OUTPUTS:
;    info_structure: structure containing the module's basic information.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : july 2000,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -a few minor modifications.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -useless variable "calib" eliminated.
;                    -variable info.help added (instead of !caos_env.help).
;
;-
;
function cnv_info

pack_name = 'AIRY_Software_Package_7.2'
                                 ; Software Package name
help_file = 'airy_help.html'
help      = !caos_env.modules+pack_name+!caos_env.delim+'help'+!caos_env.delim+help_file
ver       = fix(7)               ; version number of the Software Package used

mod_name = 'cnv'                 ; 3-char. module short name
descr    = 'object-PSF CoNVolution'
def_file = mk_par_name(mod_name, PACK_NAME=pack_name,/DEFAULT)
                                 ; default parameter filename

inp_type = 'img_t,src_t'         ; input data types
inp_opt  = [0B, 0B]              ; inputs may not be optional
out_type = 'img_t'               ; output data type

init     = 1B                    ; an initialisation STRUCTURE is required
time     = 0B                    ; time integration/delay management is NOT required

; convert in low case format and eliminate all the blanks
mod_name = strlowcase(strcompress(mod_name, /REMOVE_ALL))
inp_type = strlowcase(strcompress(inp_type, /REMOVE_ALL))
out_type = strlowcase(strcompress(out_type, /REMOVE_ALL))

; control the module name length
if strlen(mod_name) ne !caos_env.module_len then $
   message, 'the module name must have '+strtrim(!caos_env.module_len)+' chars'

; resulting info structure
info = $
   {   $
   pack_name: pack_name,$
   help     : help,     $
   ver      : ver,      $

   mod_name : mod_name, $
   descr    : descr,    $
   def_file : def_file, $

   inp_type : inp_type, $
   inp_opt  : inp_opt,  $  
   out_type : out_type, $

   init     : init,     $
   time     : time      $
   }

return, info
end
