; $Id: convolution.pro,v 6.1 last revision 2013/04/19 Andrea La Camera $
;+
; NAME:
;       convolution
;
; PURPOSE:
;
;	Compute the convolution of an image by a PSF using FFT product.
;
; CATEGORY:
;       ...
;
; CALLING SEQUENCE:
;
;	convolution, image_mod, psf, resu, nb_pix
;
; INPUTS:
;
;	nb_pix		= array number of pixels [INT]
;	image		= image to convolve [FLOAT] = Array[nb_pix,nb_pix]
;	psf		= PSF (must be normalised to unit energy)
;					    [FLOAT]= Array[nb_pix,nb_pix]
;
; OPTIONAL INPUTS:
;       none.
;
; KEYWORD PARAMETERS:
;
; OUTPUT:
;
;	resu = result of the convolution [FLOAT] = Array[nb_pix,nb_pix]
;
; OPTIONAL OUTPUTS:
;       none.
;
; COMMON BLOCKS:
;       none.
;
; SIDE EFFECTS:
;       none.
;
; RESTRICTIONS:
;       none.
;
; PROCEDURE:
;       none.
;
; EXAMPLE:
;       ...
;
; MODIFICATION HISTORY:
;       May 1999: written (Serge Correia, OAA).
;       March 2010: 
;       - Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;       After convolution, normalization is wrong! Therefore it is commented.
;       I changed also the casting of resu (abs replaced by float) and I added 
;       a normalization after FFT (as usual in IDL).
;-

pro convolution, image, psf, resu, nb_pix
new_img 			= dblarr(2*nb_pix, 2*nb_pix)
new_img[nb_pix/2, nb_pix/2] 	= image

new_psf 			= dblarr(2*nb_pix, 2*nb_pix)
new_psf[nb_pix/2, nb_pix/2] 	= psf


temp = float(fft(fft(new_img,-1,/double)*fft(new_psf,-1,/double),+1,/double))*4*nb_pix*nb_pix
temp  = shift(temp, nb_pix, nb_pix)

resu  = temp[nb_pix/2:3*nb_pix/2-1, nb_pix/2:3*nb_pix/2-1]
;resu  = resu*(total(psf,/double)*total(image,/double))/total(resu,/double)
return
end
