; $Id: dec_mask_gui.pro, last revision: 2013/04/19 Andrea La Camera $
;
;+
; NAME:
;    dec_mask_gui
;
; PURPOSE:
;    dec_mask_gui generates a Graphical User Interface (GUI)
;    for the choice of a mask and this interface is a sub-part 
;    of the dec one.
;
; CATEGORY:
;    GUI
;
; CALLING SEQUENCE:
;    dummy = dec_mask_gui (title,            $
;                          state    ,        $
;                          GROUP_LEADER=group)
;
; INPUTS:
;    title     : the title chosen for the sub-GUI.
;    
;
; OPTIONAL INPUTS:
;    none.
;
; KEYWORD PARAMETERS:
;    none.
;
; OUTPUTS:
;    parameters for the use of a choose mask
;
; OPTIONAL OUTPUTS:
;    none.
;
; COMMON BLOCKS:
;    none.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; PROCEDURE:
;    none.
;
; EXAMPLE:
;    ...
;
; PROGRAM MODIFICATION HISTORY:
;    program written: may 2004,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it].
;    modifications  : august 2004
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added mask visualization.
;                    -added warning about the mask dimensions.
;                   : may 2011,
;                     Marcel Carbillet (Fizeau) [marcel.carbillet@unice.fr]:
;                    -makes now use of makepupil for the mask computation,
;                    -mask display enhanced (larger, both mask AND object*mask.
;                   : April 2012,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -Sub-GUI rewritten for the version 6.0. 
;
;-
;
;;;;;;;;;;;;;;;;;;;;;;;
; mask set state
;;;;;;;;;;;;;;;;;;;;;;;

pro mask_gui_set, state

common mask_gui_block, id, par

 if (par.mask_choice EQ 0) then begin
    widget_control, id.mask_choice, SET_VALUE=0
    widget_control, id.mask_perc,   /SENSITIVE
    widget_control, id.mask_diam,   SENSITIVE=0
    widget_control, id.user_mask,  SENSITIVE=0
    widget_control, id.save_mask,  /SENSITIVE
  endif 
  if (par.mask_choice EQ 1) then begin
    widget_control, id.mask_choice, SET_VALUE=1
    widget_control, id.mask_perc,   SENSITIVE=0
    widget_control, id.mask_diam,   /SENSITIVE
    widget_control, id.user_mask,  SENSITIVE=0
    widget_control, id.save_mask,  /SENSITIVE
  endif
  if (par.mask_choice EQ 2) then begin
    widget_control, id.mask_choice,    SET_VALUE=2
    widget_control, id.mask_perc,      SENSITIVE=0
    widget_control, id.mask_diam,      SENSITIVE=0
    widget_control, id.user_mask,     /SENSITIVE
    widget_control, id.save_mask,     SENSITIVE=0
    widget_control, id.save_mask,     SET_VALUE=0
    par.save_mask=0
    widget_control, id.mask_filename, SENSITIVE=0
  endif
  if (par.save_mask EQ 0B) then begin 
    widget_control, id.save_mask,     SET_VALUE=0
    widget_control, id.mask_filename, SENSITIVE=0
  endif else begin
    widget_control, id.save_mask,     SET_VALUE=1
    widget_control, id.mask_filename, /SENSITIVE
  endelse 
 
  
end


pro display_object, state

common mask_gui_block, id, par

  xx = 256
  yy = 256

  case par.mask_choice of 
  0 : begin
    ;check initial image
    if (file_search(par.filename_image, /TEST_REG) EQ '') then begin
       a=dialog_message(["Load initial image before continuing..."])
       return
    endif
    ric=readfits(par.filename_image, /SILENT)
    ;if (n_elements(ric) NE (par.mask_npix*par.mask_npix)) then begin
    ;   a=dialog_message(["Fix the mask size", $
    ;                     "Initial image: "+strtrim((size(ric))[1],1), $
    ;                     "Mask size: "+strtrim(par.mask_npix,1)])
    ;   return
    ;endif 
    ;set mask_npix
    par.mask_npix=(size(ric))[1]
    widget_control, id.mask_npix, SET_VALUE=par.mask_npix    
    image_mask=make_array(par.mask_npix,par.mask_npix,/double,value=0.)
    soglia=(par.mask_perc*max(ric))
    min=where(ric GT soglia, count)
      if (count NE 0) then begin
        image_mask[min]=1.
      endif 
    expand, image_mask, xx, yy, masq
    expand, image_mask*ric, xx, yy, obj
    tvscl, [congrid(masq, xx, yy),congrid(obj/max(obj),xx,yy)]
  end
  1 : begin 
    ;check initial image
    if (file_search(par.filename_image, /TEST_REG) EQ '') then begin
       a=dialog_message(["Load initial image before continuing..."])
       return
    endif
    ric=readfits(par.filename_image, /SILENT)
;    if n_elements(ric) NE (par.mask_npix*par.mask_npix) then begin
;       a=dialog_message(["Load initial image before continuing or fix the mask size", $
;                         "Initial image: "+strtrim((size(ric))[1],1), $
;                         "Mask size: "+strtrim(par.mask_npix,1)])
;       return
;    endif 
    ;set mask_npix
    par.mask_npix=(size(ric))[1]
    image_mask=makepupil(par.mask_npix, par.mask_diam, 0.)
    widget_control, id.mask_npix, SET_VALUE=par.mask_npix
    expand, image_mask, xx, yy, masq
    expand, image_mask*ric, xx, yy, obj
    tvscl, [congrid(masq, xx, yy),congrid(obj/max(obj),xx,yy)]
  end
  2 : begin 
    obj = readfits(par.user_mask, h, /noscale, /silent)        
    expand, float(obj), xx, yy, obj
    tvscl, [congrid(obj, xx, yy),congrid(obj, xx, yy)]
  end
  endcase
  return
  
end


;;;;;;;;;;;;;;;;;;;;;;;
; GUI event loop
;;;;;;;;;;;;;;;;;;;;;;;

pro dec_mask_gui_event, event

common mask_gui_block, id, par

widget_control, event.id, GET_UVALUE=uvalue

case uvalue of

   'mask_npix': begin
         par.mask_npix = event.value
   end

    'mask_choice': begin
	      widget_control, id.mask_choice, GET_VALUE=dummy
	      par.mask_choice=dummy
	      mask_gui_set, state
	end

    'mask_perc': begin
		  widget_control, id.mask_perc, GET_VALUE=dummy
	      par.mask_perc=float(dummy)/100.
		  display_object, state
    end

    'mask_diam': begin
          widget_control, id.mask_diam, GET_VALUE=dummy
	      par.mask_diam=dummy
		  if (dummy le 0.) then begin 
              dummy=dialog_message("Mask diameter MUST BE greater than zero",  $
                                   DIALOG_PARENT=event.top,                    $
                                   TITLE='Diameter value error', /ERROR       )
              return                     
          endif
		  if (dummy gt long(par.mask_npix)) then begin 
		      dummy=dialog_message("Mask diameter MUST BE smaller than mask size", $
                                   DIALOG_PARENT=event.top,                                    $
                                   TITLE='Diameter value error',/ERROR        )
              return                     
          endif
		  display_object, state
	end

    'user_mask': begin
		   widget_control, id.user_mask , GET_VALUE=dummy
	       if file_search(dummy, /TEST_REGULAR) EQ '' then return
	       par.user_mask=dummy
		   display_object, state
   end

   'save_mask': begin
           widget_control, id.save_mask, GET_VALUE=dummy
           par.save_mask=dummy
           mask_gui_set, state
    end

    'mask_filename': begin
		   widget_control, id.mask_filename, GET_VALUE=dummy
		   par.mask_filename=dummy
	end

    'ok': widget_control, event.top, /DESTROY
   
endcase

end


;;;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code
;;;;;;;;;;;;;;;;;;;;;;;;;

function dec_mask_gui, title, state, GROUP_LEADER=group

common mask_gui_block, id, par

error = !caos_error.ok

par = {  $
   filename_image  : state.par.filename_image,$
   mask_choice     : state.par.mask_choice,   $
   mask_npix       : state.par.mask_npix,     $
   mask_perc       : state.par.mask_perc,     $
   mask_diam       : state.par.mask_diam,     $
   user_mask       : state.par.user_mask,     $
   save_mask       : state.par.save_mask,     $
   mask_filename   : state.par.mask_filename  $
   }
   
id = $
   { $
   mask_npix	      : 0L, $
   mask_choice	  : 0L, $
   mask_perc      : 0L, $
   mask_diam	  : 0L, $
   mask_filename  : 0L, $
   user_mask      : 0L, $
   save_mask      : 0L, $
   mask_image     : 0L  $
   }

modal = n_elements(group) ne 0
root_base_id = widget_base(TITLE=title, MODAL=modal, /COL, GROUP_LEADER=group)

base = WIDGET_BASE(root_base_id, /COLUMN, /FRAME)
text = WIDGET_LABEL(base, VALUE='SUPER-RESOLUTION MASK definition', /align_center)

id.mask_npix  = cw_field(base,TITLE='Mask dimension (in pixels) [ENTER]: ', $
                      /INTEGER, VALUE=par.mask_npix, /RETURN_EVENTS,  UVALUE='mask_npix'    )	
base1=widget_base(base, /ROW)
base1L=widget_base(base1, /COL)
text=widget_label(base1L, value='Mask type: ', /ALIGN_LEFT)
values = ['Based on initial image       ', $
          'Circular mask * initial image',$
          'User-defined mask            ']
id.mask_choice = CW_BGROUP(base1L, values, /COL, $
                          UVALUE='mask_choice', /EXCLUSIVE, /NO_RELEASE )

base1R=widget_base(base1, /COL)
id.mask_perc = widget_slider(base1R, TITLE='Percentage of initial image maximum', $
                            VALUE=round(par.mask_perc*100.)<100,  $
                            min=0, max=100, uvalue='mask_perc', xsize=50)

id.mask_diam = cw_field(base1R, TITLE='Diameter (in pixels) [ENTER]', /INTEGER,$
                        VALUE=par.mask_diam, UVALUE='mask_diam',/RETURN_EVENTS )

id.user_mask = cw_filename(base1R,TITLE='Load...',    $
                           VALUE=par.user_mask,UVALUE='user_mask',/ALL_EVENTS )

save1=widget_base(base, /col)
id.save_mask = cw_bgroup(save1,  ['Save the mask '], SET_VALUE=par.save_mask, $
                   UVALUE='save_mask', /NONEXCLUSIVE  )

id.mask_filename = cw_filename(save1, TITLE='Save as...', VALUE=par.mask_filename,	$
                                UVALUE='mask_filename', /ALL_EVENTS     )
			 

id.mask_image = widget_draw(base, xsize=512, ysize=256, /FRAME)
text=widget_label(base, value="   Mask                        "+$
                              "              Output           ", /ALIGN_CENTER)
; standard buttons base

   btn_base_id = widget_base(root_base_id, /FRAME, /ROW, /ALIGN_RIGHT)

   ok_id = widget_button(btn_base_id,                               $
                         VALUE="Save and close", $
                         UVALUE="ok"                                )

   if modal then widget_control, ok_id, /CANCEL_BUTTON

widget_control, root_base_id, /REALIZE
mask_gui_set, state
display_object, state
xmanager, 'dec_mask_gui', root_base_id, GROUP_LEADER=group

return, par

end
