; $Id: dec_info.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    dec_info
;
; PURPOSE:
;    dec_info is the routine that returns the basic information about
;    the module DEC.
;
; CATEGORY:
;    utility
;
; CALLING SEQUENCE:
;    info_structure = dec_info()
;
; OUTPUTS:
;    info_structure: structure containing the module's basic information.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: march 2000,
;                     Massimo Barbati (DISI) [].
;    modifications  : february 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -second output of type "src_t" added.
;                    -super-resolution stuff added.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -useless variable "calib" eliminated.
;                    -variable info.help added (instead of !caos_env.help).
;                   : march 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -add the possibility to choose between the OS-EM method and
;                     the LR/EM method (par.method added).
;                   : june 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it],
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -real acquisition button commented and second output of type
;                    "src_t" momentaneously eliminated.
;                    -normalization constant "c" from PAR to INIT structure.
;                    -background substracted from PSF too now.
;                   : july 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -comment about mask saving added
;                    -second type of mask and choice of mask added
;                    -add procedure dec_gui_set, state
;                   : february 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -added possibility of choice of a accelerate version for the
;                    OS-EM method
;                   : mai 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -added dec_mask_gui.pro for the choice of a mask
;                  : august 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                    -added dec_reg_gui.pro for regularization with OS-EM method.
;                    -visualization of a mask added.
;                    -menu for the choice of acceleration step added.
;                    -restructuration of interface
;                  : november 2004,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -choice of differents regularizations type added
;                  : january 2006,
;                    Barbara Anconelli (DISI) [anconelli@disi.unige.it]:
;                   -added possibility to reduce boundary effect
;                  : october 2005,
;                    Gabriele Desidera' (DISI) [desidera@disi.unige.it],
;                   -use of variable "time_integ" activated for the output structure.
;                  : April 2012,
;                    Andrea La Camera (DISI) [lacamera@disi.unige.it],
;                   -no changes for version 6.0.
;-
;
function dec_info

pack_name = 'AIRY_Software_Package_7.2'
                                 ; Software Package name
help_file = 'airy_help.html'
help      = !caos_env.modules+pack_name+!caos_env.delim+'help'+!caos_env.delim+help_file
                                 ; help-file address for Soft.Pack. AIRY
ver       = fix(7)               ; version number of the Software Package used

mod_name = 'dec'                ; Module name. Only LOWCASE string allowed
descr    = 'DEConvolution process'    ; Short module description
def_file = mk_par_name(mod_name, PACK_NAME=pack_name, /DEFAULT) ; default parameter file

init     = 1B                   ; initialization structure is needed
time     = 0B                   ; no time delay/integration management

inp_type = 'img_t,img_t'        ; input data types.
out_type = 'img_t'              ; output data types.
inp_opt  = [0B,0B]              ; inputs may not optional

; convert in low case format and eliminate all the blanks
mod_name = strlowcase(strcompress(mod_name, /REMOVE_ALL))
inp_type = strlowcase(strcompress(inp_type, /REMOVE_ALL))
out_type = strlowcase(strcompress(out_type, /REMOVE_ALL))

info = {                    $
       pack_name: pack_name,$
       help     : help,     $
       ver      : ver,      $

       mod_name : mod_name, $
       descr    : descr,    $

       def_file : def_file, $
       inp_type : inp_type, $
       out_type : out_type, $

       init     : init,     $
       time     : time      $
       }

return, info
end
