;+
; This part becomes from dec_init 
; main modifications:
; - deleted variables M, MM 
; - added optional output for objective function
;   that now is always computed (as well as the discrepancy)
;-


pro dec_init_rletal_methods, inp_img_t1, inp_img_t2, out_img_t, par, init
  
  common caos_block, tot_iter, this_iter
  
  if array_equal(size(inp_img_t1.image,/N_DIM),size(inp_img_t2.image,/N_DIM)) EQ 0 then begin
     message, 'WRONG INPUT SIZE: image(s) and PSF(s) dimensions do not match. (im = '+strtrim(size(inp_img_t2.image,/N_DIM),1)+' ; psf = '+strtrim(size(inp_img_t1.image,/N_DIM),1)+')'
  endif
  
                                ;posivitization AND equalization of input arrays
  ndim=size(inp_img_t1.image,/N_DIM)
  case ndim of 
   ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   1D  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     1: begin
                                ; "positivization" of input1   (PSF)
        image1=positivization((inp_img_t1.image-inp_img_t1.background))
        image1=image1/total(image1, /DOUBLE)
                                ;flux evaluation --> c
        c = total(inp_img_t2.image - inp_img_t2.background, /DOUBLE)
        cc = total(inp_img_t2.image, /DOUBLE)
                                ; POISSON + GAUSSIAN NOISE: Add
                                ; SIGMA^2 to image and background:
        sigma_ron = fxpar(*inp_img_t2.header, 'RON', missing=0.)
        gain_ron = fxpar(*inp_img_t2.header, 'GAIN', missing=1.)
        inp_img_t2.image=inp_img_t2.image+(sigma_ron/gain_ron)^2.
        bg=inp_img_t2.background+(sigma_ron/gain_ron)^2.
                                ;POSITIVIZATION of image   
        image2=positivization(inp_img_t2.image) ;do not subtract the background here!
                                ; "equalization" of the two arrays
;--- modif.MC 08.11.13 ---
;         NN=N
;         MM=1
;         M=1
;         P=1
;--- modif.MC 08.11.13 ---
        if par.bound then message, "BOUNDARY case is not implemented for 1D arrays"
        if par.reg GT 0 then message, "REGULARIZATIONS are not implemented for 1D arrays"
        if par.msm GT 0 then message, "MULTI-STEP METHOD is not implemented for 1D arrays"
        N = max([(size(image1))[1], (size(image2))[1]])
;--- modif.MC 08.11.13 ---
        NN=N
        P=1
;--- modif.MC 08.11.13 ---
        if (size(image1))[1] NE (size(image2))[1] then print, $
           'Zero-padding of input array(s) -> new size: '+strtrim(N,1)
        image1=zeropadding(image1, N)
        image2=zeropadding(image2, N)
        bg=zeropadding(bg,N)
                                ; FFT(PSF) is stored in image1 (overwriting original PSF)
        image1=FFT(SHIFT(image1,N/2),-1,/double)*N
                                ;other arrays (boundary compatibility)
        alpha=1.
        alpha_j=1.
        w_j=1.
        w=1.
        S=make_array(N,value=1.)
        R=make_array(N,value=1.)   
     end
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   2D  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     2: begin
                                ; "POSITIVIZATION" of input1   (PSF)
        image1=positivization((inp_img_t1.image-inp_img_t1.background))
        image1=image1/total(image1, /DOUBLE)
                                ; flux evaluation --> c
        c = total(inp_img_t2.image-inp_img_t2.background, /DOUBLE)
        cc = total(inp_img_t2.image, /double)
                                ; POISSON + GAUSSIAN NOISE: Add SIGMA^2 to image and background:
        sigma_ron = fxpar(*inp_img_t2.header, 'RON', missing=0.)
        gain_ron = fxpar(*inp_img_t2.header, 'GAIN', missing=1.)
        inp_img_t2.image=inp_img_t2.image+(sigma_ron/gain_ron)^2.
        bg=inp_img_t2.background+(sigma_ron/gain_ron)^2.
                                ;POSITIVIZATION of image
        image2=positivization(inp_img_t2.image)    ;;do not subtract the background here!
                                ; "EQUALIZATION" of the two arrays
        if (par.bound EQ 0) then begin   
           N=max([(size(image1))[1], (size(image2))[1]])
           P=1
           if ((size(image1))[1] NE (size(image2))[1]) then begin
              print, 'Zero-padding of input array(s) -> new size: '+strtrim(N,1)+'x'+strtrim(N,1)
           endif
           NN=N                 ; size of the region S
        endif else begin
           NN = (size(image2))[1]                            ; size of the region S
           if (size(image1))[1] GT (size(image2))[1] then begin
              N = (size(image1))[1]   ; the size of the PSF is the working size
           endif else begin
              delta=dec_psf_ee(image1, par.ee_threshold) ;width of the PSF
              N=round(max((size(image2))[1]+2*delta))                ; the size depends on the EE of the PSF
              ;N = N > 2^(ceil(alog(N)/alog(2))) ;set N to a power of two (if necessary)
              P=1 
           endelse
           print,'Zero-padding of input array(s) -> new size: '+strtrim(N,1)+'x'+strtrim(N,1)
        endelse
                                ;equalize the two arrays + background
        image1=zeropadding(image1, N,N)
        image2=zeropadding(image2, N,N)
        bg = zeropadding(bg, N, N)
                                ; FFT(PSF) is stored in image1 (overwriting original PSF)
        image1=FFT(SHIFT(image1,N/2,N/2),-1,/double)*N*N
                                ;boundary effects arrays
        alpha=1.
        alpha_j=1.
        w_j=1.
        w=1./alpha
        S=make_array(NN,NN,value=1.)
        R=make_array(N,N,value=1.)
        if (par.bound EQ 1) then begin
           S=zeropadding(S,N,N)
           alpha_j=float(FFT(CONJ(image1)*FFT(S,-1,/DOUBLE),+1,/DOUBLE))
           alpha = alpha_j
           w_j = 1./alpha_j
           w = 1./alpha
           sigma_BEC=0.
           dec_find_sigma_opt,alpha, sigma_BEC, 0., 0.5, 1e-5
           pos=where(alpha LE sigma_BEC, count)
           if count GT 0 then begin
              R[pos]=0. 
              w_j[pos]=0.
              w[pos]=0.
           endif    
        endif
     end
      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   3D  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     3: begin
        P= (size(inp_img_t1.image))[3]
        P2=(size(inp_img_t2.image))[3]
        if P NE P2 then message, $
           'WRONG INPUT SIZE: images and PSFs do not match in number.'
                                ; "positivization" of input1   (PSF)
        image1=inp_img_t1.image
        for j=0, P-1 do begin
           temp=positivization((inp_img_t1.image[*,*,j]-inp_img_t1.background[*,*,j]))
           temp=temp/total(temp,/DOUBLE)
           image1[*,*,j]=temp
        endfor 
                                ;"positivization" of input2 (IMAGE) and flux evaluation --> c
        image2=inp_img_t2.image
        bg=inp_img_t2.background
        c=0.
        cc=0.
        sigma_ron = fxpar(*inp_img_t2.header, 'RON', missing=0.)
        gain_ron = fxpar(*inp_img_t2.header, 'GAIN', missing=1.)        
        for j=0,P-1 do begin
           c=c+total((inp_img_t2.image[*,*,j] - inp_img_t2.background[*,*,j]), /DOUBLE)
           cc=cc+total(inp_img_t2.image[*,*,j], /DOUBLE)
                                ; POISSON + GAUSSIAN NOISE: Add SIGMA^2 to image and background:
           inp_img_t2.image[*,*,j]=inp_img_t2.image[*,*,j]+(sigma_ron/gain_ron)^2.
           bg[*,*,j]=inp_img_t2.background[*,*,j]+(sigma_ron/gain_ron)^2.
                                ;POSITIVIZATION of image
           temp=positivization(inp_img_t2.image[*,*,j]) ;do not subtract the background here!
           image2[*,*,j]=temp
        endfor
        c=c/float(P)
                                ; "EQUALIZATION" of the two arrays                
        if par.bound EQ 0 then begin
           N=max([(size(image1))[1], (size(image2))[1]])
           if ((size(image1))[1] NE (size(image2))[1]) then begin
              print, 'Zero-padding of input array(s) -> new size: '+strtrim(N,1)+'x'+strtrim(N,1)+'x'+strtrim(P,1)
           endif
           NN=N                 ; size of the region S  
        endif else begin
           NN = (size(image2))[1] ; size of the region S
           if (size(image1))[1] GT (size(image2))[1] then begin
              N = (size(image1))[1]   ; the size of the PSF is the working size
           endif else begin
              delta=fltarr(P)
              for j=0,P-1 do begin
                 delta[j]=dec_psf_ee(image1[*,*,j], par.ee_threshold) ;width of the PSF[j]
              endfor
              N=round(max((size(image2))[1]+2*delta))  ;the size depends on the EE of the PSF
              ;N = N > 2^(ceil(alog(N)/alog(2))) ;set N to a power of two (if necessary)
           endelse
           print, 'Zero-padding of input array(s) -> new size: '+strtrim(N,1)+'x'+strtrim(N,1)+'x'+strtrim(P,1)
        endelse
        image1=zeropadding(image1, N,N)  ;zeropadding is able to manage 3d arrays
        image2=zeropadding(image2, N,N)  ;zeropadding is able to manage 3d arrays
        bg = zeropadding(bg,N,N) ; zeropadding of the background
                                ; FFT(PSF) is stored in image1 (overwriting original PSF)
        dummy=image1
        image1=make_array(N,N,P,/DCOMPLEX)
        for j=0,P-1 do image1[0,0,j]=FFT(SHIFT(dummy[*,*,j],N/2,N/2),-1,/double)*N*N
                                ;boundary effects arrays
        alpha=float(P)
        alpha_j=1.
        w_j=1.
        w=1./alpha
        S=make_array(NN,NN,value=1.)
        R=make_array(N,N,value=1.)
        if (par.bound EQ 1) then begin
           S=zeropadding(S,N,N)
           alpha_j=make_array(N,N,P)
           w_j=make_array(N,N,P)
           for j=0,P-1 do begin
              alpha_j[*,*,j]=float(FFT(CONJ(image1[*,*,j])*FFT(S,-1,/DOUBLE),+1,/DOUBLE))
              w_j[*,*,j] = 1./alpha_j[*,*,j]
           endfor
           alpha = total(alpha_j,3, /DOUBLE)
           w = 1./alpha
           sigma_BEC=0.
           dec_find_sigma_opt, alpha, sigma_BEC, 0., 0.5, 1e-5
           pos=where(alpha LE sigma_BEC, count)
           if count GT 0 then begin
              R[pos]=0. 
              for j=0,P-1 do begin
                 dummy = w_j[*,*,j]
                 dummy[pos]=0.
                 w_j[*,*,j]=dummy
              endfor
              w[pos]=0.
           endif    
        endif
     end
     else: message, 'WRONG INPUT TYPE: only 1D/2D/3D arrays are supported.'
  endcase
  
                                ;ISRA and OS-ISRA modifications
  atg=make_array(N,N,P)        ;numerator of the generic iteration of ISRA: atg = sum_j (ATj gj)
  atg_sum=0.
  if (par.method GE 2) then begin
     for j=0,P-1 do begin
        atg[*,*,j]=float(FFT(CONJ(image1[*,*,j])*FFT(image2[*,*,j],-1,/double),+1,/double))
        atg_sum=atg_sum+atg[*,*,j]
     endfor
                                ; R region re-defined
     R=make_array(N,N, value=1.)
     cc=cc/(NN*NN)
     pos=where(atg_sum LE 0.01*cc, count)
     if count GT 0 then R[pos]=0.
  endif
  
                                ;initial image (constant  or user-defined)
  if (par.init_alg) then begin
     fn=readfits(par.filename_image)
     if (N_ELEMENTS(fn) NE (long(N)*long(N))) then begin
        print, "The array does not fit the image size ("+strtrim(N,1)+" x "+strtrim(N,1)+")"
        print, "A zero-padding will be applied..."
        fn = zeropadding(fn,N,N)
     endif    
  endif else begin
     fn = make_array(N,N, value=c/total(R*alpha, /DOUBLE)) ;GENERAL CASE
  endelse
                                ;mask for super-resolution
  if (par.init_mask) then begin
     image_mask=1.              ;initialize the mask
     case par.mask_choice of
        0: begin
                                ;check initial image
           if (file_search(par.filename_image, /TEST_REG) EQ '') then message, $
              "User-defined image is not a valid FITS file"
           ref=readfits(par.filename_image, /SILENT)
           thr=(par.mask_perc*max(ref))
           index=where(ref GT thr, count)
           image_mask=make_array(N,N,/double,value=0.)
           if (count NE 0) then begin
              image_mask[index]=1.
           endif 
        end
        1: begin
           image_mask=makepupil(N,par.mask_diam, 0.)
        end
        2: begin
           if (file_search(par.user_mask, /TEST_REG) EQ '') then message, $
              "User-defined mask is not a valid FITS file"
           image_mask=readfits(par.user_mask, /SILENT)
           if par.bound then image_mask=zeropadding(image_mask,N,N)
        end
        else:  message, "Super-resolution mask not defined!"
     endcase
     fn=image_mask*fn
                                ;save the mask       
     if (par.save_mask EQ 1B) then begin
        fits_write, par.mask_filename, image_mask
     endif
  endif
   
;                                ;U and V functions (REGULARIZATIONS) 
;  U=make_array(N,N)
;  V=make_array(N,N)
                                ;apodization matrix (for Edge
                                ;Preserving and Laplacian regularizations)
;  apo=S
;  convolution, apo, gauss_ell(N/2, N/2, 6, 6, N), apo, N
;  apo=apo/max(apo)
  
                                ;ACCELERATION ARRAYS --> 
  acc_arrays=0.
  if par.acc GT 0 then begin
     acc_arrays=make_array(N,N,3) ;0=f(k-1); 1=e(k-1); 2=e(k-2)
     acc_arrays[*,*,2]=fn         ;the initialization of the algorithm
  endif   
  lambda=0.

                                ;multi-step method: add the diffuse component to background
  if par.msm then begin
     if file_test(par.msm_file, /READ, /REGULAR) then begin
        x_E=readfits(par.msm_file)
        if par.bound then x_E=zeropadding(x_E, N,N) ; equalize the input image 
     endif else message, "ERROR while loading the file "+par.msm_load
     bg = bg + afunction(x_E,image1)
     stop
  endif
  
           

                                ;working index: where R > 0, how many pixels, complementary index, 
  posR=where(R GT 0., nposR, complement=c_posR, ncomplement=nc_posR)
  
                                ;working index_ where S > 0, how many pixels
  posS=where(S GT 0., nposS)
    
                                ;initialize the discrepancy array
                                ;[alway computed]
  discrepancy = make_array(tot_iter+1, /double)
  discrepancy[0]=!values.D_NAN
                                ;initialize the objective function array
                                ;[alway computed]
  obj_fun = make_array(tot_iter+1, /double)
  obj_fun[0]=!values.D_NAN

                                ; restoration error
  object = 0.
  rho=0.
  if (par.save_rho) OR (par.stopping_rule EQ 4) then begin
     if file_test(par.load_obj_file, /READ, /REGULAR) then begin
        object=readfits(par.load_obj_file)
     endif else message, "ERROR while loading the file "+par.load_obj_file
     rho = make_array(tot_iter+1, /double)
     rho[0]=!values.D_NAN
  endif
                                ;open the files for optional outputs
                                ;and load the true object (stopping rule #4)
  unit_d=0
  unit_r=0
  unit_o=0
  if par.save_discrepancy then openw, unit_d, par.save_discrepancy_file, /GET_LUN
  if par.save_obj_fun then openw, unit_o, par.save_obj_fun_file, /GET_LUN
  if par.save_rho then openw, unit_r, par.save_rho_file, /GET_LUN
  
  
                                ; INIT  structure initialization (in
                                ; the case of RL, OSEM, ISRA, and OSISRA)
  init =                       $
     {                         $
     image1: image1,    $      ; FFT(psf)
     image2: image2,    $      ; input image(s)
     fn: fn, $                 ; initialization of the algorithm 
     prev: make_array(NN,NN),$ ; previous iterate (for Stop.Rule#2)
     N: N, $                   ; working dim
     P: P, $                   ; number of images (if cube)
     NN: NN, $                 ; output dim
     bg: bg, $                 ; background
     S: S, $                   ; v----------- 
     R: R, $                   ;
     alpha: alpha, $           ; for BOUNDARY 
     alpha_j: alpha_j, $       ;    case
     w_j: w_j, $               ;
     w: w, $                   ; ^-----------
     atg: atg, $               ; A^T g (for ISRA)
     atg_sum: atg_sum, $ 
     acc_arrays: acc_arrays, $  ; ACCELERATIONS PARAMETERS
     lambda: lambda, $          ;     "            " 
     posR: posR, $              ;four arrays related to R 
     nposR: nposR, $
     c_posR: c_posR, $
     nc_posR: nc_posR, $
     posS: posS, $              ;two arrays related to S
     nposS: nposS, $
     object: object,          $  ; the true object 
     unit_r: unit_r,     $  ; save rho
     unit_d: unit_d,     $  ; save discrepancy
     unit_o: unit_o,     $  ; save objective function
     obj_fun: obj_fun,    $  ; | three arrays (with tot_iter+1 elements) used 
     discrepancy: discrepancy,$  ; | in optional output, rho is defined only in #4, 
     rho: rho,                 $  ; | while obj_fun and discrepancy always are.
     c: c   $                    ; normalization constant
     }
   
end
