pro dec_init_sgp_method, inp_img_t1, inp_img_t2, out_img_t, par, init

common caos_block, tot_iter, this_iter

  
ndim=size(inp_img_t1.image,/N_DIM)
if ((par.bound) AND (ndim EQ 1)) then message, "BOUNDARY case is not implemented for 1D arrays"
if ((par.reg GT 0) AND (ndim EQ 1)) then message, "REGULARIZATIONS are not implemented for 1D arrays"
if ((par.msm GT 0) AND (ndim EQ 1)) then message, "MULTI-STEP METHOD is not implemented for 1D arrays"
if ((size(inp_img_t1.image))[0] LE 2) then P=1 else P=(size(inp_img_t1.image))[3]
if ((size(inp_img_t2.image))[0] LE 2) then P2=1 else P2=(size(inp_img_t2.image))[3]
if P NE P2 then message, 'WRONG INPUT SIZE: images and PSFs do not match in number.'
psf=inp_img_t1.image
gn=inp_img_t2.image
bg=inp_img_t2.background

sigma_ron = fxpar(*inp_img_t2.header, 'RON', missing=0.)
gain_ron = fxpar(*inp_img_t2.header, 'GAIN', missing=1.)

if ndim EQ 1 then begin
                                ; "positivization" of input1   (PSF)
   psf=positivization((inp_img_t1.image-inp_img_t1.background))
   psf=psf/total(psf, /DOUBLE)
                                ;flux evaluation --> c
   c = total(inp_img_t2.image - inp_img_t2.background, /DOUBLE)
   cc = total(inp_img_t2.image, /DOUBLE)
                                ; POISSON + GAUSSIAN NOISE: Add
                                ; SIGMA^2 to image and background:
   gn=inp_img_t2.image+(sigma_ron/gain_ron)^2.
   bg=inp_img_t2.background+(sigma_ron/gain_ron)^2.
                                ;POSITIVIZATION of image   
   gn=positivization(gn)        ;do not subtract the background here!
endif else begin
   for j=0, P-1 do begin
                                ; "positivization" of input1   (PSF)
      temp=positivization((inp_img_t1.image[*,*,j]-inp_img_t1.background[*,*,j]))
      temp=temp/total(temp,/DOUBLE)
      psf[*,*,j]=temp
                                ;"positivization" of input2 (IMAGE) and 
                                ; POISSON + GAUSSIAN NOISE: Add SIGMA^2 to image and background
      gn[*,*,j]=inp_img_t2.image[*,*,j]+(sigma_ron/gain_ron)^2.
      bg[*,*,j]=inp_img_t2.background[*,*,j]+(sigma_ron/gain_ron)^2.
                                ;POSITIVIZATION of image
      gn[*,*,j]=positivization(gn[*,*,j]) ;do not subtract the background here!
   endfor
endelse
                                ;multi-step method: add the diffuse component to background
if par.msm then begin
   if file_test(par.msm_file, /READ, /REGULAR) then begin
      x_E=readfits(par.msm_file)
      dim = size(psf,/dimensions)
      if (size(psf,/n_dimensions) eq 2) then dim = [dim,1]
      TF = dcomplexarr(dim)
      for i = 0, dim[2]-1 do begin
         TF[*,*,i] = fft(shift(reform(psf[*,*,i]),dim[0:1]/2))*dim[0]*dim[1]
      endfor
      bg = bg + afunction(x_E,TF)
   endif else message, "ERROR while loading the file "+par.msm_file
endif
;;;;;;;;


; SGP default parameters
alpha_min = 1.e-5               ; alpha lower bound
alpha_max = 1.e5                ; alpha upper bound
theta = 0.4                     ; backtracking parameter
beta = 1.e-4                    ; for sufficient decrease
initalpha = 1.3                 ; initial alpha
M = 1                           ; memory in obj function value (if M = 1 monotone)
Malpha = 3                      ; alfaBB1 memory
tau = 0.5                       ; alternating parameter
initflag = 2                    ; 2 -> constant image
errflag = 0B                    ; 0 -> no error calculation
err = -1

; data scaling
scaling = max(gn)
gn = gn/scaling
bg = bg/scaling
par.delta = par.delta/scaling

; initializations and computations that need only once
dim = size(gn,/dimensions)
if (size(gn,/n_dimensions) eq 1) then begin
   dim = [dim,1,1]
   obj_size = dim[0]
endif
if (size(gn,/n_dimensions) eq 2) then dim = [dim,1]
if (size(gn,/n_dimensions) GE 2) then obj_size = dim[0:1]
flux = total(total(gn-bg,1,/double),1,/double) ; exact flux -- it's an array!
;  iter = 1                                       ; iteration counter
alpha = initalpha                                 ; initial alpha
Valpha = alpha_max * (dblarr(Malpha)+1.)          ; memory buffer for alpha
Fold = -1.e30 * (dblarr(M)+1.)                    ; memory buffer for obj. func.
Discr_coeff = 2./(dim[0]*dim[1]*dim[2])*scaling   ; discrepancy coefficient


; modified ALC [2016/05/09]
; WARNING: at this level, if psf and gn differ in dimensions (i.e. size(gn) < 
; size(psf)), then both newdim and work_size are set to be equal to size(psf)
; Otherwise the two arrays MUST be equal each other and they are here enlarged 
; to dim_psf+2*delta_max (defined by Enclosed Energy of the PSF).
delta=fltarr(P)                 ; no bound --> delta = 0
if (par.bound) then begin
   for j=0,P-1 do delta[j]=dec_psf_ee(psf[*,*,j], par.ee_threshold) ;width of the PSF[j]
   delta_max = max(delta)
   dim_psf = size(psf,/dimensions)
   if n_elements(dim_psf) eq 2 then dim_psf = [dim_psf,1]
   if array_equal(dim_psf,dim) then begin
      newdim = [dim_psf[0]+2*delta_max, dim_psf[1]+2*delta_max]
      work_size = newdim
      margine = (newdim - dim[0:1])/2
      temp = dblarr(newdim[0],newdim[1],dim[2])
      temp[margine[0]:margine[0]+dim[0]-1,margine[1]:margine[1]+dim[1]-1,*] = psf
      if (par.verb) then print, "BOUNDARY -> work size = ", strtrim(work_size,1)
   endif else begin
      newdim = [dim_psf[0],dim_psf[1]]
      work_size = newdim
      margine = (newdim - dim[0:1])/2
      temp = psf
      if (par.verb) then print, "BOUNDARY -> work size = ", strtrim(work_size,1)
   end
                                ; computing TF and CTF
   TF = dcomplexarr(newdim[0],newdim[1],dim[2])
   CTF = TF
   for i = 0, dim[2]-1 do begin
      TF[*,*,i] = fft(shift(reform(temp[*,*,i]),newdim/2))*newdim[0]*newdim[1]
      CTF[*,*,i] =conj(TF[*,*,i])
   endfor
                                ; defining mask for boundary conditions
   Mask = dblarr(work_size)
   Mask[margine[0]:margine[0]+dim[0]-1,margine[1]:margine[1]+dim[1]-1] = 1.
   obj_index = long(where(Mask gt 0.))
   Weight = Afunction(Mask,CTF)
   sigma_BEC=0.
   dec_find_sigma_opt, Weight[*,*,0], sigma_BEC, 0., 0.5, 1e-5
   regionR_index = where(Weight[*,*,0] gt sigma_BEC)
   if dim[2] gt 1 then begin
      temp = intarr(newdim[0],newdim[1])
      temp[regionR_index] = 1
      for i = 1, dim[2]-1 do begin
         regionR_index_temp = where(Weight[*,*,i] gt sigma_BEC)
         temp2 = intarr(newdim[0],newdim[1])
         temp2[regionR_index_temp] = 1
         temp = temp and temp2
      endfor
      regionR_index = where(temp eq 1)
      Weight = total(Weight,3,/double)
   endif
   ATM = dblarr(work_size)
   ATM[regionR_index] = Weight[regionR_index]
   temp = dblarr(work_size)
   temp[regionR_index] = 1./Weight[regionR_index]
   Weight = temp
   temp = dblarr([work_size,dim[2]])
   for i = 0, dim[2]-1 do begin
      temp[*,*,i] = Mask
   endfor
   obj_index_multi = where(temp gt 0)
                                ; resizing gn
   temp = dblarr(work_size[0],work_size[1],dim[2])
   temp[obj_index_multi] = gn[*]
   gn = temp
                                ; resizing bg
   temp = make_array(work_size[0],work_size[1],dim[2], /DOUBLE, value=median(bg))
   temp[obj_index_multi] = bg[*]
   bg = temp
   
endif else begin
   if (ndim*P EQ 1) then begin  ;1D CASE 
      TF = fft(shift(psf,dim[0]/2))*dim[0]
      CTF = conj(TF)
   endif else begin             ;2D and 3D cases
      TF = dcomplexarr(dim)
      CTF = TF
      for j = 0, P-1 do begin
         TF[*,*,j] = fft(shift(reform(psf[*,*,j]),dim[0:1]/2))*dim[0]*dim[1]
         CTF[*,*,j] = conj(TF[*,*,j])
      endfor
   endelse
   work_size = obj_size         ; in ALL cases these quantities are well defined!
   obj_index = lindgen(product(work_size))
   obj_index_multi = lindgen(dim[2]*product(work_size))
   regionR_index = obj_index
   ATM = make_array(work_size, value = dim[2], /double)
   Weight = make_array(work_size, value = 1./dim[2], /double)
endelse
n_R = n_elements(regionR_index)
obj_index = obj_index[*]
obj_index_multi = obj_index_multi[*]
regionR_index = regionR_index[*]

                                ; Read the optional parameters ;

if file_test(par.load_obj_file, /READ, /REGULAR) then begin
   errflag = 1B
endif else begin
   if (par.save_rho) then $
      message, "ERROR while loading the file "+par.load_obj_file $
   else errflag = 0B
endelse

                                ; starting point ;
                                ;initial image (constant  or
                                ;user-defined)
c = mean(flux,/double)
if (par.init_alg) then begin
   x = readfits(par.filename_image)
   if (n_elements(x) ne 0) then begin
      x = x/scaling
      if not(array_equal(size(x,/dimensions),dim[0:1])) then begin ; x is explicitly given, check dimensions
         message, 'Invalid size of the initial point x.'
      endif
   endif 
endif else begin
   if (par.bound) then begin
      x0 = c*dim[2]/total(ATM[regionR_index],/double)
      x = dblarr(work_size)
      x[regionR_index] = x0
   endif else begin
      x = c/product(work_size)*make_array(work_size,value=1.,/double)
   endelse
endelse

                                ; change the null pixels of the observed image ;
sgp_par = machar(/double)
vmin = min(gn(where(gn gt 0)))
gn_nonpos = where(gn le 0, n_nonpos)
if (n_nonpos gt 0) then gn(gn_nonpos) = vmin*sgp_par.eps^2

                                ; vector allocation ;
rho=0.
obj=0.
obj_sum=0.
if errflag then begin
   obj = readfits(par.load_obj_file)
   obj = double(obj)/scaling
   obj_sum = total(obj*obj)
   rho = make_array(tot_iter+1, /double)
   rho[0]=!values.D_NAN
endif
                                ;initialize the discrepancy array
                                ;[alway computed]
discrepancy = make_array(tot_iter+1, /double)
                                ;initialize the objective function array
                                ;[alway computed]
obj_fun = make_array(tot_iter+1, /double)
                                ;open the files for optional outputs
                                ;and load the true object (stopping rule #4)
unit_d=0
unit_o=0
unit_r=0
if par.save_discrepancy then openw, unit_d, par.save_discrepancy_file, /GET_LUN
if par.save_obj_fun then openw, unit_o, par.save_obj_fun_file, /GET_LUN
if par.save_rho then openw, unit_r, par.save_rho_file, /GET_LUN

;;;;;;;;;;;;;;;;
; start of SGP ;
;;;;;;;;;;;;;;;;
                                ; projection of the initial point
if par.flux_con then begin
   x[regionR_index] = project_fletcher_nonneg(reform(ATM[regionR_index]), c*dim[2], reform(x[regionR_index]), make_array(n_R, value = 1., /double))
endif else begin
   temp =  where(x lt 0, n_nonpos)
   if (n_nonpos gt 0) then x[temp] = 0.
endelse

                                ;mask for super-resolution (projection
                                ;of the initial point on Supp(x)
                                ;mask for super-resolution
image_mask=1.                   ;initialize the mask
if (par.init_mask) then begin
   case par.mask_choice of
      0: begin
                                ;check initial image
         if (file_search(par.filename_image, /TEST_REG) EQ '') then message, $
            "User-defined image is not a valid FITS file"
         ref=readfits(par.filename_image, /SILENT)
         thr=(par.mask_perc*max(ref))
         index=where(ref GT thr, count)
         image_mask=make_array(work_size[0],work_size[1],/double,value=0.)
         if (count NE 0) then begin
            image_mask[index]=1.
         endif 
      end
      1: begin
         image_mask=makepupil(work_size[0],par.mask_diam, 0.)
      end
      2: begin
         if (file_search(par.user_mask, /TEST_REG) EQ '') then message, $
            "User-defined mask is not a valid FITS file"
         image_mask=readfits(par.user_mask, /SILENT)
         if par.bound then image_mask=zeropadding(image_mask,work_size[0],work_size[1])
      end
      else:   message, "Super-resolution mask not defined!" 
   endcase
   x=image_mask*x
                                ;save the mask       
   if (par.save_mask EQ 1B) then begin
      fits_write, par.mask_filename, image_mask
   endif
endif

                                ; error
if errflag then begin
   e = x[obj_index] - obj[*]
   rho[0] = sqrt(total(e*e)/obj_sum)
endif
                                ; objective function value
x_tf = afunction(x,TF)
den = x_tf + bg
temp = gn/den
g = afunction(temp,CTF)
                                ;REGULARIZATION PART --> new!
if par.reg gt 0 then begin
   f_R = reg_f(x, par)
   grad_dec = reg_grad_dec(x, par)
endif else begin
   grad_dec = {U_R:0., V_R:0.}
   f_R = 0.
endelse

if P gt 1 then begin
   g = total(g,3)
endif
g = ATM - g + par.beta*(grad_dec.V_R - grad_dec.U_R)
                                ; discrepancy
if (P*dim[1] EQ 1) then begin
   f_0 = total(gn*alog(temp)) + total(x_tf) - flux
endif else begin
   f_0 = total(gn[obj_index_multi]*alog(temp[obj_index_multi]),/double) + total(x_tf[obj_index_multi],/double) - total(flux,/double)
endelse
fv = f_0 + par.beta*f_R
discrepancy[0] = Discr_coeff * f_0
obj_fun[0] = scaling* fv        ;????????? CHECK!

                                ; bounds for the scaling matrices
temp = ATM + par.beta*grad_dec.V_R
Weight_R = dblarr(work_size)
Weight_R[regionR_index] = 1./temp[regionR_index]
y = x - Weight_R*x*g
y_min = min(y[where(y gt 0)])
y_max = max(y)
X_low_bound = y_min		; Lower bound for the scaling matrix
X_upp_bound = y_max             ; Upper bound for the scaling matrix
if X_upp_bound/X_low_bound lt 50. then begin
   X_low_bound = X_low_bound/10.
   X_upp_bound = X_upp_bound*10.
endif

                                ; scaling matrix
XX = Weight_R*x
XX_toolow = where(XX lt X_low_bound, nlow)
if (nlow gt 0) then XX[XX_toolow] = X_low_bound
XX_toohigh = where(XX gt X_upp_bound, nhigh)
if (nhigh gt 0) then XX[XX_toohigh] = X_upp_bound
if (par.bound) then begin
   temp = dblarr(work_size)
   temp[regionR_index] = XX[regionR_index]
   XX = temp
endif
D = dblarr(work_size)
D[regionR_index] = 1./XX[regionR_index]


                                ; init structure 
init = {                          $
       x : x,                     $    ; initial point x 
       x_tf : x_tf,               $    ; Ax matrix
       gn : gn,                   $     ; gn matrix
       bg : bg,                   $    ; background
       XX : XX,                   $    ; scaling matrix XX
       D : D,                     $    ; matrix D
       X_low_bound : X_low_bound, $    ; bounds
       X_upp_bound : X_upp_bound, $    ; bounds
       Weight : Weight,           $    ; weights of boundary
       regionR_index : regionR_index, $ ; index of boundary   ***CHECK!!! regionR_index!!!!
       g : g,                     $     ; g matrix
       Valpha : Valpha,           $     ; memory buffer for alpha
       Malpha : Malpha,           $     ; alfaBB1 memory
       M : M,                     $     ; memory in obj. function value (if M=1 monotone)
       Fold : Fold,               $     ; memory buffer for obj. func.
       fv : fv,                   $     ; obj function / discrepancy
       scaling : scaling,         $     ; scaling 
       Discr_coeff : Discr_coeff, $     ; discrepancy coefficient 
       alpha : alpha,             $     ; initial alpha
       alpha_max : alpha_max,     $     ; initial alpha_max
       alpha_min : alpha_min,     $     ; initial alpha_min
       beta : beta,               $     ; initial beta (do not confuse it with REG PARAM)
       theta : theta,             $     ; initial theta
       tau : tau,                 $     ; initial tau
       TF : TF,                   $     ; TF array
       CTF : CTF,                 $     ; CTF array
       ATM : ATM,                 $     ; ATM array
       dim : dim,                 $     ; dim size
       image_mask : image_mask,   $     ; image mask for super-resolution
       obj_index : obj_index,     $         ; obj_index     
       obj_index_multi : obj_index_multi,     $ ; obj_index_multi     
       work_size : work_size,     $         ; work size 
       obj_size : obj_size,       $         ; obj size
       P : P ,                    $         ; number of images (if cube)
       NN : obj_size[0],          $         ; output dimension NNxNN (region S)
       flux : flux,               $         ; flux of image - background [array]
       c : c,                     $         ; flux constant
       obj : obj,                 $         ; the true object 
       errflag : errflag,         $         ; compute restoration error flag
       obj_sum : obj_sum,         $         ; ( L2 norm of obj )^2
       unit_r       : unit_r,     $         ; save rho
       unit_d       : unit_d,     $         ; save discrepancy
       unit_o       : unit_o,     $         ; save objective function
       discrepancy : discrepancy, $         ; discrepancy array 
       obj_fun : obj_fun,         $         ; objective function array
       rho : rho                  $         ; restoration error array
       }


end
