;last update: 2016/05/05; Andrea La Camera
;+
;deconvolve.pro 
;PURPOSE:
;   DECONVOLVE computes the generic iteration of the [RLM|OSEM|ISRA|OS-ISRA] 
;   algorithm. If a stopping rule is selected, DECONVOLVE computes also the
;   necessary arrays (discrepancy and/or functional)
;         
;
;INPUT: 
;   fn - the reconstructed image at k-th iteration
;   init - the initial structure; 
;   par  - the parameter structure; 
;   this_iter - this iteration number        
;
;OUTPUT: 
;   out = the reconstructed image at (k+1)-th iteration             
;-

function jzero, x,y 
   return, total(y*alog(y/x)+x-y, /double)
end

pro deconvolve, fn, init, par, out, this_iter

U=0
V=0
J_R=0. ; regularization functional 
if (par.reg GT 0) then begin   ;refresh U and V with current fn
   dummy = reg_grad_dec(fn, par)
   U=dummy.U_R
   V=dummy.V_R
   J_R = reg_f(fn,par)
endif   

case par.method of
   0: begin                     ;RL
      fnt=FFT(fn,-1,/double)
      sum=0.
      J_0=0.
      for j=0, init.P-1 do begin
         dummy = float(FFT(init.image1[*,*,j]*fnt,+1,/DOUBLE))+init.bg[*,*,j]
         g=init.image2[*,*,j]
         J_0=J_0+jzero(dummy[init.posS],g[init.posS])
         dummy = init.image2[*,*,j]/dummy
         sum = sum + float(FFT(CONJ(init.image1[*,*,j])*FFT(dummy,-1,/double),+1,/double))
      endfor
      fn = init.w*fn*(sum+par.beta*U)/(1.+par.beta*init.w*V)
      init.discrepancy[this_iter]=J_0/(0.5*init.P*init.nposS)
      init.obj_fun[this_iter]=J_0+par.beta*J_R
   end                          
   
   1: begin                     ;OSEM
      J_0=0.
      for j=0,init.P-1 do begin
         dummy = float(FFT(init.image1[*,*,j]*FFT(fn,-1,/double),+1,/double))+init.bg[*,*,j]
         g=init.image2[*,*,j]
         J_0=J_0+jzero(dummy[init.posS],g[init.posS])
         dummy = init.image2[*,*,j]/dummy
         dummy = float(FFT(CONJ(init.image1[*,*,j])*FFT(dummy,-1,/double),+1,/double))
         dummy = dummy + (par.beta/init.P*U)
         if (par.bound EQ 1) then $
            fn=init.w_j[*,*,j]*fn/(1.+par.beta/init.P*init.w_j[*,*,j]*V)*dummy $
         else $
            fn=fn/(1.+par.beta/init.P*V)*dummy
      endfor
      init.discrepancy[this_iter]=J_0/(0.5*init.P*init.nposS)
      init.obj_fun[this_iter]=J_0+par.beta*J_R
   end                          
   
   
   2: begin                     ;ISRA
      if (par.bound EQ 0) then begin 
         fnt=FFT(fn,-1,/double)
         sum=0.
         for j=0,init.P-1 do begin
            sum=sum+float(FFT(init.image1[*,*,j]*CONJ(init.image1[*,*,j])*fnt,+1,/double))+init.bg[*,*,j]
         endfor
         fn =fn*(init.atg_sum + par.beta*U)/(sum + par.beta*V)
      endif else begin
         fnt=FFT(fn,-1,/double)
         sum=0.
         for j=0,init.P-1 do begin
            dummy=init.S*float(FFT(init.image1[*,*,j]*fnt,+1,/double))+init.bg[*,*,j]
            sum=sum+float(FFT(CONJ(init.image1[*,*,j])*FFT(dummy,-1,/double),+1,/double))
         endfor
         fn =fn*init.R*(init.atg_sum + par.beta*U)/(sum + par.beta*V)
      endelse
   end                          
   
   
   3: begin                     ;OS-ISRA
      if (par.bound EQ 0) then begin
         for j=0,init.P-1 do begin
            fnt=FFT(fn,-1,/double)
            dummy=float(FFT(init.image1[*,*,j]*CONJ(init.image1[*,*,j])*fnt,+1,/double))+init.bg[*,*,j]
            fn = fn*(init.atg[*,*,j]+ par.beta/init.P*U)/(dummy + par.beta/init.P*V)
         endfor
      endif else begin
         for j=0,init.P-1 do begin
            fnt=FFT(fn,-1,/double)
            dummy=init.S*float(FFT(init.image1[*,*,j]*fnt,+1,/double))+init.bg[*,*,j]
            dummy=float(FFT(CONJ(init.image1[*,*,j])*FFT(dummy,-1,/double),+1,/double))
            fn = fn*init.R*(init.atg[*,*,j]+par.beta/init.P*U)/(dummy+par.beta/init.P*V)
         endfor
      endelse
   end                          
endcase
out=fn
end
