; $Id: fsm_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    fsm_gen_default
;
; PURPOSE:
;    fsm_gen_default generates the default parameter structure
;    for the FSM module and save it in the right location.
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    fsm_gen_default
; 
; ROUTINE MODIFICATION HISTORY:
;    routine written: october 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS").
;
;-
;
pro fsm_gen_default

; obtain module infos
info = fsm_info()

; generate module description structure.
module = gen_def_module(info.mod_name, info.ver)

par =   $
   {    $
   fsm, $                                    ; structure named fsm
   module         : module, $                ; module description structure
   tresh_detect   : 15.,    $                ; treshold of the detection [rms background value]
   cflux_dir      : 1B,     $                ; Direct calibration of the flux (1=yes, 0=no)
   cal_flux       : 100.  , $                ;      Calibrated flux [counts]
   cal_mag        : 10.  ,  $                ;      Calibrated magnitude in the considered band 
   band_tab       : ['R','I','J','H','K'], $ ; observation band 
   band_index     : 0L                   , $ ; observation band index
   efficiency     : .3,     $                ; overall efficiency (atm.+optics+detector)
   tel_surf       : 104.,   $                ; telescope collecting surface [/m^2]
   int_time       : 1.,     $                ; integration time [s]
   aper_phot      : [3,5,7] $                ; Aperture photometry set [pixel]
}

; save the default parameter structure in the default file
save, par, FILENAME=info.def_file

end
