; $Id: fsm_init.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+ 
; NAME: 
;    fsm_init 
; 
; PURPOSE: 
;    fsm_init executes the initialization for the Find Star Module (FSM).
; 
; CATEGORY: 
;    module's initialization routine 
; 
; CALLING SEQUENCE: 
;    error = fsm_init(inp_img_t, $ ; img_t input structure
;                     par,       $ ; parameters structure
;                     INIT=init  ) ; initialization structure
; 
; INPUTS/OUTPUTS/KEYWORDS/ETC.: 
;    see fsm.pro's header for a detailed description. 
; 
; ROUTINE MODIFICATION HISTORY: 
;    routine written: october 2000,
;                     Serge  Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : june 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -common variables calibration and signature eliminated.
;
;- 
; 
function fsm_init, inp_img_t, $
                   par,       $
                   INIT=init 

; CAOS global common block
common caos_block, tot_iter, this_iter

; initialization of the error code: no error as default
error = !caos_error.ok 

; retrieve the module's informations
info = fsm_info()

; get the individual output structure types
if info.out_type ne '' then out_type = str_sep(info.out_type,",")

; STANDARD CHECKS
;
; compute and test the requested number of fsm arguments
n_par = 1  ; the parameter structure is always present within the arguments
if info.inp_type ne '' then begin
   inp_type = str_sep(info.inp_type,",")
   n_inp    = n_elements(inp_type)
endif else n_inp = 0
if info.out_type ne '' then begin
   out_type = str_sep(info.out_type,",")
   n_out    = n_elements(out_type)
endif else n_out = 0
n_par = n_par + n_inp + n_out
if n_params() ne n_par then message, 'wrong number of arguments'

; test the parameter structure
if test_type(par, /STRUCTURE, N_ELEMENTS=n) then $
   message, 'FSM error: par must be a structure'
if n ne 1 then message, 'FSM error: par cannot be a vector of structures'
if strlowcase(tag_names(par, /STRUCTURE_NAME)) ne info.mod_name then $
   message, 'par must be a parameter structure for the module FSM'

; check the input arguments

; test if any optional input exists
if n_inp gt 0 then begin
   inp_opt = info.inp_opt
endif

dummy = test_type(inp_img_t, TYPE=type)
if type eq 0 then begin         ; undefined variable
   inp_img_t = $
      {        $
      data_type  : inp_type[0],         $
      data_status: !caos_data.not_valid $
      }
endif
if test_type(inp_img_t, /STRUC, N_EL=n, TYPE=type) then $
   message, 'FSM error: wrong definition for the first input.'
if n ne 1 then message, $
   'FSM error: first input cannot be a vector of structures'

; test the data type
if inp_type[0] ne 'gen_t' then begin
   if inp_img_t.data_type ne inp_type[0] then                $
      message, 'wrong input data type: '+inp_img_t.data_type $
              +' ('+inp_type[0]+' expected).'
endif

;-------------------MODIFICA PER GESTIRE ERRORE IN CASO MULTI-FRAME
pos=where(inp_img_t.time_integ[*,1] gt 1,num)
if ((size(inp_img_t.image))[0] eq 4) then begin
   if (num eq 0) then begin 
      inp_img_t.image=reform(inp_img_t.image,(size(inp_img_t.image))[1],(size(inp_img_t.image))[2],(size(inp_img_t.image))[3]) 
   endif else begin
      dummy=dialog_message('WRONG INPUT TYPE (MULTI-FRAME input is not supported)',/ERROR)
      message,'WRONG INPUT TYPE (MULTI-FRAME input is not supported)'
   endelse
endif
;------------------FINE MODIFICA ERRORE PER GESTIRE CASO MULTI-FRAME


if inp_img_t.data_status eq !caos_data.not_valid and not inp_opt[0] then $
      message, 'undefined input is not allowed'

; STRUCTURE "INIT" DEFINITION
;


   init = $
      {   $
      counter : 0L,                                          $
      data    : fltarr(inp_img_t.npixel, inp_img_t.npixel)   $
      }

; back to calling program
return, error 
end
