; $Id: mcd_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    mcd_gen_default
;
; PURPOSE:
;    mcd_gen_default generates the default parameter structure for the MCD
;    module and save it in the rigth location.
;    (see mcd.pro's header --or file airy_help.html-- for details about the
;     module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    mcd_gen_default
; 
; ROUTINE MODIFICATION HISTORY:
;    routine written: may 2016
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it]
;    modifications  : date,
;                     author (institute) [email@address]:
;                    -description of modification.
;-
;
pro mcd_gen_default

; obtain module infos
info = mcd_info()

; generate module description structure
module = gen_def_module(info.mod_name, info.ver)

; parameter structure
par = $
   {  $
   mcd,                          $ ; structure named com
   module       : module,         $ ; module description structure
   method    : 0,      $        ; deconvolution method (0=MC-RL, 1=MC-SGP)
                                ; default is SGP
   reg	     : 0,      $        ; regularizations [0=none, 1=TI0, 2=TI2, 3=ENT, 4=MRF, 
                                ;                  5=HDR,  6=MIS, 7=HYP, 8=TI1]
   beta        : 0.d,   $        ; regularization parameter 
   delta       : 0.d,   $        ; threshold for the regularization
   flux_con    : 0B,   $        ;     
   ee_threshold: 0.95, $        ; threshold of the PSF's Enclosed Energy 
   bound     : 0B,     $        ; reduce boundary effects [0=no, 1=yes]
   init_P    : 0B,     $        ; initialization of algorithm [0=default array, 
                                ;                              1=user-defined image]
   init_E    : 0B,     $        ; initialization of algorithm [0=default array, 
                                ;                              1=user-defined image]
   init_P_filename  : '', $          ; initial image file address
   init_E_filename  : '', $          ; initial image file address
   mask : '',   $               ; mask M_P file name 
   stopping_rule : 0, $         ;stopping criteria: 0=iter>TOT_ITER; 1=Discrepancy crosses 1; 
                                ; 2=total functional is approx. constant; 3=min of restoration error.
   stop_tol : 1.d0, $           ;tolerance for the given stopping rule 
   load_obj_file : '' , $       ; object map for the stopping rule nb 3 
   save_discrepancy: 0, $       ; save the discrepancy values
   save_obj_fun:0,      $       ; save the objective function values
   save_discrepancy_file : '', $ ; filename (ASCII file)
   save_obj_fun_file : '',     $ ; filename (ASCII file) 
   save_rho : 0, $               ;save the restoration error values
   save_rho_file : '', $         ;filename of restoration error (ASCII file)
   verb : 0B $                   ; verbosity of the algorithm(s)
   }   

; save the default parameters structure in the file def_file
save, par, FILENAME=info.def_file

;back to calling program
end
