; $Id: mcd_info.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    mcd_info
;
; PURPOSE:
;    mcd_info is the routine that returns the basic informations for the
;    module MCD (see mcd.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    info_structure = mcd_info()
; 
; OUTPUTS:
;    info_structure: structure containing the module's basic informations.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: may 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it].
;    modifications  : date,
;                     author (institute) [email@address]:
;                    -description of modification.
;-
;
function mcd_info

pack_name = 'AIRY_Software_Package_7.2'
                                 ; Software Package name
help_file = 'airy_help.html'
help      = !caos_env.modules+pack_name+!caos_env.delim+'help'+!caos_env.delim+help_file
                                 ; help-file address for Soft.Pack. AIRY
ver       = fix(7)               ; version number of the Software Package used

mod_name = 'mcd'                ; Module name. Only LOWCASE string allowed
descr    = 'Multi-Component Deconvolution'    ; Short module description
def_file = mk_par_name(mod_name, PACK_NAME=pack_name, /DEFAULT) ; default parameter file

init     = 1B                   ; initialization structure is needed
time     = 0B                   ; no time delay/integration management

inp_type = 'img_t,img_t'        ; input data types.
out_type = 'img_t,img_t'              ; output data types.
inp_opt  = [0B,0B]              ; inputs may not optional

; convert in low case format and eliminate all the blanks
mod_name = strlowcase(strcompress(mod_name, /REMOVE_ALL))
inp_type = strlowcase(strcompress(inp_type, /REMOVE_ALL))
out_type = strlowcase(strcompress(out_type, /REMOVE_ALL))

; control the module name length
if strlen(mod_name) ne !caos_env.module_len then $
   message, 'the module name must have '+strtrim(!caos_env.module_len)+' chars'

info = {                    $
       pack_name: pack_name,$
       help     : help,     $
       ver      : ver,      $

       mod_name : mod_name, $
       descr    : descr,    $

       def_file : def_file, $
       inp_type : inp_type, $
       out_type : out_type, $

       init     : init,     $
       time     : time      $
       }

return, info
end

