pro mcd_init_mcsgp, inp_img_t1, inp_img_t2, out_img_tE, out_img_tP, par, init
common caos_block, tot_iter, this_iter
  
  ndim=size(inp_img_t1.image,/N_DIM)
  if ((size(inp_img_t1.image))[0] LE 2) then P=1 else P=(size(inp_img_t1.image))[3]
  if ((size(inp_img_t2.image))[0] LE 2) then P2=1 else P2=(size(inp_img_t2.image))[3]
  if P NE P2 then message, 'WRONG INPUT SIZE: images and PSFs do not match in number.'
  psf=inp_img_t1.image
  gn=inp_img_t2.image
  bg=inp_img_t2.background

  sigma_ron = fxpar(*inp_img_t2.header, 'RON', missing=0.)
  gain_ron = fxpar(*inp_img_t2.header, 'GAIN', missing=1.)
  
  for j=0, P-1 do begin
                                ; "positivization" of input1   (PSF)
     temp=positivization((inp_img_t1.image[*,*,j]-inp_img_t1.background[*,*,j]))
     temp=temp/total(temp,/DOUBLE)
     psf[*,*,j]=temp
                                ;"positivization" of input2 (IMAGE) and 
                                ; POISSON + GAUSSIAN NOISE: Add SIGMA^2 to image and background
     gn[*,*,j]=inp_img_t2.image[*,*,j]+(sigma_ron/gain_ron)^2.
     bg[*,*,j]=inp_img_t2.background[*,*,j]+(sigma_ron/gain_ron)^2.
                                ;POSITIVIZATION of image
     gn[*,*,j]=positivization(gn[*,*,j]) ;do not subtract the background here!
  endfor

  dim = size(gn,/dimensions)
  if (ndim eq 2) then dim = [dim,1]
                                ; READ THE MASK --- MANDATORY!
                                ;check initial image
  if (file_search(par.mask, /TEST_REG) EQ '') then message, "Mask image is not a valid FITS file"
  mask_po=readfits(par.mask)
  if (N_ELEMENTS(mask_po) NE (long(dim[0])*long(dim[1]))) then begin
     print, "The array does not fit the image size ("+strtrim(dim[0],1)+" x "+strtrim(dim[1],1)+")"
     print, "A zero-padding will be applied..."
     mask_po = zeropadding(mask_po,dim[0],dim[1])
  endif    


; SGP default parameters
  alpha_min = 1.e-5             ; alpha lower bound
  alpha_max = 1.e5              ; alpha upper bound
  theta = 0.4                   ; backtracking parameter
  beta = 1.e-4                  ; for sufficient decrease
  initalpha = 1.3               ; initial alpha
  M = 1                         ; memory in obj function value (if M = 1 monotone)
  Malpha = 3                    ; alfaBB1 memory
  tau = 0.5                     ; alternating parameter
  initflag = 2                  ; 2 -> constant image
  errflag = 0B                  ; 0 -> no error calculation
  err = -1

; data scaling
  scaling = max(gn)
  gn = gn/scaling
  bg = bg/scaling
  par.delta = par.delta/scaling
  
; initializations and computations that need only once
  obj_size = dim[0:1]
  flux = total(total(gn-bg,1,/double),1,/double) ; exact flux -- it's an array!
;  iter = 1                                       ; iteration counter
  alpha = initalpha                              ; initial alpha
  Valpha = alpha_max * (dblarr(Malpha)+1.)       ; memory buffer for alpha
  Fold = -1.e30 * (dblarr(M)+1.)                 ; memory buffer for obj. func.
  Discr_coeff = 2./(dim[0]*dim[1]*dim[2])*scaling ; discrepancy coefficient
  
  
; modified ALC [2016/05/09]
; WARNING: at this level, if psf and gn differ in dimensions (i.e. size(gn) < 
; size(psf)), then both newdim and work_size are set to be equal to size(psf)
; Otherwise the two arrays MUST be equal each other and they are here enlarged 
; to dim_psf+2*delta_max (defined by Enclosed Energy of the PSF).
  delta=fltarr(P)               ; no bound --> delta = 0
  if (par.bound) then begin
     for j=0,P-1 do delta[j]=dec_psf_ee(psf[*,*,j], par.ee_threshold) ;width of the PSF[j]
     delta_max = max(delta)
     dim_psf = size(psf,/dimensions)
     if n_elements(dim_psf) eq 2 then dim_psf = [dim_psf,1]
     if array_equal(dim_psf,dim) then begin
        newdim = [dim_psf[0]+2*delta_max, dim_psf[1]+2*delta_max]
        work_size = newdim
        margine = (newdim - dim[0:1])/2
        temp = dblarr(newdim[0],newdim[1],dim[2])
        temp[margine[0]:margine[0]+dim[0]-1,margine[1]:margine[1]+dim[1]-1,*] = psf
        if (par.verb) then print, "BOUNDARY -> work size = ", strtrim(work_size,1)
     endif else begin
        newdim = [dim_psf[0],dim_psf[1]]
        work_size = newdim
        margine = (newdim - dim[0:1])/2
        temp = psf
        if (par.verb) then print, "BOUNDARY -> work size = ", strtrim(work_size,1)
     end
                                ; computing TF and CTF
     TF = dcomplexarr(newdim[0],newdim[1],dim[2])
     CTF = TF
     for i = 0, dim[2]-1 do begin
        TF[*,*,i] = fft(shift(reform(temp[*,*,i]),newdim/2))*newdim[0]*newdim[1]
        CTF[*,*,i] =conj(TF[*,*,i])
     endfor
     
                                ; resizing mask with zero-padding
     temp = dblarr(newdim[0],newdim[1])
     temp[margine[0]:margine[0]+dim[0]-1,margine[1]:margine[1]+dim[1]-1] = mask_po
     mask_po = temp
     
                                ; defining mask for boundary conditions
     Mask = dblarr(work_size)
     Mask[margine[0]:margine[0]+dim[0]-1,margine[1]:margine[1]+dim[1]-1] = 1.
     obj_index = long(where(Mask gt 0.))
     weight = Afunction(Mask,CTF)
     sigma_BEC=0.
     dec_find_sigma_opt, weight[*,*,0], sigma_BEC, 0., 0.5, 1e-5
     regionR_index = where(weight[*,*,0] gt sigma_BEC)
     if dim[2] gt 1 then begin
        temp = intarr(newdim[0],newdim[1])
        temp[regionR_index] = 1
        for i = 1, dim[2]-1 do begin
           regionR_index_temp = where(weight[*,*,i] gt sigma)
           temp2 = intarr(newdim[0],newdim[1])
           temp2[regionR_index_temp] = 1
           temp = temp and temp2
        endfor
        regionR_index = where(temp eq 1)
        weight = total(weight,3,/double)
     endif
     ATM = dblarr(work_size)
     ATM[regionR_index] = weight[regionR_index]
     temp = dblarr(work_size)
     temp[regionR_index] = 1./weight[regionR_index]
     weight = temp
     temp = dblarr([work_size,dim[2]])
     for i = 0, dim[2]-1 do begin
        temp[*,*,i] = Mask
     endfor
     obj_index_multi = where(temp gt 0)
                                ; resizing gn
     temp = dblarr(work_size[0],work_size[1],dim[2])
     temp[obj_index_multi] = gn[*]
     gn = temp
                                ; resizing bg
     temp = make_array(work_size[0],work_size[1],dim[2], /DOUBLE, value=median(bg))
     temp[obj_index_multi] = bg[*]
     bg = temp

  endif else begin
     TF = dcomplexarr(dim)
     CTF = TF
     for j = 0, P-1 do begin
        TF[*,*,j] = fft(shift(reform(psf[*,*,j]),dim[0:1]/2))*dim[0]*dim[1]
        CTF[*,*,j] = conj(TF[*,*,j])
     endfor
     work_size = obj_size       ; in ALL cases these quantities are well defined!
     obj_index = lindgen(product(work_size))
     obj_index_multi = lindgen(dim[2]*product(work_size))
     regionR_index = obj_index
     ATM = make_array(work_size, value = dim[2], /double)
     weight = make_array(work_size, value = 1./dim[2], /double)
  endelse
  n_R = n_elements(regionR_index)
  obj_index = obj_index[*]
  obj_index_multi = obj_index_multi[*]
  regionR_index = regionR_index[*]
  
                                ; Read the optional parameters ;

  if file_test(par.load_obj_file, /READ, /REGULAR) then begin
     errflag = 1B
  endif else begin
     if (par.save_rho) then $
        message, "ERROR while loading the file "+par.load_obj_file $
     else errflag = 0B
  endelse
  ind_po = where(mask_po eq 1, n_po)
  if (n_po eq 0) then message, 'Please insert a valid mask.'
  c = mean(flux,/double)
                                ; starting points 
  if (par.init_P) then x_p=readfits(par.init_P_filename)
  if (par.init_E) then x_d=readfits(par.init_E_filename)
  if (n_elements(x_d) ne 0) and (n_elements(x_p) ne 0) then begin
   x_d = x_d/scaling
   x_p = x_p[ind_po]/scaling ; x_p now is a vector (1D array)
   if not(array_equal(size(x_d,/dimensions),dim[0:1])) then begin ; x_d is explicitly given, check dimensions
      message, 'Invalid size of the initial point x_d.'
   endif else begin
      if n_elements(x_p) ne n_po then begin
         message, 'Mask and Initial image (P) must have the same non-zero elements.'
      endif
   endelse
endif else begin
   temp = gn[*,*,0]
   x_p = temp[ind_po] - bg[ind_po] ; initial value for x_p
   if par.bound then begin
      x0 = (c - total(ATM[ind_po]*x_p,/double)/dim[2])*dim[2]/total(ATM[regionR_index],/double)
      x_d = dblarr(work_size)
      x_d[regionR_index] = x0   ; initial value of x_d
   endif else begin
      x_d = (c - total(x_p,/double))/product(work_size)*make_array(work_size,value=1.,/double)
   endelse
endelse
                                ; change the null pixels of the observed image ;
  sgp_par = machar(/double)
  vmin = min(gn(where(gn gt 0)))
  gn_nonpos = where(gn le 0, n_nonpos)
  if (n_nonpos gt 0) then gn(gn_nonpos) = vmin*sgp_par.eps^2

                                ; vector allocation ;
   rho=0.
   obj=0.
   obj_sum=0.
   if errflag then begin
      obj = readfits(par.load_obj_file)
      obj = double(obj)/scaling
      obj_sum = total(obj*obj)
      rho = make_array(tot_iter+1, /double)
      rho[0]=!values.D_NAN
   endif
                                ;initialize the discrepancy array
                                ;[alway computed]
   discrepancy = make_array(tot_iter+1, /double)
                                ;initialize the objective function array
                                ;[alway computed]
  obj_fun = make_array(tot_iter+1, /double)
                                ;open the files for optional outputs
                                ;and load the true object (stopping rule #4)
   unit_d=0
   unit_o=0
   unit_r=0
   if par.save_discrepancy then openw, unit_d, par.save_discrepancy_file, /GET_LUN
   if par.save_obj_fun then openw, unit_o, par.save_obj_fun_file, /GET_LUN
   if par.save_rho then openw, unit_r, par.save_rho_file, /GET_LUN
   
;;;;;;;;;;;;;;;;
; start of SGP ;
;;;;;;;;;;;;;;;;
   
                                ; projection of the initial point
   if par.flux_con then begin
      temp = [x_d[regionR_index],x_p]
      temp = project_fletcher_nonneg(reform([ATM[regionR_index],ATM[ind_po]]), c*dim[2], reform(temp), make_array(n_elements(temp), value = 1., /double))
      x_d[regionR_index] = temp[0:n_R-1]
      x_p = temp[n_R:n_R+n_po-1]
   endif else begin
      temp =  where(x_d lt 0, n_nonpos)
      if (n_nonpos gt 0) then x_d[temp] = 0.
      temp =  where(x_p lt 0, n_nonpos)
      if (n_nonpos gt 0) then x_p[temp] = 0.
   endelse
   
                                ; compute image x
   x = x_d
   x[ind_po] = x[ind_po] + x_p
                                ; error
   if errflag then begin
      e = x_d[obj_index] - obj[*]
      err[0] = sqrt(total(e*e,/double)/obj_sum)
   endif
                                ; objective function value
   x_tf = afunction(x,TF)
   temp = gn/(x_tf + bg)
   g = afunction(temp,CTF)
   ;REGULARIZATION PART --> new!
   if par.reg gt 0 then begin
      f_R = reg_f(x_d, par)
      grad_dec = reg_grad_dec(x_d, par)
   endif else begin
      grad_dec = {U_R:0., V_R:0.}
      f_R = 0.
   endelse
   
   if dim[2] gt 1 then begin
      g = total(g,3)
   endif
   g_d = ATM - g + par.beta*(grad_dec.V_R - grad_dec.U_R)
   g_p = ATM[ind_po] - g[ind_po]
   f_0 = total(gn[obj_index_multi]*alog(temp[obj_index_multi]),/double) + total(x_tf[obj_index_multi],/double) - total(flux,/double)
   fv = f_0 + par.beta*f_R
   discrepancy[0] = Discr_coeff * f_0
   obj_fun[0] = scaling* fv 

                                ; bounds for the scaling matrices
   temp = ATM + par.beta*grad_dec.V_R
   weight_R = dblarr(work_size)
   weight_R[regionR_index] = 1./temp[regionR_index]
   y = x_d - weight_R*x_d*g_d
   y_min = min(y[where(y gt 0)])
   y_max = max(y)
   X_low_bound_d = y_min        ; Lower bound for the scaling matrix - diffuse part
   X_upp_bound_d = y_max    	; Upper bound for the scaling matrix - diffuse part
   if X_upp_bound_d/X_low_bound_d lt 50. then begin
      X_low_bound_d = X_low_bound_d/10.
      X_upp_bound_d = X_upp_bound_d*10.
   endif
   y = x_p - weight[ind_po]*x_p*g_p
   y_min = min(y[where(y gt 0)])
   y_max = max(y)
   X_low_bound_p = y_min        ; Lower bound for the scaling matrix - pointwise part
   X_upp_bound_p = y_max    	; Upper bound for the scaling matrix - pointwise part
   if X_upp_bound_p/X_low_bound_p lt 50. then begin
      X_low_bound_p = X_low_bound_p/10.
      X_upp_bound_p = X_upp_bound_p*10.
   endif
   
                                ; scaling matrix - diffuse part
   XX_d = weight_R*x_d
   XX_toolow = where(XX_d lt X_low_bound_d, nlow)
   if (nlow gt 0) then XX_d[XX_toolow] = X_low_bound_d
   XX_toohigh = where(XX_d gt X_upp_bound_d, nhigh)
   if (nhigh gt 0) then XX_d[XX_toohigh] = X_upp_bound_d
   if par.bound then begin
      temp = dblarr(work_size)
      temp[regionR_index] = XX_d[regionR_index]
      XX_d = temp
   endif
   D_d = dblarr(work_size)
   D_d[regionR_index] = 1./XX_d[regionR_index]
                                ; scaling matrix - pointwise part
   XX_p = weight[ind_po]*x_p
   XX_toolow = where(XX_p lt X_low_bound_p, nlow)
   if (nlow gt 0) then XX_p[XX_toolow] = X_low_bound_p
   XX_toohigh = where(XX_p gt X_upp_bound_p, nhigh)
   if (nhigh gt 0) then XX_p[XX_toohigh] = X_upp_bound_p
   D_p = 1./XX_p
   
                                ; init structure 
   init = {                          $
          gn: gn,                   $     ; blurred image
          bg: bg,                   $     ; background
          x_d: x_d,                 $     ; initial point x_d
          x_p: x_p,                 $     ; initial point x_p
          ind_po: ind_po, $               ; index positions of bright sources
          n_po: n_po, $                   ; number of bright sources
          x_tf: x_tf,               $     ; Ax matrix
          XX_p: XX_p,                   $ ; scaling matrix XX_p
          XX_d: XX_d,                   $ ; scaling matrix XX_d
          D_d: D_d,                     $ ; matrix D_d
          D_p: D_p,                     $ ; matrix D_p
          X_low_bound_p: X_low_bound_p, $     ; bounds
          X_upp_bound_p: X_upp_bound_p, $     ; bounds
          X_low_bound_d: X_low_bound_d, $     ; bounds
          X_upp_bound_d: X_upp_bound_d, $     ; bounds
          regionR_index: regionR_index, $ ; index of boundary
          weight: weight, $               ; weight matrix for x_p
          n_R: n_R, $                     ; number of pixels in R
          g_d: g_d,                     $ ; g_d matrix
          g_p: g_p,                     $ ; g_p matrix
          Valpha: Valpha,           $     ; memory buffer for alpha
          Malpha: Malpha,           $     ; alfaBB1 memory
          M: M,                     $ ; memory in obj. function value (if M=1 monotone)
          Fold: Fold,               $ ; memory buffer for obj. func.
          fv: fv,                   $ ; obj function / discrepancy
          scaling: scaling,         $ ; scaling 
          Discr_coeff: Discr_coeff, $ ; discrepancy coefficient 
          alpha: alpha,             $ ; initial alpha
          alpha_max: alpha_max,     $ ; initial alpha_max
          alpha_min: alpha_min,     $ ; initial alpha_min
          beta: beta,               $ ; initial beta (do not confuse it with REG PARAM)
          theta: theta,             $ ; initial theta
          tau: tau,                 $ ; initial tau
          TF: TF,                   $ ; TF array
          CTF: CTF,                 $ ; CTF array
          ATM: ATM,                 $ ; ATM array
          dim: dim,                 $ ; dim size
          obj_index: obj_index,     $ ; obj_index     
          obj_index_multi: obj_index_multi, $ ; obj_index_multi  
          work_size: work_size,     $         ; work size 
          obj_size: obj_size,       $         ; obj size
          P: P ,                    $         ; number of images (if cube)
          NN: obj_size[0],          $         ; output dimension NNxNN (region S)
          flux: flux,               $         ; flux of image - background [array]
          c: c,                     $         ; flux constant
          obj: obj,                 $         ; the true object 
          errflag: errflag,         $         ; compute restoration error flag
          obj_sum: obj_sum,         $         ; ( L2 norm of obj )^2
          unit_r: unit_r,     $         ; save rho
          unit_d: unit_d,     $         ; save discrepancy
          unit_o: unit_o,     $         ; save objective function
          discrepancy: discrepancy, $         ; discrepancy array 
          obj_fun: obj_fun,         $         ; objective function array
          rho: rho                  $         ; restoration error array
          }
   
   
end
