; $Id: obj_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    obj_gen_default
;
; PURPOSE:
;    obj_gen_default generates the default parameter structure
;    for the OBJ module and save it in the right location.
;    (see obj.pro's header --or file airy_help.html-- for details
;     about the module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;       obj_gen_default
;
; ROUTINE MODIFICATION HISTORY:
;    program written: may 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : september 2000, 
;                     Serge Correia (OAA) [correia@arcetri.astro.it]:
;                    -a few modifications made.
;                   : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS").
;                    -all paths updated.
;                   : june 2003,
;                     Marcel  Carbillet (OAA)  [marcel@arcetri.astro.it],
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it].
;                    -sky background control added.
;                   : november 2003,
;                     Serge Correia (AIP) [scorreia@aip.de]:
;                    -added the object type "single star".
;                   : january 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -Flux constants are now "float" instead of "int".
;
;-
;
pro obj_gen_default

info   = obj_info()
module = gen_def_module(info.mod_name, info.ver)


; parameter structure
; -------------------------------------------------
; === initialization default variables ============

  type_obj_tab      = ['Single Star', 'Binary Object', 'Open Cluster', 'Planetary Nebula', 'SN Remnant',                   $
                       'Spiral Galaxy', 'YSO', 'Stellar Surface', 'User-defined'];, 'AGNs','AGB stars',         $
                       ;'Be stars','TNOs - KBOs']
;  obj_base          = ['single_base', 'binary_base', 'cluster_base', 'nebulae_base', 'SN_remnant_base', 'galaxy_base', 'YS0_base', $
;                       'star_surf_base', 'user_defined_base']

  lambda_tab        = 1e-6 * [ 0.70, 0.90, 1.25, 1.65, 2.20 ]


  type_obj_nb = (size(type_obj_tab))[1]

;---------------------------- Single star parameter -------------------------

  single_star_mag   = 20.

;---------------------------- Binary stars parameters -------------------------

  bin_separation    = 12e-3
  bin_pa            = 36.
  bin_mag           = 20.
  bin_delt_m        = 2.

;---------------------------- Random generated open cluster parameters --------

  clust_nb_star     = 100
  clust_mag_max     = 14.
  clust_mag_min     = 8.
  clust_seed        = 1

;---------------------------- Planetary Nebula parameters --------------------

  ; save_spec structure
  pl_emmi_lines_data = filepath("pl_neb_emmi_lines_4102_7325.sav", $
                 ROOT=!caos_env.modules,    $
                 SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim $
                 +"obj"+!caos_env.delim+"obj_data"])

  restore, filename = pl_emmi_lines_data  

  name_obj= ['NGC 7027']
  obj_file= filepath("NGC7027_HST_1998.fits", ROOT=!caos_env.modules,              $
            SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim $
            +"obj"+!caos_env.delim+"obj_data"]) 		   
			   
  extension     = [0.3]
  em_lines_text = ['!17H!7d!X'  , '!17H!7c!X'  , 'OIII'  , 'HeI'        , 'HeI'  , $
                   'HeII'       , 'HeII'       , 'ArIV'  , '!17H!7b!X'  , 'HeI'  , $
                   '[OIII]'     , '[OIII]'     , 'NI'    , 'HeII'       , 'ClIII', $
                   'ClIII'      , 'NII'        , 'HeI'   , 'OI'         , 'SIII' , $
                   'OI'         , 'ArV'        , '[NII]' , '!17H!7a!X'  , '[NII]', $
                   'HeI'        , 'SII'        , 'SII'   , 'ArV'        , 'HeI'  , $
                   'ArIII'      , '[OII]' ]

user_def_spectrum = { user_def_spectrum,                            $
                      name             : em_lines_text,             $
                      lambda           : save_spec.lambda,          $
                      flux             : save_spec.spectrum_flux,   $
                      status           : ''                         }

; image caption #1 (general characteristics and object type)
  caption1 = ['', $
              ''  $                                                           
            ]                                                           
; image caption #2 (data acquisition characteristics)
  caption2 = ['', $ 
              ''  $                                                           
             ]                                                           

; image caption #3 (paper reference)
  caption3 = ['', $ 
              ''  $                                                           
             ]                                                           

; image credit 
  credit = ['', $    
            ''  $              
           ]  

  neb   = {  neb_st,                                     $
             name_obj          : name_obj,               $
             obj_file          : obj_file,               $
             extension         : extension,              $
             spectrum          : save_spec,              $ 
             user_def_spectrum : user_def_spectrum,      $
             em_lines_text     : em_lines_text,          $
             flux              : 100.,                   $
             caption1          : caption1,               $
             caption2          : caption2,               $
             caption3          : caption3,               $
             credit            : credit                  }

;---------------------------- SuperNovae Remnant parameters --------------------

  ; save_spec structure
  pl_emmi_lines_data = filepath("pl_neb_emmi_lines_4102_7325.sav", ROOT=!caos_env.modules, $
                                SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"])
  restore, filename = pl_emmi_lines_data  

  name_obj      = ['Crab Nebula (M1)']
  obj_file      = filepath("M1_esoVLT.fits",ROOT=!caos_env.modules, $
                           SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"]) 
  extension     = [0.3]
  em_lines_text = ['!17H!7d!X'  , '!17H!7c!X'  , 'OIII'  , 'HeI'        , 'HeI'  , $
                   'HeII'       , 'HeII'       , 'ArIV'  , '!17H!7b!X'  , 'HeI'  , $
                   '[OIII]'     , '[OIII]'     , 'NI'    , 'HeII'       , 'ClIII', $
                   'ClIII'      , 'NII'        , 'HeI'   , 'OI'         , 'SIII' , $
                   'OI'         , 'ArV'        , '[NII]' , '!17H!7a!X'  , '[NII]', $
                   'HeI'        , 'SII'        , 'SII'   , 'ArV'        , 'HeI'  , $
                   'ArIII'      , '[OII]' ]

user_def_spectrum = { user_def_spectrum,                            $
                      name             : em_lines_text,             $
                      lambda           : save_spec.lambda,          $
                      flux             : save_spec.spectrum_flux,   $
                      status           : ''                         }

; image caption #1 (general characteristics and object type)
  caption1 = ['', $
              ''  $                                                           
            ]                                                           
; image caption #2 (data acquisition characteristics)
  caption2 = ['', $ 
              ''  $                                                           
             ]                                                           

; image caption #3 (paper reference)
  caption3 = ['', $ 
              ''  $                                                           
             ]                                                           

; image credit 
  credit = ['', $    
            ''  $              
           ]                                                           

  SNR   = {  SNR_st,                                     $
             name_obj          : name_obj,               $
             obj_file          : obj_file,               $
             extension         : extension,              $
             spectrum          : save_spec,              $ 
             user_def_spectrum : user_def_spectrum,      $
             em_lines_text     : em_lines_text,          $
             flux              : 100.,                   $
             caption1          : caption1,               $
             caption2          : caption2,               $
             caption3          : caption3,               $
             credit            : credit                  }

  

;---------------------------- spiral galaxy parameters ---------------------------------

  name_obj      = ['NGC 1288',              $              ; Objects names
                   'M 100'                  ]

                                                           ; .fits map filename

  obj_file      = [filepath("NGC1288_eso_skycat.fits",ROOT=!caos_env.modules, $
                            SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"]), $
                   filepath("M100_HST.fits",ROOT=!caos_env.modules, $
                            SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"])]


  tot_mag       = [0., 0.]; Original object total magnitude

  bright        = [22.36, 22.15]; Original brightness [mag/arcsec2] 

  extension     = [0.3, 0.3]; Original extension [arcsec]

  wave_range_choice     = [0, 0]; Wavelength range considered
                                ; 0 : Broad band standard filter
                                ; 1 : Custom wavelength range

  bright_mag_choice     = [1, 1]; Choice beetween luminosity express in
                                ; 0 : totale magnitude 
                                ; 1 : average brightness  
                                ; (for broad band only)

  flux                  = [500., 500.]; Flux in photons/s/m2 emitted by the object
                                      ; (for custom wavelength range only)

; image caption #1 (general characteristics and object type)
  caption1 = ['', $
              ''  $                                                           
            ]                                                           
; image caption #2 (data acquisition characteristics)
  caption2 = ['', $ 
              ''  $                                                           
             ]                                                           

; image caption #3 (paper reference)
  caption3 = ['', $ 
              ''  $                                                           
             ]                                                           

; image credit 
  credit = ['', $    
            ''  $              
           ]                                                           

  
  gal   = {  gal_st,                                $
             name_obj          : name_obj,          $
             obj_file          : obj_file,          $
             tot_mag           : tot_mag,           $
             bright            : bright,            $
             extension         : extension,         $
             wave_range_choice : wave_range_choice, $
             bright_mag_choice : bright_mag_choice, $
             flux              : flux,              $
             caption1          : caption1,          $
             caption2          : caption2,          $
             caption3          : caption3,          $
             credit            : credit             }



;---------------------------- YSOs parameters ---------------------------------

  name_obj      =  ['CoKu Tau/1',                     $
                    'IRAS 04302+2247',                $
                    'HH 47 Jet - [SII]',              $ ; Objects names
                    'HH 47 Jet - H_alpha',            $
                    'star forming region in NGC 253', $
                    'GG Tau - J band',                $
                    'GG Tau - scale in R band'        ]
                                                           ; .fits map filename

  obj_file = [filepath("CoKu_TAU1_HST_1999.fits",ROOT=!caos_env.modules, $
                       SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"]), $
              filepath("IRAS_04302+2247_HST_1999.fits",ROOT=!caos_env.modules, $
                       SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"]), $
              filepath("HH47_SII_HST.fits",ROOT=!caos_env.modules, $
                       SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"]), $
              filepath("HH47_Halpha_HST.fits",ROOT=!caos_env.modules, $
                       SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"]), $
              filepath("NGC253_HST.fits",ROOT=!caos_env.modules, $
                       SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"]), $
              filepath("GGtauJ_scal.fits",ROOT=!caos_env.modules, $
                       SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"]), $
              filepath("GGtauJ_scal_R.fits",ROOT=!caos_env.modules, $
                       SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"])  ]

  tot_mag       = [10.85, 11.07, 0., 0., 0., 11., 17.]; Original object total magnitude

  bright        = [12.,12., 12., 12., 12., 12., 12.]; Original brightness [mag/arcsec2] 

  extension     = [0.3, 0.3, 0.3, 0.3, 0.3, 0.6, 0.4]; Original extension [arcsec]

  wave_range_choice     = [0, 0, 1, 1, 0, 0, 0]; Wavelength range considered
                                               ; 0 : Broad band standard filter
                                               ; 1 : Custom wavelength range

  bright_mag_choice     = [0, 0, 0, 0, 1, 0, 0]; Choice beetween luminosity express in
                                               ; 0 : totale magnitude 
                                               ; 1 : average brightness  
                                               ; (for broad band only)

  flux                  = [0., 0., 500., 500., 0., 0., 0.]; Flux in photons/s/m2
                                             ;emitted by the object
                                             ;(for custom wavelength range only)

; image caption #1 (general characteristics and object type)
  caption1 = ['Faint Herbig-Haro object exciting star.', $
              'Class I source.', $                                                           
              'Herbig-Haro object with bipolar outflow jets', $                                                           
              'Herbig-Haro object with bipolar outflow jets', $                                                           
              '', $                                                           
              'Binary T Tauri stars (close pair of the quadripule GG Tau system).', $         
              'Binary T Tauri stars (close pair of the quadripule GG Tau system).'  $
            ]                                                           
; image caption #2 (data acquisition characteristics)
  caption2 = ['Composition of 3 near-IR HST/NICMOS broadband filters [F110W (1.1 mum), F160W (1.6 mum)'+ $
              ' and F205(2.05 mum)].', $ 
              'Composition of 3 near-IR HST/NICMOS broadband filters [F110W (1.1 mum), F160W (1.6 mum)'+ $
              ' and F205(2.05 mum)].', $                                                           
              'Raw HST/WFPC2 data, narrow-band filter in the [SII] doublet (6716-6731 Ang.),'+ $
              ' HST F673N (center : 6732 Ang., width : 47 Ang.)', $              
              'Raw HST/WFPC2 data, narrow-band filter in H_alpha HST F673N (center : 6563 Ang., width : 21 Ang.)', $ 
              '', $                                                           
              'Adaptive optics J band enhanced by deconvolution', $     
              'Adaptive optics J band enhanced by deconvolution, and scaled in R-band'  $
             ]                                                           

; image caption #3 (paper reference)
  caption3 = ['(Padgett, D.L. et al, AJ, 1999, 117, 1490)', $ 
              '(Padgett, D.L. et al, AJ, 1999, 117, 1490)', $                                                           
              '(Heathcote et al., 1996, AJ, 112, 1141)', $                                                           
              '(Heathcote et al., 1996, AJ, 112, 1141)', $                                                           
              '', $                                                           
              '(Roddier, C. et al, ApJ, 1996, 463, 326 )', $                                                           
              '(Roddier, C. et al, ApJ, 1996, 463, 326 ) and (Correia et al., 2000, Proc. SPIE Vol. 4006)'  $
             ]                                                           

; image credit 
  credit = ['D. Padgett (IPAC/Caltech), W. Brandner (IPAC), K. Stapelfeldt (JPL) and NASA ', $    
            'D. Padgett (IPAC/Caltech), W. Brandner (IPAC), K. Stapelfeldt (JPL) and NASA ', $              
            'S. Heathcote, J.A. Morse, P. Hartigan, B. Reipurth, R.D. Schwartz, J. Bally, and J.M. Stone', $     
            'S. Heathcote, J.A. Morse, P. Hartigan, B. Reipurth, R.D. Schwartz, J. Bally, and J.M. Stone', $ 
            '', $                                                           
            'C. Roddier, F. Roddier, M.J. Northcott, J.E. Graves, and K. Jim (IA, Univers. of Hawaii)', $   
            'C. Roddier, F. Roddier, M.J. Northcott, J.E. Graves, and K. Jim (IA, Univers. of Hawaii) and S.Correia'  $
           ]                                                           
  
  YSO   = {  YSO_st,                                $
             name_obj          : name_obj,          $
             obj_file          : obj_file,          $
             tot_mag           : tot_mag,           $
             bright            : bright,            $
             extension         : extension,         $
             wave_range_choice : wave_range_choice, $
             bright_mag_choice : bright_mag_choice, $
             flux              : flux,              $
             caption1          : caption1,          $
             caption2          : caption2,          $
             caption3          : caption3,          $
             credit            : credit             }


;---------------------------- Stellar surface parameters ----------------------

  star_mag          = 15.
  star_diam         = 30.
  star_ax_ratio     = 0.9
  spot_diam         = 3.
  spot_flux         = 5.
  spot_dist         = 6.
  spot_pa           = 24.
 
;---------------------------- User-defined object parameters ------------------


  name_obj      = ['User-defined']                         ; Objects names

                                                           ; .fits map filename
  obj_file      = [filepath("obj_saved.fits",ROOT=!caos_env.modules, $
                            SUB=[info.pack_name+!caos_env.delim+"modules"+!caos_env.delim+"obj"+!caos_env.delim+"obj_data"])]          

  tot_mag       = [0.]                                     ; Original object total magnitude

  bright        = [22.36]                                  ; Original brightness [mag/arcsec2] 

  extension     = [0.3]                                    ; Original extension [arcsec]

  wave_range_choice     = [0]                              ; Wavelength range considered
                                                           ; 0 : Broad band standard filter
                                                           ; 1 : Custom wavelength range

  bright_mag_choice     = [1]                              ; Choice beetween luminosity express in
                                                           ; 0 : totale magnitude 
                                                           ; 1 : average brightness  
                                                           ; (for broad band only)

  flux                  = [500.]                            ; Flux in photons/s/m2 emitted by the object
                                                           ; (for custom wavelength range only)
  
  user_def   = {  user_def_st,                      $
             name_obj          : name_obj,          $
             obj_file          : obj_file,          $
             tot_mag           : tot_mag,           $
             bright            : bright,            $
             extension         : extension,         $
             wave_range_choice : wave_range_choice, $
             bright_mag_choice : bright_mag_choice, $
             flux              : flux               }


;-------------------------- sky background defaults ---------------------------

; get the sky background default magnitude in each pre-defined band
; (see .../lib/n_phot.pro)
dummy = n_phot(0., BACK_MAG=dumback)

skymag=dumback[3:7]   ; sky magnitudes from R-band to K-band

;------------------------------------------------------------------------------

;per default object model
save_obj_fits     = {                                                                                 $
                    save_obj_fits_st,                                                                 $
                    file        :  " ",         $  
                    enable      :  0B                                                                 }


par = {                                                                     $
           obj                                                            , $
           module                  : module                               , $
           type_obj_tab            : type_obj_tab                         , $
           band_tab                : [ "R",  "I",  "J",  "H",  "K" ]      , $
           band_index              : 0L                                   , $
           lambda_tab              : lambda_tab                           , $
           lambda                  : 4500.                                , $
           width                   : 40.                                  , $
           pix_size_tab            : 1e-3 * [ 1.6, 2.0, 2.8, 3.6, 5.0]    , $
           pix_size                : 2.1333e-3                            , $
           array_dim               : 128L                                 , $
           nb_pix_out              : 128L                                 , $
           type_obj_nb             : type_obj_nb                          , $
           type_obj                : 0                                    , $
           obj_nb                  : 0                                    , $
           save_obj_fits           : save_obj_fits                        , $
           field_extension         : 0.                                   , $
           sky_flux                : 0.                                   , $
              
           single_star_mag         : single_star_mag                      , $

           bin_separation          : bin_separation                       , $
           bin_pa                  : bin_pa                               , $
           bin_mag                 : bin_mag                              , $
           bin_delt_m              : bin_delt_m                           , $

           clust_nb_star           : clust_nb_star                        , $
           clust_mag_max           : clust_mag_max                        , $
           clust_mag_min           : clust_mag_min                        , $
           clust_seed              : clust_seed                           , $
                             
           neb                     : neb                                  , $
           neb_field_mag           : 0.                                   , $              
           neb_spectra_choice      : 0B                                   , $              
           neb_spectra_mag         : 0.                                   , $              
           neb_spectra_nb          : 0.                                   , $

           SNR                     : SNR                                  , $
           SNR_field_mag           : 0.                                   , $              
           SNR_spectra_choice      : 0B                                   , $              
           SNR_spectra_mag         : 0.                                   , $              
           SNR_spectra_nb          : 0.                                   , $
              
           gal                     : gal                                  , $
           gal_field_mag           : 0.                                   , $
              
           YSO                     : YSO                                  , $
           YSO_field_mag           : 0.                                   , $

           star_mag                : star_mag                             , $
           star_diam               : star_diam                            , $
           star_ax_ratio           : star_ax_ratio                        , $
           spot_diam               : spot_diam                            , $
           spot_flux               : spot_flux                            , $
           spot_dist               : spot_dist                            , $
           spot_pa                 : spot_pa                              , $

           user_def                : user_def                             , $
           user_def_field_mag      : 0.                                   , $
           flag                    : -1                                   , $

           skymag                  : skymag                                 }  ; sky magnitudes (from R-band to K-band)


save, par, FILENAME=info.def_file

end
