; $Id: obj_info.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;       obj_info
;
; PURPOSE:
;       obj_info is the routine that returns the basic information about
;       the module OBJ.
;
; CATEGORY:
;       module's utility routine
;
; CALLING SEQUENCE:
;       info_structure = obj_info()
; 
; OUTPUTS:
;       info_structure: structure containing the module's basic information.
;
; MODIFICATION HISTORY:
;    program written: may 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -adapted to version 4.0 of the whole system CAOS
;                     (variable "pack_name" added, and variable "mod_type"
;                     changed into "mod_name").
;                    -useless variable "calib" eliminated.
;                    -variable info.help added (instead of !caos_env.help).
;
;-
;
function obj_info

pack_name = 'AIRY_Software_Package_7.2'
                               ; Software Package name
help_file = 'airy_help.html'
help      = !caos_env.modules+pack_name+!caos_env.delim+'help'+!caos_env.delim+help_file
                               ; help-file address for Soft.Pack. AIRY
ver       = fix(7)             ; version number of the Software Package used

mod_name = 'obj'               ; Module name. Only LOWCASE string allowed
descr    = 'OBJect definition'
def_file = mk_par_name(mod_name, PACK_NAME=pack_name, /DEFAULT) ; default parameter file

inp_type = ''                  ; no input
out_type = 'src_t'             ; output data type

init    = 1B                   ; initialisation file management is required
time    = 0B                   ; time integration/delay management is NOT required

; convert in low case format and eliminate all the blanks
mod_name = strlowcase(strcompress(mod_name, /REMOVE_ALL))
inp_type = strlowcase(strcompress(inp_type, /REMOVE_ALL))
out_type = strlowcase(strcompress(out_type, /REMOVE_ALL))

; control the module name length
IF (STRLEN(mod_name) NE !caos_env.module_len) THEN $
   MESSAGE, 'the module name must have '+STRTRIM(!caos_env.module_len)+' chars'

; resulting info structure
info = {                      $
         pack_name: pack_name,$
         help     : help,     $
       ver      : ver,      $

         mod_name: mod_name,  $
         descr   : descr,     $
         def_file: def_file,  $

         inp_type: inp_type,  $
         out_type: out_type,  $

         init    : init,      $
         time    : time       $
       }

return, info
end
