function model_binary_star, x_main, y_main, x_comp, y_comp, sep, pa, $
                             mag, delt_m, nb_pix_in, band, pix_size,  $
                             binary, nb_ph_object
;;
; last modif. : 12 Nov. 2003, Marcel Carbillet (OAA) [marcel@arcetri.astro.it]
;              -Warning added when binary sep. and/or binary pos. angle has
;               been changed in order to match the pixels.
;              -procedure-->function and error management added in order to
;               possibly stop the program in the standard "clean" CAOS way.
;;

error = !caos_error.ok

binary		= fltarr(nb_pix_in, nb_pix_in)
main            = fltarr(nb_pix_in, nb_pix_in)
comp            = fltarr(nb_pix_in, nb_pix_in)

;Main component ---------------------------------
xm = round(x_main) 
ym = round(y_main) 

nb_ph_main    = (n_phot(mag, BAND=band))[0]
main[xm,ym]   = nb_ph_main

;Second component -------------------------------
xc = round(x_comp)
yc = round(y_comp)

nb_ph_comp    = (n_phot(mag+delt_m, BAND=band))[0]
comp[xc,yc]   = nb_ph_comp


binary = main + comp


field_extension  = pix_size * nb_pix_in        ; in arcsec
nb_ph_object = nb_ph_main + nb_ph_comp

; Centered  -> new separation and PA
sep_T = sqrt((xc-xm)^2 + (yc-ym)^2)

if ((xc-xm)/sep_T) eq -1. then pa_T = 180. $   ; remove the indetermination
                                               ; of the following equation
 else pa_T = 2*atan (((ym-yc)/sep_T)/(1 + (xc-xm)/sep_T)) * 180/!pi

if pa_T lt 0. then pa_T = pa_T + 360.          ; enforce the positivity of PA.

; print object characteristics

print, ''
print, ''
print, 'Binary star :'
print, ''

print, 'Requested separation [arcsec]                 =', sep
print, 'Rounded pixel centering separation [arcsec]   =', sep_T*pix_size

print, 'Requested Position Angle [deg.]               =', pa
print, 'Rounded pixel centering Position Angle [deg.] =', pa_T

if (sep-sep_T*pix_size)/sep gt 1E-2 or (pa-pa_T)/pa gt 1E-2 then begin
   dummy=dialog_message(                                                                   $
      ['The actual binary separation and/or position angle has been slightly modified in', $
       'order to match the pixels (see actual values just printed in the main window)...', $
       ''                                                                                , $
       'If you agree with this just click YES in order to continue running the program. ', $
       'Otherwise click NO to abort the program and possibly change your input values.  '],$
      /QUEST, TITLE="OBJ WARNING")
   if dummy eq 'No' then begin
      print, 'OBJ: program aborted as requested by user.'
      error = !caos_error.module_error
      return, error
   endif
endif
print, 'Main component magnitude       =', mag
print, 'Second component magnitude     =', mag+delt_m    

print, 'Pixel size [arcsec]            =', pix_size
print, 'Array dimensions [pixels]      =', nb_pix_in
print, 'Total field extension [arcsec] =', field_extension
print, 'Total object flux [ph/s/m2]    =', nb_ph_object
print, ''     

; update new values of separation and position angle before exiting
sep = sep_T*pix_size
pa  = pa_T

return, error
end
