; $Id: pex_open_beta_gui.pro,v 6.1 2013/04/19 Andrea La Camera $
;
;+
; NAME:
;    pex_open_beta_gui
;
; PURPOSE:
;    pex_open_beta_gui generates a Graphical User Interface (GUI) 
;    for setting up the parameters for automatic search procedure of 
;    beta (Moffat power law index)
;
; CATEGORY:
;    GUI
;
; CALLING SEQUENCE:
;    result = pex_open_beta_gui (title,state,GROUP_LEADER=group)
;
; INPUTS:
;    title     : the title choose for the sub-GUI.
;
; OPTIONAL INPUTS:
;    none.
;
; KEYWORD PARAMETERS:
;    none.
;
; OUTPUTS:
;    parameters for the automatic search procedure 
;
; OPTIONAL OUTPUTS:
;    none.
;
; COMMON BLOCKS:
;    none.
;
; SIDE EFFECTS:
;    none.
;
; RESTRICTIONS:
;    none.
;
; PROCEDURE:
;    none.
;
; EXAMPLE:
;    ...
;
; PROGRAM MODIFICATION HISTORY:
;    program written: May 2012,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it].
;    modifications  : 
;
;
;-
;
pro pex_open_beta_gui_setting, id
  
  common beta_common, param, orig
  
  
  if (param.back_user eq 0B) then begin
     widget_control, id.val_back, SENSITIVE=0
     widget_control, id.reset_val_back, SENSITIVE=0
     widget_control, id.val_back,SET_VALUE=['']
     widget_control, id.back_arr_sel, SENSITIVE=0
     widget_control, id.back_file, SENSITIVE=0 
  endif else begin
     widget_control, id.back_arr_sel, /SENSITIVE
     if (param.back_variable EQ 0B) then begin
        widget_control, id.val_back, /SENSITIVE
        widget_control, id.reset_val_back, /SENSITIVE
        widget_control, id.val_back,SET_VALUE=transpose(param.back_val)
        widget_control, id.back_file, SENSITIVE=0
     endif else begin
        widget_control, id.val_back, SENSITIVE=0
        widget_control, id.reset_val_back, SENSITIVE=0
        widget_control, id.val_back,SET_VALUE=['']
        widget_control, id.back_file, /SENSITIVE
     endelse    
  endelse
  
end


pro pex_open_beta_gui_event, event
  
  common beta_common, param, orig
  
  WIDGET_CONTROL, event.id, GET_UVALUE=uvalue
  WIDGET_CONTROL, event.top, GET_UVALUE=id
  case uvalue of
     'mag_val' : begin
        dummyx = event.x
        dummyy = event.y
        WIDGET_CONTROL, id.val_mag, get_value=dummy
        param.mag[dummyy]=dummy[dummyx,dummyy]
     end
     
     'reset_val_mag' : begin
        param.mag[*]=0.
        WIDGET_CONTROL, id.val_mag, SET_VALUE=transpose(param.mag)
     end
     
     
     'back_user' : begin
        param.back_user=event.value 
        pex_open_beta_gui_setting, id          
     end
     'back_arr_sel': begin
        dummy= event.value
        if (dummy EQ 0) then begin
           param.back_variable = 0B
        endif else begin
           param.back_variable=1B
        endelse
        pex_open_beta_gui_setting, id  
     end
     
     'back_val'  : begin
        dummyx = event.x
        dummyy = event.y
        WIDGET_CONTROL, id.val_back, get_value=dummy
        param.back_val[dummyy]=dummy[dummyx,dummyy]
     end
     
     'reset_val_back' : begin
        param.background[*]=0.
        WIDGET_CONTROL, id.val_back, SET_VALUE=transpose(param.background)
     end
     'back_file': begin
        if event.value EQ '' then return
        param.back_file=event.value
     end
     
     'int_time'  : begin
        dummyx = event.x
        dummyy = event.y
        WIDGET_CONTROL, id.int_time, get_Value=dummy
        
                                ; check whether multi-frame images are set up  or not.
        a=where(param.time_integ[*,1] GT 1., num)
        if (event.x EQ 1) AND (dummy[dummyx,dummyy] GT 1) then begin
           a=dialog_message("Multi-frame images loading: NOT "+$
                            "YET IMPLEMENTED!", DIALOG_PARENT=event.top,$
                            TITLE='LRF error',/ERROR )
           dummy[dummyx,dummyy]=1
        endif
        
        if dummyx eq 0 then begin
           param.time_integ[dummyy,dummyx]=double(dummy[dummyx,dummyy])
        endif else begin
           param.time_integ[dummyy,dummyx]=double(fix(dummy[dummyx,dummyy]))
           WIDGET_CONTROL, id.int_time,  SET_VALUE=transpose(param.time_integ)             
        endelse
     end
     
     'reset_int_time' : begin
        param.time_integ[*,0]=0.
        param.time_integ[*,1]=1.
        WIDGET_CONTROL, id.int_time, SET_VALUE=transpose(param.time_integ)
     end
     
     
     'tel_surf'   : begin
        param.tel_surf=event.value      
     end
     'efficiency' : begin
        param.efficiency=event.value
     end
     'beta1': begin
        param.moffat_beta1=event.value
     end               
     'beta2': begin
        param.moffat_beta2=event.value
     end               
     'tol': begin
        param.moffat_tol=event.value
     end               
     'reset': begin
        param.mag=orig.mag
        param.back_user=orig.back_user 
        param.back_val=orig.back_val
        param.back_file = orig.back_file
        param.back_variable = orig.back_variable
        param.time_integ=orig.time_integ
        param.tel_surf=orig.tel_surf
        param.efficiency=orig.efficiency
        param.moffat_beta1=orig.moffat_beta1
        param.moffat_beta2=orig.moffat_beta2
        param.moffat_tol=orig.moffat_tol
        WIDGET_CONTROL, id.val_mag, SET_VALUE=transpose(param.mag)
        WIDGET_CONTROL, id.val_back, SET_VALUE=transpose(param.background)
        WIDGET_CONTROL, id.int_time, SET_VALUE=transpose(param.time_integ)
        WIDGET_CONTROL, id.tel_surf, SET_VALUE=param.tel_surf
        WIDGET_CONTROL, id.efficiency, SET_VALUE=param.efficiency
        WIDGET_CONTROL, id.beta1, SET_VALUE=param.moffat_beta1
        WIDGET_CONTROL, id.beta2, SET_VALUE=param.moffat_beta2
        WIDGET_CONTROL, id.tol, SET_VALUE=param.moffat_tol
        pex_open_beta_gui_setting, id  
     end
     
     'ok'   : begin
        WIDGET_CONTROL, event.top, /DESTROY
     end
  endcase
  
end


;;;;;;;;;;;;;;;;;;;;;;;;;
; GUI generation code
;;;;;;;;;;;;;;;;;;;;;;;;;
function pex_open_beta_gui, title, state, GROUP_LEADER=group
  
  common beta_common, param, orig
  param=state.par
  orig=state.par
;param is the structure (of the parameters) containing edited/modified values
;orig is the structure containing original values
  
  error = !caos_error.ok
  id = { $
       root_base_id0   : 0L, $
       ok_manage_filter:0L, $
       val_mag:0L, $
       reset_val_mag:0L, $
       val_back:0L, $
       back_file:0L, back_arr_sel:0L, $ 
       reset_val_back:0L, $
       int_time:0L, $
       reset_int_time:0L, $
       tel_surf:0L, $
       efficiency:0L, $
       beta1:0L, $
       beta2:0L, $
       tol:0L $
       }
  
  modal = n_elements(group) ne 0
  root_base_id0 = widget_base(TITLE=title,/COL,MODAL=modal, GROUP_LEADER=group)
  root_base_id = widget_base(root_base_id0, /FRAME,/COL)
  
  title=widget_base(root_base_id,/ROW)
  text = WIDGET_LABEL(title, VALUE=' Settings ')
  
  middle=widget_base(root_base_id,/FRAME,/row, /ALIGN_BOTTOM)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  middle1=widget_base(middle,/COL, /FRAME, /ALIGN_TOP)
  text = WIDGET_LABEL(middle1, VALUE=' Magnitude of the reference-star: ')
  mag_table=widget_base(middle1,xsize=300,ysize=160)
  rlabel=make_array(360,/string,value='Angle #')
  num=indgen(360)
  rlabel=rlabel+strcompress(string(num),/remove_all)
  id.val_mag = widget_table(mag_table,$
                            ALIGNMENT=2,$
                            xoffset=55,yoffset=0,$
                            value=transpose(param.mag),$
                            uvalue="mag_val",$
                            editable=1,$
                            COLUMN_LABELS=['Magnitude'],$
                            ROW_LABELS=rlabel,$
                            COLUMN_WIDTHS=150,$
                            y_scroll_size=5   )
  
  button_mag=widget_base(mag_table,xoffset=58,yoffset=3,xsize=64,ysize=26)
  id.reset_val_mag=widget_button(button_mag,$
                                 VALUE="Reset Val", $
                                 UVALUE="reset_val_mag")
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  middle2=widget_base(middle,/COL, /FRAME, /ALIGN_TOP)
  text = WIDGET_LABEL(middle2, VALUE=' Background: ')
  values = ['consider the background evaluated by the MMM algorithm', $
          'consider a user-defined background'                      ]
  back_group = cw_bgroup (middle2, values, ROW=2,         $
                          SET_VALUE=param.back_user,$
                          UVALUE='back_user', /EXCLUSIVE )
  id.back_arr_sel = cw_bgroup (middle2,        $
                               ['Constant background (single value per image)',$
                                'Variable background (Fits file shaped '+$
                                'as input image)'],       $
                               ROW=2,         $
                             SET_VALUE=param.back_variable,   $
                               UVALUE='back_arr_sel', $
                               /EXCLUSIVE     )
  
  back_table=widget_base(middle2,xsize=300,ysize=160)
  rlabel=make_array(360,/string,value='Angle #')
  num=indgen(360)
  rlabel=rlabel+strcompress(string(num),/remove_all)
  
  id.val_back = widget_table(back_table,$
                             ALIGNMENT=2,$
                             xoffset=55,yoffset=0,$
                             value=transpose(param.back_val),$
                             uvalue="back_val",$
                             editable=1,$
                             COLUMN_LABELS=['Background flux'],$
                             ROW_LABELS=rlabel,$
                             COLUMN_WIDTHS=150,$
                             y_scroll_size=5   )
  
  button_bkg=widget_base(back_table,xoffset=58,yoffset=3,xsize=64,ysize=26)
  id.reset_val_back=widget_button(button_bkg,$
                                  VALUE="Reset Val", $
                                  UVALUE="reset_val_back")
  
  id.back_file=CW_FILENAME(middle2, TITLE="Load a FITS file",$
                           VALUE=param.back_file,$
                           UVALUE='back_file',/ALL_EVENTS)
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  middle3=widget_base(middle,/COL, /FRAME, /ALIGN_TOP)
  text = WIDGET_LABEL(middle3, VALUE=' INTEGRATION TIME:       ')
  int_time_table=widget_base(middle3,xsize=380,ysize=160)
  rlabel=make_array(360,/string,value='Angle #')
  num=indgen(360)
  rlabel=rlabel+strcompress(string(num),/remove_all)
  id.int_time = widget_table(int_time_table,$
                             ALIGNMENT=2,$
                             xoffset=55,yoffset=0,$
                             value=transpose(param.time_integ),$
                             uvalue="int_time",$
                             editable=1,$
                             COLUMN_LABELS=['Integration x frame [s]','Frames #'],$
                             ROW_LABELS=rlabel,$
                             COLUMN_WIDTHS=[150,60],$
                             y_scroll_size=5   )
  button_int_time=widget_base(int_time_table,xoffset=58,yoffset=3,xsize=64,ysize=26)
  id.reset_int_time=widget_button(button_int_time,$
                                  VALUE="Reset Val", UVALUE="reset_int_time")
  
  
  
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  bottom=widget_base(root_base_id,/FRAME,/row)
  bottom_l=widget_base(bottom,/FRAME,/col)
  
  id.tel_surf=CW_FIELD(bottom_l, /ROW,  /return_events, $
                       TITLE='Telescope surface (m^2): [ENTER]', $
                       VALUE=param.tel_surf, $
                       UVALUE='tel_surf',xsize=7)
  id.efficiency=CW_FIELD(bottom_l, /ROW,  /return_events, $
                         TITLE='Overall efficiency: [ENTER]', $
                         VALUE=param.efficiency, $
                         UVALUE='efficiency',xsize=7)
  
  bottom_r=widget_base(bottom,/FRAME,/col,/ALIGN_CENTER)
  text = WIDGET_LABEL(bottom_r, VALUE='Extreme values of the automatic procedure:')
  bottom_autobeta=widget_base(bottom_r,/row)
  id.beta1=CW_FIELD(bottom_autobeta, /ROW, /return_events, $
                    TITLE='Initial value: [ENTER]', $
                    VALUE=param.moffat_beta1, uvalue='beta1',xsize=7)
  id.beta2=CW_FIELD(bottom_autobeta, /ROW, /return_events, $
                    TITLE='Final value: [ENTER]', $
                    VALUE=param.moffat_beta2, uvalue='beta2',xsize=7)
  
  id.tol=CW_FIELD(bottom_r, /ROW, /return_events, $
                  TITLE='Tolerance: [ENTER]', $
                  VALUE=param.moffat_tol, uvalue='tol',xsize=7)
  
  
  
; standard buttons base
  btn_base_id = widget_base(root_base_id0,/FRAME,/ROW, /ALIGN_RIGHT)
  reset_id = widget_button(btn_base_id,                               $
                           VALUE="Reset to default settings", $
                           UVALUE="reset"                                )
  
  ok_id = widget_button(btn_base_id,                               $
                        VALUE=" Save and close ", $
                        UVALUE="ok"                                )
  
  
  
  WIDGET_CONTROL, root_base_id0, SET_UVALUE=id
  WIDGET_CONTROL, root_base_id0, /REALIZE
  pex_open_beta_gui_setting, id          
  xmanager, 'pex_open_beta_gui', root_base_id0, /NO_BLOCK, GROUP_LEADER=group
  return, param
end
