; $Id: pex_open_gui.pro, v 7.1 last revision 2017/01/09 Andrea La Camera $
;+
; pex_open_gui is used to open PEX_GUI either in worksheet or during project's
; execution.
;-
;**************************************************************************
;index of PROcedure:
;>PEX_GUI_IMAGE (shows image into input tab)
;>PEX_GUI_PSF (shows image in PSF tab, with zoom/scale parameters)
;>PEX_GUI_BANDSET (Indicating in GUI which bands have been selected)
;>PEX_GUI_LOAD (load image and display all parameters)
;>PEX_OPEN_GUI_EVENT (PSF EXTRACT HANDLE EVENT)
;>function pex_open_gui

;-------------------------------------------------------------------
;PEX_GUI_IMAGE (show entire image into area)
;-------------------------------------------------------------------
pro pex_gui_image, state
COMMON pex_image, image_read, _N, _M, _P, header


loadct, state.par.color, /SILENT

WIDGET_CONTROL, /HOURGLASS
a_dim=256.
wset, state.id.iarea
if _P GT 1 then a_img=image_read[*,*,state.par.inumber] else a_img=image_read

case state.par.scale of
   '0': tvscl, CONGRID(a_img,a_dim,a_dim)
   '1': begin
      a_img2=ALOG10(1.+a_img)
      tvscl, CONGRID(a_img2,a_dim,a_dim)
      end
   '2': tvscl, (CONGRID(a_img,a_dim,a_dim))^state.par.power
   ;'3': tvscl, SQRT(CONGRID(a_img,a_dim,a_dim))
   ;'4': tvscl, CGGMASCL(CONGRID(a_img,a_dim,a_dim), gamma=0.2)
   ;'5': tvscl, CGGMASCL(CONGRID(a_img,a_dim,a_dim), gamma=0.1)  
endcase

return
end; of pex_gui_image

;-------------------------------------------------------------------
;PEX_GUI_PSF (show image in psf tab, with zoom/scale parameters)
;-------------------------------------------------------------------
pro pex_gui_psf, state, REDRAW=redraw
COMMON pex_image, image_read, _N,_M,_P, header

loadct, state.par.color, /SILENT

p_dim=512
WIDGET_CONTROL, /HOURGLASS

if _P GT 1 then img=image_read[*,*,state.par.inumber] else img=image_read
xdim=(size(img))[1]
ydim=(size(img))[2]
;show image (REDRAW if requested)
IF KEYWORD_SET(redraw) THEN BEGIN
IF xdim*state.par.zoom GT p_dim OR ydim*state.par.zoom GT p_dim THEN BEGIN
   Widget_Control, state.id.pbase, Map=0
   Widget_Control, state.id.pkill, /destroy
   drawID = Widget_Draw(state.id.pbase, /SCROLL, KEYBOARD_EVENTS=2, $
         /button_events, uvalue='parea', /ALIGN_CENTER, $
         XSize=xdim*state.par.zoom, $
         YSize=ydim*state.par.zoom, $
         X_Scroll_Size= p_dim,$
         Y_Scroll_Size= p_dim,$
         TOOLTIP='shift + click to select, ctrl + click to remove; '+$
                 'right-click for options')
   Widget_Control, drawID, /Realize
   Widget_Control, drawID, Get_Value=wid
   state.id.parea = wid
   state.id.pkill = drawID
   Widget_Control, state.id.pbase, Map=1
ENDIF ELSE BEGIN
   Widget_Control, state.id.pbase, Map=0
   Widget_Control, state.id.pkill, /destroy
   drawID = Widget_Draw(state.id.pbase, KEYBOARD_EVENTS=2, $
        /button_events, uvalue='parea', /ALIGN_CENTER,  $
        XSize=xdim*state.par.zoom, $
        YSize=ydim*state.par.zoom, $
        TOOLTIP='shift + click to select, ctrl + click to remove; '+$
                'right-click for options')
  Widget_Control, drawID, /Realize
  Widget_Control, drawID, Get_Value=wid
  state.id.parea = wid
  state.id.pkill = drawID
  Widget_Control, state.id.pbase, Map=1
ENDELSE
ENDIF

wset, state.id.parea
case state.par.scale of
  '0': tvscl, CONGRID(img,xdim*state.par.zoom, ydim*state.par.zoom)
  '1': begin
     img2=ALOG10(1.+img)
     tvscl,CONGRID(img2,xdim*state.par.zoom, ydim*state.par.zoom)
     end
  '2': tvscl, CONGRID(img,xdim*state.par.zoom, ydim*state.par.zoom)^state.par.power
  ;'3': tvscl, SQRT(CONGRID(img,xdim*state.par.zoom, ydim*state.par.zoom))
  ;'4': tvscl, CGGMASCL(CONGRID(img,xdim*state.par.zoom, ydim*state.par.zoom),$
  ;                   gamma=0.2)
  ;'5': tvscl, CGGMASCL(CONGRID(img,xdim*state.par.zoom, ydim*state.par.zoom),$
  ;                   gamma=0.1)
endcase


;mark selected stars with "+" and circle
color1 = 255 
for k=0,state.par.nb_click[state.par.inumber]-1 do begin
   plots, state.par.p_xsave[k,state.par.inumber]*state.par.zoom, $
          state.par.p_ysave[k,state.par.inumber]*state.par.zoom, $
       PSYM=1, /DEVICE, COLOR= color1, SYMSIZE=_N/250*state.par.zoom
   pex_circle, state.par.p_xsave[k,state.par.inumber]*state.par.zoom, $
           state.par.p_ysave[k,state.par.inumber]*state.par.zoom, $
           state.par.radius*state.par.zoom, vecx, vecy
   plots, vecx, vecy, /DEVICE, COLOR=color1
endfor
if state.par.nb_click[state.par.inumber] GT 0 then begin
   dummy=STRCOMPRESS(string(state.par.nb_click[state.par.inumber])+$
         ' star(s) selected')
endif else  begin
   dummy='Select TARGET and/or PSF...'
endelse
WIDGET_CONTROL, state.id.label, SET_VALUE= dummy

return
end ; of pex_gui_psf


;-------------------------------------------------------------------
; PEX_GUI_BANDSET  (Indicating in GUI which bands have been selected)
; ------------------------------------------------------------------
pro pex_gui_bandset, state

COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band

dummy  = CLOSEST(state.par.lambda,lambda_tab)
value  = INTARR(N_ELEMENTS(band_tab))
n_band = N_ELEMENTS(band_tab)
f_band = FLTARR(n_band)

IF ((state.par.lambda*state.par.width NE 0) AND (dummy GT -1)) THEN BEGIN
   IF (((state.par.lambda - lambda_tab[dummy]) LT 1e-12)  AND        $
       ((state.par.width  - width_tab[dummy ]) LT 1e-12)) THEN BEGIN
      flag_band     = 0B ;Indicates a single band!!
      value[dummy]  = 1
      f_band[dummy] = 1.
   ENDIF ELSE BEGIN
      flag_band = 1B    ;Indicates wavelength range spans over more than 1 band
      lambda1   = state.par.lambda - state.par.width/2
      lambda2   = state.par.lambda + state.par.width/2
      FOR i=0,n_band-1 DO BEGIN
         band1 = lambda_tab[i]-width_tab[i]/2.
         band2 = lambda_tab[i]+width_tab[i]/2.
         f_band[i] = INTERVAL2(lambda1, lambda2, band1, band2, d1, d2)
         lambda1 = d1
         lambda2 = d2
      ENDFOR
      r1 = WHERE(f_band GT 0., c1)
      IF (c1 GT 0) THEN $
        value[r1] = 1.  $
      ELSE              $
        MESSAGE,'Bandwidth selected not within the standard bands. Check!!'
   ENDELSE
ENDIF
WIDGET_CONTROL, state.id.band, SET_VALUE= value
return
end ; of pex_gui_bandset

;-------------------------------------------------------------------
;PEX_GUI_LOAD (loading image and display all parameters)
;-------------------------------------------------------------------
pro pex_gui_load, state, FIRST_TIME=first_time
COMMON pex_image, image_read, _N, _M, _P, header

if state.par.mode EQ 0 then begin
   WIDGET_CONTROL, state.id.name, SET_VALUE=state.par.name
   if (state.par.name EQ '') then begin
      WIDGET_CONTROL, state.id.label, SET_VALUE= 'LOAD AN IMAGE FIRST!!!'
      return
   endif
   ;load image and read size (_N=column='X' and _M=row='Y', _P=number of image 
   ;in the cube)
   ;fits_read, state.par.name, image_read
   image_read = readfits(state.par.name, header)
endif
_N=(size(image_read))[1]
_M=(size(image_read))[2]
IF (size(image_read))[0] EQ 3 THEN _P=(size(image_read))[3] 
IF (size(image_read))[0] EQ 2 THEN _P=1 

state.par.N_out=max([_N,_M])
WIDGET_CONTROL, state.id.oout, SET_VALUE=state.par.N_out

;CUBE or SINGLE image information
if _P GT 1 then begin
   info=[strtrim(_N,1)+'x'+strtrim(_M,1)+' [pixels]', 'Cube: '+STRTRIM(_P,1)+$
        ' images', ' ']
   ;dummy=strcompress('Image #'+string(indgen(_P)+1))
   ;WIDGET_CONTROL, state.id.inumber, SET_VALUE=dummy
endif
if _P EQ 1 then begin
   info=[strtrim(_N,1)+'x'+strtrim(_M,1)+' [pixels]', 'Single image', ' ']
   ;WIDGET_CONTROL, state.id.inumber, SET_VALUE=STRCOMPRESS('Image #'+string(1))
endif
;show info
WIDGET_CONTROL, state.id.info, SET_VALUE=info

; set image number
state.par.inumber=0
;WIDGET_CONTROL, state.id.inumber, SET_COMBOBOX_SELECT=state.par.inumber
WIDGET_CONTROL, state.id.main_im_ac, $
   SET_VALUE=strtrim(state.par.inumber+1,1)
WIDGET_CONTROL, state.id.psf_im_ac, $
   SET_VALUE=strtrim(state.par.inumber+1,1)

;show entire image into left widget_draw
pex_gui_image, state

;display into both PSF and TARGET tabs
pex_gui_psf, state, /REDRAW

;enable other commands
;WIDGET_CONTROL, state.id.inumber, /SENSITIVE
widget_control, state.id.main_im_fi, /SENSITIVE
widget_control, state.id.main_im_pr, /SENSITIVE
widget_control, state.id.main_im_ac, /SENSITIVE
widget_control, state.id.main_im_ne, /SENSITIVE
widget_control, state.id.main_im_la, /SENSITIVE
widget_control, state.id.psf_im_fi, /SENSITIVE
widget_control, state.id.psf_im_pr, /SENSITIVE
widget_control, state.id.psf_im_ac, /SENSITIVE
widget_control, state.id.psf_im_ne, /SENSITIVE
widget_control, state.id.psf_im_la, /SENSITIVE
IF _P GT 1 THEN begin
   WIDGET_CONTROL, state.id.copy, /SENSITIVE
   WIDGET_CONTROL, state.id.delete, /SENSITIVE
ENDIF
IF _P EQ 1 then begin
   WIDGET_CONTROL, state.id.copy, SENSITIVE=0
   WIDGET_CONTROL, state.id.delete, SENSITIVE=0
ENDIF
WIDGET_CONTROL, state.id.radius, /SENSITIVE
WIDGET_CONTROL, state.id.extrap, /SENSITIVE
if (state.par.extrap) then begin
   WIDGET_CONTROL, state.id.find_beta, /SENSITIVE
   WIDGET_CONTROL, state.id.moffat_beta, /SENSITIVE
   if state.par.find_beta EQ 0 then begin
      widget_control, state.id.moffat_beta, /SENSITIVE
      widget_control, state.id.open_beta, SENSITIVE=0
   endif else begin 
      widget_control, state.id.moffat_beta, SENSITIVE=0
      widget_control, state.id.open_beta, /SENSITIVE
   endelse
endif else begin
   WIDGET_CONTROL, state.id.find_beta, SENSITIVE=0
   widget_control, state.id.moffat_beta, SENSITIVE=0
   widget_control, state.id.open_beta, SENSITIVE=0
endelse
WIDGET_CONTROL, state.id.oout, /SENSITIVE
WIDGET_CONTROL, state.id.lambda, /SENSITIVE
WIDGET_CONTROL, state.id.width, /SENSITIVE
WIDGET_CONTROL, state.id.resolution, /SENSITIVE
WIDGET_CONTROL, state.id.reset, /SENSITIVE
WIDGET_CONTROL, state.id.band, /SENSITIVE
WIDGET_CONTROL, state.id.lambda, SET_VALUE=state.par.lambda
WIDGET_CONTROL, state.id.width, SET_VALUE=state.par.width
WIDGET_CONTROL, state.id.resolution, SET_VALUE=state.par.resolution

pex_gui_bandset, state
;show instructions
WIDGET_CONTROL, state.id.label, SET_VALUE='Select one or more stars in PSF Extraction tab...'


return
end;of pex_gui_load



;------------------------------------------------------------------
;PEX_OPEN_GUI_EVENT (PSF EXTRACT HANDLE EVENT)
;------------------------------------------------------------------
pro pex_open_gui_event, event

common error_block, error
COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band
COMMON pex_image, image_read, _N, _M, _P, header

WIDGET_CONTROL, event.ID, GET_UVALUE=choice
WIDGET_CONTROL, event.TOP, GET_UVALUE=state
; handle a kill request (considered as a cancel event).
if tag_names(event, /STRUCTURE_NAME) eq 'WIDGET_KILL_REQUEST' then begin
   error = !caos_error.cancel
   widget_control, event.top, /DESTROY
endif

case choice of
'iarea' : begin
   if (state.par.name EQ '') AND (state.par.mode EQ 0) then return
   if (event.press EQ 4) then begin
      ;select options (zoom, scale, color, etc)
      result=pex_img_options("OPTIONS", state, group_leader=event.top)
          state.par.scale = result.scale
          state.par.power=result.power
          state.par.color = result.color
          pex_gui_image, state
          pex_gui_psf, state, /REDRAW
   endif
end

; ------------------------------------------------------------------
'filename': begin
   state.par.name=event.value
   pex_gui_load, state, /first_time
end ; of filename
; ------------------------------------------------------------------
   'read_keywords': begin
      if state.par.name EQ '' then return
      header = headfits(state.par.name) ;Read header
      dummy = fxpar(header, 'RESOLUT', count = a)
      if (a) then begin
         state.par.resolution = dummy
         widget_control, state.id.resolution, set_value=dummy
      endif
      dummy = fxpar(header, 'LAMBDA', count = a)
      if (a) then begin
         state.par.lambda = dummy
         dummy= CLOSEST(state.par.lambda,lambda_tab)
         widget_control, state.id.lambda, SET_VALUE=state.par.lambda
         pex_gui_bandset, state
      endif
      dummy = fxpar(header, 'WIDTH', count = a)
      if (a) then begin
         state.par.width = dummy
         widget_control, state.id.width, SET_VALUE=state.par.width
         pex_gui_bandset, state
      endif
   end
; ------------------------------------------------------------------
'show_header' : begin
   if state.par.name EQ '' then return
   header = headfits(state.par.name) ;Read header
   xdispstr, header, TITLE="FITS HEADER", group_leader=event.top
end
; ------------------------------------------------------------------
'lambda' : state.par.lambda  = event.value
; ------------------------------------------------------------------
'width' : state.par.width  = event.value
; ------------------------------------------------------------------
'resolution'   : state.par.resolution  = event.value
; ------------------------------------------------------------------
'inumber': begin
   state.par.inumber=event.index
   pex_gui_image, state
   pex_gui_psf, state
   widget_control, state.id.psf_im_ac, set_value=strtrim(state.par.inumber+1,1)
end


; ------------------------------------------------------------------
'parea': begin
   if (state.par.name EQ '') AND (state.par.mode EQ 0) then return
   IF event.clicks EQ 1 THEN BEGIN
      IF event.modifiers EQ 1 AND event.press EQ 1 then begin
         ;select image
         if _P GT 1 then img=image_read[*,*,state.par.inumber] $
         else img=image_read
         ;find centroid
         CNTRD, img, event.x/state.par.zoom, event.y/state.par.zoom, $
                xcen, ycen, 5, /SILENT ;fwhm = 5 pixel
         IF (xcen EQ -1.) OR (ycen EQ -1.) THEN return
         ;save new array psf_xsave and psf_ysave
         pex_append, state, xcen, ycen
         ;show images
         pex_gui_image, state
         pex_gui_psf, state
      ENDIF
      IF event.modifiers EQ 2 AND event.press EQ 1 then begin
         if state.par.nb_click[state.par.inumber] LE 0 then return
         ;select image
         if _P GT 1 then img=image_read[*,*,state.par.inumber] $
         else img=image_read
         ;find centroid 
         CNTRD, img, event.x/state.par.zoom, event.y/state.par.zoom, $
                xcen, ycen, 5, /SILENT ;fwhm = 5 pixel
         IF (xcen EQ -1.) OR (ycen EQ -1.) THEN return
         ;save new array psf_xsave and psf_ysave
         pex_remove, state, xcen, ycen
         ;show images
         pex_gui_image, state
         pex_gui_psf, state
      ENDIF
      if (event.press EQ 4) then begin
         ;select options (zoom, scale, color, etc)
         result=pex_img_options("OPTIONS", state, group_leader=event.top,/PSF)
         state.par.color = result.color
         state.par.scale = result.scale
	 state.par.power = result.power
         state.par.zoom = result.zoom
         pex_gui_image, state
         pex_gui_psf, state, /REDRAW
      endif
      
   ENDIF
end

; ------------------------------------------------------------------
'im_fi': begin ;first image
   state.par.inumber = 0
   pex_gui_image, state
   pex_gui_psf, state 
   ;WIDGET_CONTROL, state.id.inumber, SET_COMBOBOX_SELECT=state.par.inumber
   WIDGET_CONTROL, state.id.main_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
   WIDGET_CONTROL, state.id.psf_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
end
; ------------------------------------------------------------------
'im_pr': begin ;previous image
   state.par.inumber=(state.par.inumber-1) > 0
   pex_gui_image, state
   pex_gui_psf, state 
   ;WIDGET_CONTROL, state.id.inumber, SET_COMBOBOX_SELECT=state.par.inumber
   WIDGET_CONTROL, state.id.main_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
   WIDGET_CONTROL, state.id.psf_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
end
; ------------------------------------------------------------------
'im_ac': begin ;actual image (select new "actual" image)
   state.par.inumber = 0 > fix(event.value-1) < (_P-1)
   pex_gui_image, state
   pex_gui_psf, state 
   ;WIDGET_CONTROL, state.id.inumber, SET_COMBOBOX_SELECT=state.par.inumber
   WIDGET_CONTROL, state.id.main_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
   WIDGET_CONTROL, state.id.psf_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
end
; ------------------------------------------------------------------
'im_ne': begin ;next image
   state.par.inumber=(state.par.inumber+1) < (_P-1)
   pex_gui_image, state
   pex_gui_psf, state 
   ;WIDGET_CONTROL, state.id.inumber, SET_COMBOBOX_SELECT=state.par.inumber
   WIDGET_CONTROL, state.id.main_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
   WIDGET_CONTROL, state.id.psf_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
end
; ------------------------------------------------------------------
'im_la': begin ;last image
   state.par.inumber = _P-1
   pex_gui_image, state
   pex_gui_psf, state 
   ;WIDGET_CONTROL, state.id.inumber, SET_COMBOBOX_SELECT=state.par.inumber
   WIDGET_CONTROL, state.id.main_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
   WIDGET_CONTROL, state.id.psf_im_ac, $
      set_value=strtrim(state.par.inumber+1,1)
end
; ------------------------------------------------------------------
'copy': begin
   ;copy psf_xsave and psf_ysave of this image in ALL images
   ;centroids will be re-calculated!!!
   IF _P GT 1 THEN BEGIN
      new_index=shift(indgen(_P),-state.par.inumber)
      FOR k=1, _P-1 DO BEGIN
         w_img=image_read[*,*,new_index[k]]
         FOR i=0,state.par.nb_click[state.par.inumber]-1 do begin 
            CNTRD, w_img, state.par.p_xsave[i,state.par.inumber], $
                          state.par.p_ysave[i,state.par.inumber], $
                          xcen, ycen, 5, /SILENT
            state.par.p_xsave[i,new_index[k]]=xcen
            state.par.p_ysave[i,new_index[k]]=ycen
         ENDFOR
         state.par.nb_click[new_index[k]]=state.par.nb_click[state.par.inumber]
      ENDFOR
   ENDIF
end
; ------------------------------------------------------------------
'delete': begin
   ;create arrays NB_CLICK[_P] with 0 values (no star selected)
   ;and psf_xsave and psf_ysave with -1. values (no star selected)
   state.par.nb_click[*]=0.
   state.par.p_xsave[*,*]=-1.
   state.par.p_ysave[*,*]=-1.
   pex_gui_image, state
   pex_gui_psf, state
end
; ------------------------------------------------------------------
'radius': begin
   if (state.par.name EQ '') AND (state.par.mode EQ 0) then begin
      WIDGET_CONTROL, state.id.label, SET_VALUE= 'LOAD AN IMAGE FIRST!!!'
      return
   endif
   WIDGET_CONTROL, state.id.label, SET_VALUE='Change Radius'
   state.par.radius=event.value
   pex_gui_psf, state
   if (state.par.N_out LE state.par.radius*2) then begin
      dummy = dialog_message([$
      'Please, decrease the radius value and/or increase the output size. ', $
      'Otherwise there will be an error when the project will run.        '],$
                /info)
   endif
end
; ------------------------------------------------------------------
'psf_extrap': begin
   state.par.extrap = event.value
   if (state.par.extrap) then begin
      WIDGET_CONTROL, state.id.find_beta, /SENSITIVE
      WIDGET_CONTROL, state.id.moffat_beta, /SENSITIVE
      if state.par.find_beta EQ 0 then begin
         widget_control, state.id.moffat_beta, /SENSITIVE
         widget_control, state.id.open_beta, SENSITIVE=0
      endif else begin 
         widget_control, state.id.moffat_beta, SENSITIVE=0
         widget_control, state.id.open_beta, /SENSITIVE
      endelse
   endif else begin
      WIDGET_CONTROL, state.id.find_beta, SENSITIVE=0
      widget_control, state.id.moffat_beta, SENSITIVE=0
      widget_control, state.id.open_beta, SENSITIVE=0
   endelse

end
; ------------------------------------------------------------------
'find_beta': begin
   state.par.find_beta = event.value
   if state.par.find_beta EQ 0 then begin
      widget_control, state.id.moffat_beta, /SENSITIVE
      widget_control, state.id.open_beta, SENSITIVE=0
   endif else begin 
      widget_control, state.id.moffat_beta, SENSITIVE=0
      widget_control, state.id.open_beta, /SENSITIVE
   endelse
   ;lex_gui_psf, state
end
; ------------------------------------------------------------------
'moffat_beta': begin
   state.par.moffat_beta = event.value
   pex_gui_psf, state
end
; ------------------------------------------------------------------
'open_beta': begin
   title='AUTOMATIC SEARCH PROCEDURE - Settings'
   result=pex_open_beta_gui(title, state, GROUP_LEADER=event.top)
   state.par.mag=result.mag
   state.par.back_user=result.back_user
   state.par.back_variable = result.back_variable
   state.par.back_val = result.back_val
   state.par.back_file = result.back_file
   state.par.time_integ=result.time_integ
   state.par.tel_surf=result.tel_surf
   state.par.efficiency=result.efficiency
   state.par.moffat_beta1=result.moffat_beta1
   state.par.moffat_beta2=result.moffat_beta2
   state.par.moffat_tol=result.moffat_tol
end
; ------------------------------------------------------------------
'oout': begin
   if (event.value mod 2) then state.par.N_out=event.value-1 else state.par.N_out=event.value
   widget_control, state.id.oout, SET_VALUE=state.par.N_out
   if (state.par.N_out LE state.par.radius*2) then begin
      dummy = dialog_message(['Please, decrease the radius value',    $
                ' and/or increase the output size. ', $
                'Otherwise there will be an error when the project will run.'],$
                /info)
   endif

end
; ------------------------------------------------------------------
'menu_band': BEGIN
   WIDGET_CONTROL, event.id , GET_VALUE=dummy
   r1 = WHERE(dummy GT 0, c1)
   IF (c1 GT 0) THEN BEGIN
      min_lambda = MIN(lambda_tab[r1]-width_tab[r1]/2.)
      max_lambda = MAX(lambda_tab[r1]+width_tab[r1]/2.)
      lambda = (max_lambda+min_lambda)/2
      width  = (max_lambda-min_lambda)
      state.par.lambda= lambda
      state.par.width = width
   ENDIF ELSE BEGIN
      state.par.lambda= 0.
      state.par.width = 0.
   ENDELSE
   WIDGET_CONTROL, state.id.lambda, SET_VALUE=state.par.lambda
   WIDGET_CONTROL, state.id.width,  SET_VALUE=state.par.width
   pex_gui_bandset, state
END
; ------------------------------------------------------------------
'reset':BEGIN
   state.par.lambda = 0.
   state.par.width  = 0.
   WIDGET_CONTROL,state.id.lambda, SET_VALUE=state.par.lambda
   WIDGET_CONTROL,state.id.width , SET_VALUE=state.par.width
   pex_gui_bandset, state
END

; ------------------------------------------------------------------
; STANDARD BUTTONS
; ------------------------------------------------------------------
'save': begin
   if (state.par.name EQ '') AND (state.par.mode EQ 0) then begin
      WIDGET_CONTROL, state.id.label, SET_VALUE= 'LOAD AN IMAGE FIRST!!!'
      return
   endif
   ;check before continue if user have select almost one star per image
   for k=0, _P-1 do begin 
      dummy=WHERE(state.par.nb_click[k] GT 0, count)
      if count EQ 0 then begin
         dummy = DIALOG_MESSAGE('IMAGE ## '+strtrim(k+1,1)+$
                 ': please, select one or more stars.',$
                 DIALOG_PARENT=event.top, TITLE='PEX warning')
         return
      endif
   endfor
   ;if auto-beta procedure, check if it is selected one star/image
   if (state.par.find_beta EQ 1) AND $ 
      (total(state.par.nb_click EQ 1,/DOU) NE _P) then begin
      dummy = DIALOG_MESSAGE(['Only one reference-star per image can be',$
                    'selected in automatic search procedure (Moffat).'], $
                          DIALOG_PARENT=event.top, TITLE='LEX warning')
      return
   endif
      
   check_file = findfile(state.par.str_sav_file)
   if check_file[0] ne '' then begin
      answ = dialog_message(['file '+state.par.str_sav_file+$
               ' already exists.',   $
               'do you want to overwrite it ?'], DIALOG_PARENT=event.top, $
               TITLE='PEX warning', /QUEST)
      ; return without saving if the user doesn't want to overwrite the
      ; existing parameter file
      if strlowcase(answ) eq "no" then return
   endif else begin
      answ = dialog_message(['file '+state.par.str_sav_file+$
                 ' will be saved.'], $
                 DIALOG_PARENT=event.top, TITLE='PEX information', /INFO)
      ; inform were the parameters will be saved
   endelse
   state.par.firsttime='no'  
   par=state.par
   save,par,FILENAME=state.par.str_sav_file 
   ;kill the GUI returning a null error
   error = !caos_error.ok
   widget_control, event.top, /DESTROY
   return
end
; ------------------------------------------------------------------
; handle event from standard help button (IT'S DIFFERENT BETWEEN WIN AND LINUX)
'help': begin
   online_help, book=(pex_info()).help, /FULL_PATH
;   widget_control, /HOURGLASS
;   CASE !VERSION.OS_FAMILY OF
;     'Windows' : begin
;        spawn, !caos_env.browser+" "+(pex_info()).help, /noshell
;     end
;     else : begin
;        spawn, !caos_env.browser+" "+(pex_info()).help+"&"
;     end
;   ENDCASE
end
; ------------------------------------------------------------------
'restore': begin
   tmp=state.par.name
   par = 0
   title = "parameter file to restore"
   restore, filename_gui(state.par.str_def_file,                  $
                         title,                                   $
                         GROUP_LEADER=event.top,                  $
                         FILTER=state.par.module.mod_name+'*sav', $
                         /NOEDIT,                                 $
                         /MUST_EXIST,                             $
                         /ALL_EVENTS                              )
   ; update the current module number
   par.module.n_module = state.par.module.n_module
   ; update the state structure and SET the GUI
   state.par = par
   state.par.name=tmp
   ; set the default values
   WIDGET_CONTROL, state.id.radius, SET_VALUE=par.radius
   WIDGET_CONTROL, state.id.find_beta, SET_VALUE=par.find_beta
   WIDGET_CONTROL, state.id.moffat_beta, SET_VALUE=par.moffat_beta
   WIDGET_CONTROL, state.id.lambda, SET_VALUE=par.lambda
   WIDGET_CONTROL, state.id.width, SET_VALUE=par.width
   WIDGET_CONTROL, state.id.resolution, SET_VALUE=par.resolution
   pex_gui_bandset, state
   pex_gui_load, state, /first_time
   if state.par.name NE '' then begin
      pex_gui_image, state
      pex_gui_psf, state, /REDRAW
   endif
end
; ------------------------------------------------------------------
'cancel': begin
   error = !caos_error.cancel
   widget_control, event.top, /DESTROY
   return
end

else: return
endcase ;end of main CASE 

; write the GUI state structure
WIDGET_CONTROL, event.top, SET_UVALUE=state
end ; of the PEX_OPEN_GUI_EVENT  

;---------------------------

;-------------------------------------------------------------------
;PEX_OPEN_GUI (main procedure)
;-------------------------------------------------------------------

function pex_open_gui, par, image=image

COMMON error_block, error
COMMON wavelengths, band_tab, lambda_tab, width_tab, flag_band, f_band
COMMON pex_image, image_read, _N, _M, _P, header

; retrieve the module information
info = pex_info()

;image yet read?
if KEYWORD_SET(image) then begin
   image_read=image
   _N=(size(image_read))[1]
   _M=(size(image_read))[2]
   IF  (size(image_read))[0] EQ 3 THEN _P=(size(image_read))[3] 
   IF  (size(image_read))[0] EQ 2 THEN _P=1 
endif



id = { name: 0L, info: 0L, label:0L, $
       read_keywords: 0L, show_header: 0L, resolution:0L,$
       iarea:0L, inumber: 0L, $
       main_im_fi:0L, main_im_pr:0L, main_im_ac:0L, main_im_ne:0L, main_im_la:0L, $
       parea:0L, pkill:0L, pbase:0L, $ 
       dlabel:0L, $
       copy:0L, delete:0L, radius:0L, extrap:0L, find_beta:0L, moffat_beta:0L, open_beta:0L, $
       psf_im_fi:0L, psf_im_pr:0L, psf_im_ac:0L, psf_im_ne:0L, psf_im_la:0L, $
       oout:0L,  $
       lambda:0L, width:0L, band:0L, reset:0L }

state={id:id, par:par}

; WIDGET |
;        V

modal = n_elements(group) ne 0
dummy = strupcase(info.mod_name)+' parameters setting GUI'
base_id = widget_base(TITLE=dummy, MODAL=modal, /COL, GROUP_LEADER=group)
;**************************************************
base0=WIDGET_BASE(base_id, /COL)
; -------------------------------------------------
base_instr=WIDGET_BASE(base0, /ROW, /FRAME)
dummy=WIDGET_LABEL(base_instr, VALUE='Instructions:', /ALIGN_LEFT)
state.id.label=WIDGET_LABEL(base_instr, VALUE=state.par.label, /DYNAMIC_RESIZE)
; *************************************************
base1=WIDGET_BASE(base0, /COL, /FRAME)
basetab=WIDGET_TAB(base1, location=0, uvalue='')
;--------------------------------------------------
baseA=WIDGET_BASE(basetab, /COL, TITLE='PARAMETERS')
;--------------------------------------------------
base_input=WIDGET_BASE(baseA, /ROW, /ALIGN_TOP)

base_inputL = WIDGET_BASE(base_input, /COL, /ALIGN_TOP, /FRAME)
state.id.name=CW_FILENAME(base_inputL,TITLE="Image filename",$
                          UVALUE="filename",/RETURN_EVENTS)
dummy=WIDGET_LABEL(base_inputL, value= 'Image info/settings: ',/align_center)
state.id.info=CW_FIELD(base_inputL, TITLE='Dimensions: ', $
                       VALUE=' ', /NOEDIT, /ROW, YSIZE=3)
base_input_header = widget_base(base_inputL, /ROW)
state.id.read_keywords = widget_button(base_input_header, $
                                       value='Retrieve parameters from header', $
                                       UVALUE = 'read_keywords' )
state.id.show_header = widget_button(base_input_header, $
                                     value='Show header', $
                                     UVALUE = 'show_header' )
bandbase=widget_base(base_inputL, /COL)
state.id.resolution  = cw_field(bandbase, TITLE='Pixel size [arcsec]    ', $
                                /FLOATING,  VALUE=state.par.resolution,      $
                                UVALUE='resolution', /ALL_EVENTS )
dummy=WIDGET_LABEL(bandbase,VALUE=' Wavelength Band: ',/align_left)
bandstuff=widget_base(bandbase, /COL)
state.id.lambda=cw_field(bandstuff,TITLE='Center wavelength [m] ',/FLOATING,$
                         UVALUE='lambda', /ALL_EVENTS)
state.id.width=cw_field(bandstuff,TITLE='Band-width [m]        ',/FLOATING,$
                        UVALUE='width', /ALL_EVENTS)
dummy = N_PHOT(0.,BAND=dummy1,LAMBDA=dummy2,WIDTH=dummy3)
band_tab  = dummy1[3:9]
lambda_tab= dummy2[3:9]
width_tab = dummy3[3:9]
value  = INTARR(N_ELEMENTS(band_tab))
n_band = N_ELEMENTS(band_tab) 
f_band = FLTARR(n_band)
det_base2= WIDGET_BASE(bandbase, /ROW, /BASE_ALIGN_CENTER)
state.id.reset = WIDGET_BUTTON(det_base2,UVALUE='reset',VALUE='Reset Bands')
state.id.band = CW_BGROUP(det_base2, band_tab, /ROW, SET_VALUE=0, $
                          UVALUE='menu_band', BUTTON_UVALUE=INDGEN(n_band),$
                          /NONEXCL)

base_inputR=WIDGET_BASE(base_input, /COL, /ALIGN_TOP,/FRAME)
dummy=WIDGET_LABEL(base_inputR, value= 'Image preview: ',/align_center)
dummy=WIDGET_LABEL(base_inputR, value='Select image # ')
base_button=WIDGET_BASE(base_inputR, /ROW, /FRAME,/ALIGN_CENTER)
state.id.main_im_fi = WIDGET_BUTTON(base_button,VALUE='FIRST', $
                                    UVALUE='im_fi',xsize=50)
state.id.main_im_pr = WIDGET_BUTTON(base_button,VALUE='PREV.', $
                                    UVALUE='im_pr',xsize=50)
state.id.main_im_ac = CW_FIELD(base_button,VALUE='',UVALUE='im_ac',xsize=3,$
                               /int, title='', /return_event, /row)
state.id.main_im_ne = WIDGET_BUTTON(base_button,VALUE='NEXT', $
                                    UVALUE='im_ne',xsize=50)
state.id.main_im_la = WIDGET_BUTTON(base_button,VALUE='LAST', $
                                    UVALUE='im_la',xsize=50)
baseAdraw=WIDGET_BASE(base_inputR, /ALIGN_CENTER, /ROW)
iareadraw=WIDGET_DRAW(baseAdraw, XSIZE=256, YSIZE=256,/button_events,$
                      uvalue='iarea', tooltip="Right click for options",$
                      keyboard_events=2)

base_input_bottom=WIDGET_BASE(base_input, /COL, /ALIGN_TOP, /FRAME)
state.id.oout=CW_FIELD(base_input_bottom, UVALUE='oout', /return_events, $
                       VALUE=state.par.N_out, /COLUMN, $
                       TITLE='Output size: [ENTER] ' )

;-------------------------------------------------
baseB=WIDGET_BASE(basetab, /COL, TITLE='PSF EXTRACTION')
;-------------------------------------------------
base_psf=WIDGET_BASE(baseB, /COL, /ALIGN_TOP)
                   
base_psf1=WIDGET_BASE(base_psf, /ROW, /ALIGN_TOP)

;LEFT PART: scale, zoom, buttons, etc...
base_psfL=WIDGET_BASE(base_psf1, /COL, /ALIGN_TOP, /FRAME)


dummy=WIDGET_LABEL(base_psfL, value=' ')
dummy=WIDGET_LABEL(base_psfL, value='Select image # ')
base_button=WIDGET_BASE(base_psfL, /ROW, /FRAME,/ALIGN_CENTER)
state.id.psf_im_fi = WIDGET_BUTTON(base_button,VALUE='FIRST', $
                     UVALUE='im_fi',xsize=50)
state.id.psf_im_pr = WIDGET_BUTTON(base_button,VALUE='PREV.', $
                     UVALUE='im_pr',xsize=50)
state.id.psf_im_ac = CW_FIELD(base_button,VALUE='',UVALUE='im_ac',xsize=3,$
                     /int, title='', /return_event, /row)
state.id.psf_im_ne = WIDGET_BUTTON(base_button,VALUE='NEXT', $
                     UVALUE='im_ne',xsize=50)
state.id.psf_im_la = WIDGET_BUTTON(base_button,VALUE='LAST', $
                     UVALUE='im_la',xsize=50)
                    
base_psfL1=WIDGET_BASE(base_psfL, /COL, /ALIGN_CENTER, xsize=250)
state.id.copy=WIDGET_BUTTON(base_psfL1,value='COPY',uvalue='copy')
dummy=WIDGET_LABEL(base_psfL1, value='Copy selected star(s) to ALL images', /align_center)
base_psfL2=WIDGET_BASE(base_psfL, /COL, /ALIGN_CENTER, xsize=250)
state.id.delete=WIDGET_BUTTON(base_psfL2,value='DELETE',uvalue='delete')
dummy=WIDGET_LABEL(base_psfL2, value='Delete all stars from ALL images', /align_center)
                
base_psfLL=WIDGET_BASE(base_psfL, /COL, /ALIGN_CENTER)
state.id.radius=CW_FIELD(base_psfLL, /COLUMN,  /return_events, $
                TITLE='Radius of selection: [ENTER]', VALUE=state.par.radius, $
                UVALUE='radius')
state.id.extrap=CW_BGROUP(base_psfLL, $
                          ['Extraction only','Extraction+Extrapolation'], $
                          LABEL_TOP='PSF Extraction mode:', $
                          set_value=state.par.extrap, $
                          uvalue='psf_extrap', /row, /EXCLUSIVE)

state.id.find_beta=CW_BGROUP(base_psfL, $
                   ['Known value','Automatic search procedure (*)'], $ 
                   LABEL_TOP='Moffat power law index (beta):', $
                   set_value=state.par.find_beta, $
                   UVALUE='find_beta', /row, /EXCLUSIVE) 
base_psfL3=WIDGET_BASE(base_psfL, /ROW, /ALIGN_CENTER)
base_psfL3a=WIDGET_BASE(base_psfL3, /COL, /ALIGN_TOP)
base_psfL3b=WIDGET_BASE(base_psfL3, /COL, /ALIGN_TOP)

state.id.moffat_beta=CW_FIELD(base_psfL3a, value=state.par.moffat_beta, $
                    UVALUE='moffat_beta', /COLUMN, /return_events, /FLOATING, $
                    TITLE='Value [ENTER]:',xsize=10 )
state.id.open_beta=WIDGET_BUTTON(base_psfL3b, value='Set parameters', $
                   uvalue='open_beta', xsize=150)
dummy=WIDGET_LABEL(base_psfL3b, value='(*) Only one reference-star', /ALIGN_LEFT)
dummy=WIDGET_LABEL(base_psfL3b, value='per image can be selected  ', /ALIGN_LEFT)
dummy=WIDGET_LABEL(base_psfL3b, value='in automatic search mode.  ', /ALIGN_LEFT)
                    
     
;RIGHT PART: THE IMAGE
base_psfR=WIDGET_BASE(base_psf1, /COL, /ALIGN_TOP, /FRAME)
state.id.pbase=WIDGET_BASE(base_psfR, /ALIGN_TOP)
state.id.pkill=WIDGET_DRAW(state.id.pbase, XSIZE=512, YSIZE=512, /SCROLL, $
    X_SCROLL_SIZE=512, Y_SCROLL_SIZE=512, KEYBOARD_EVENTS=2, $
    /button_events, uvalue='parea', /ALIGN_CENTER, $
    TOOLTIP='shift+click to select, ctrl+click to remove')

   
; -----------------------------------------------------------------------
;Standard buttons
; -----------------------------------------------------------------------
base2  = WIDGET_BASE(base_id, FRAME=10, /ROW)
help_id = WIDGET_BUTTON(base2, VALUE='HELP',UVALUE='help')
cancel_id  = WIDGET_BUTTON(base2, VALUE='CANCEL',UVALUE='cancel')
restore_id = WIDGET_BUTTON(base2, VALUE='RESTORE PARAMETERS',UVALUE='restore')
save_id    = WIDGET_BUTTON(base2, VALUE='SAVE PARAMETERS',UVALUE='save')

IF modal THEN WIDGET_CONTROL, cancel_id, /CANCEL_BUTTON
IF modal THEN WIDGET_CONTROL, save_id, /DEFAULT_BUTTON

WIDGET_CONTROL, base_id, /REALIZE
WIDGET_CONTROL, iareadraw, GET_VALUE=iID
state.id.iarea=iID
WIDGET_CONTROL, state.id.pkill, GET_VALUE=dID
state.id.parea=dID

;first time?
if par.firsttime EQ 'yes' then begin
   WIDGET_CONTROL, state.id.name, SET_VALUE=par.name
   WIDGET_CONTROL, state.id.radius, SET_VALUE=par.radius
   WIDGET_CONTROL, state.id.moffat_beta, SET_VALUE=par.moffat_beta
   widget_control, state.id.main_im_fi, SENSITIVE=0
   widget_control, state.id.main_im_pr, SENSITIVE=0
   widget_control, state.id.main_im_ac, SENSITIVE=0
   widget_control, state.id.main_im_ne, SENSITIVE=0
   widget_control, state.id.main_im_la, SENSITIVE=0
   widget_control, state.id.psf_im_fi, SENSITIVE=0
   widget_control, state.id.psf_im_pr, SENSITIVE=0
   widget_control, state.id.psf_im_ac, SENSITIVE=0
   widget_control, state.id.psf_im_ne, SENSITIVE=0
   widget_control, state.id.psf_im_la, SENSITIVE=0
   WIDGET_CONTROL, state.id.copy, SENSITIVE=0
   WIDGET_CONTROL, state.id.delete, SENSITIVE=0
   WIDGET_CONTROL, state.id.radius, SENSITIVE=0
   WIDGET_CONTROL, state.id.extrap, SENSITIVE=0

   WIDGET_CONTROL, state.id.find_beta, SENSITIVE=0
   WIDGET_CONTROL, state.id.moffat_beta, SENSITIVE=0
   WIDGET_CONTROL, state.id.open_beta, SENSITIVE=0
   
   WIDGET_CONTROL, state.id.oout, SENSITIVE=0
   
   WIDGET_CONTROL, state.id.lambda, SENSITIVE=0
   WIDGET_CONTROL, state.id.width, SENSITIVE=0
   WIDGET_CONTROL, state.id.resolution, SENSITIVE=0
   WIDGET_CONTROL, state.id.reset, SENSITIVE=0
   WIDGET_CONTROL, state.id.band, SENSITIVE=0
   pex_gui_bandset, state
endif else begin
   pex_gui_load, state 
endelse

if state.par.mode then WIDGET_CONTROL, state.id.name, SENSITIVE=0

WIDGET_CONTROL, base_id, SET_UVALUE=state
XMANAGER, 'pex_open_gui', base_id, GROUP_LEADER=group
return, {error:error, par:par}
end
