; $Id: pex_info.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    pex_info
;
; PURPOSE:
;    pex_info is the routine that returns the basic information for the
;    module PES of the Software Package AIRY (see pex.pro's header --or
;    file airy_help.html-- for details about the module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    info_structure = pex_info()
;
; OUTPUTS:
;    info_structure: structure containing the module's basic information.
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: January 2007,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it].
;    modifications  : ...
;
;-
;
function pex_info

pack_name= 'AIRY_Software_Package_7.2' ; Software Package name
help_file= 'airy_help.html'
help     = !caos_env.modules+pack_name+!caos_env.delim+'help'+$
           !caos_env.delim+help_file
           ; help file address for Software Package AIRY
ver      = fix(7)             ; version number of the Software Package used
mod_name = 'pex'
descr    = 'Psf EXtraction'
def_file = mk_par_name(mod_name, PACK_NAME=pack_name, /DEFAULT)  
           ;default parameter filename
inp_type = 'img_t'
inp_opt  = 1B
out_type = 'img_t'
init     = 0B ;an initialisation STRUCTURE is required (yes=1B, no=0B)
time     = 0B ;time integration/delay management is required (yes=1B, no=0B)

; convert in low case format and eliminate all the blanks
mod_name = strlowcase(strcompress(mod_name, /REMOVE_ALL))
inp_type = strlowcase(strcompress(inp_type, /REMOVE_ALL))
out_type = strlowcase(strcompress(out_type, /REMOVE_ALL))

; control the module name length
if strlen(mod_name) ne !caos_env.module_len then $
   message,'the module name must have '+strtrim(!caos_env.module_len)+' chars'

; resulting info structure
info = $
   {   $
   pack_name: pack_name,$
   help     : help,     $
   ver      : ver,      $
   mod_name : mod_name, $
   descr    : descr,    $
   def_file : def_file, $
   inp_type : inp_type, $
   inp_opt  : inp_opt,  $
   out_type : out_type, $
   init     : init,     $
   time     : time      $
   }

; back to calling program
return, info
end
