; $Id: pre_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    pre_gen_default
;
; PURPOSE:
;    pre_gen_default generates the default parameter structure for the pre
;    module and save it in the right location.
;    (see pre.pro's header --or file airy_help.html-- for details about the
;     module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    pre_gen_default
;
; ROUTINE MODIFICATION HISTORY:
;    routine written: june 2000,
;                     Serge Correia (OAA) [correia@arcetri.astro.it].
;    modifications  : february 2003,
;                     Marcel Carbillet (OAA) [marcel@arcetri.astro.it]:
;                    -"mod_type"->"mod_name"
;                     (for version 4.0 of the whole system CAOS).
;                   : october 2003,
;                     Barbara Anconelli (DISI) [anconelli@disi.unige.it]
;                    -insert possility for the user to subtract sky
;                     background in the image.
;                   : for version 5.0,
;                     Andrea La Camera (DISI) [lacamera@disi.unige.it]:
;                    -APODIZATION removed, SHIF-AND-ADD and DATA REDUCTION
;                     procedures added, RECENTERING changed.
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -background is now an array, which dimensions are
;                     the same of the input image. The user can now
;                     load the background contained in a FITS file
;                     (that must have the correct dimensions). 
;                    -BACK_ENABLE -> BACK_SUB (Better name!)
;                    -moved here SIGMA_RON and GAIN_RON (from DEC)
;-
;
pro pre_gen_default

; obtain module infos
info = pre_info()

; generate module description structure
module = gen_def_module(info.mod_name, info.ver)

back_val = make_array(360,/double)

; parameter structure
par = $
   {  $
   pre,                          $ ; structure named pre
   module       : module,        $ ; module description structure
;   apo_enable   : 0B,            $ ; Enable the apodization of the frame with apodizing filter 
;                                   ; (for fix the "wrap-around effect" and the artifacts arising in the 
;                                   ;  re-centering by FFTshift) 
;   marge_apo    : 10,            $ ; Marge parameter of the apodizing filter (zero padding band) [pixel].
;   sigma_apo    : 3.,            $ ; Sigma of the gaussian shape transition of the apodizing filter [pixel].
;   rec_enable   : 0B,            $ ; Enable the re-centering of the frames with respect to the first frame by  
;                                   ; FFTshift. 
;   rec_method   : 0B,            $ ; Method of computation of the photocenter for the re-centering procedure:
;                                   ;  0 : on the total aera over a treshold
;                                   ;  1 : on a circular aera around the maximum pixel.
;   line_rec     : 3,             $ ; number of x pixels in the left side band 
;                                   ; to evaluate the background level and rms for the 
;                                   ; recentring procedure. (band = [0:line_rec-1, *])
;   tresh_rec    : 5,             $ ; Treshold value of the array to perform the photocenter computation.
;                                   ; Expressed in background rms. (the background level is also added) 
;   radius_rec   : 10.,           $ ; Radius of the circular aera around the maximum pixel 
;                                   ; used to compute the photo-center.
   line_SNR     : 2,             $ ; number of x pixels in the left side band 
                                   ; to evaluate the noise rms value for the 
                                   ; SNR computation. (band = [0:line_rec-1, *])
   re_center_enable  : 0B,       $ ; Enable re-centering of the output
   re_center_accuracy: 1.,       $ ; Accuracy in pixel 
   buser_enable : 0B,	         $ ; enable user-defined sky background subtraction
   back_variable: 0B,            $ ; Different background selection 
                                   ; 0 = Constant value [default]
                                   ; 1 = Variable array
   back_val     : back_val,      $ ; background value [photons/pixel]
   back_file    : '',            $ ; filename of the background array [must be 
                                   ; a fits file] 			
   back_sub     : 0B,	         $ ; enable sky background subtraction 
   header_ron : 1B,    $           ; read from header the two values (yes by default)
   sigma_ron : 0.,     $           ; std dev of the read-out noise in [electrons/pixel]
   gain_ron  : 1.,     $           ; gain of the detector in [electrons/ADU]
   saa_enable   : 0B,            $ ; SHIFT AND ADD [multi-frames]
   saa_method   : 0B,            $ ; 0=only "ADD", 1=shift values are in a text file	
                                   ; 2=automatic, based on correlation	
   saa_file     : "",            $ ;  text file which contains shift values
   saa_auto     : 1.,            $ ; sub-pixel accuracy: default=1 pixel
   red_enable   : 0B,            $ ; enable data-reduction
   red_bp       : 0B,            $ ; take in account bad-pixel map 
   red_bp_file  : "",            $ ; bad-pixel map file
   red_flat     : 0B,            $ ; take in account flat-field map
   red_flat_file: "",            $ ; flat-field map file
   red_dark     : 0B,            $ ; take in account dark current map
   red_dark_file: "",            $ ; dark current map file
   red_bias     : 0B,            $ ; take in account bias map
   red_bias_file: "",            $ ; bias map file
   pos_enable   : 0B,            $ ; Enable the positivization of the frames
   pos_method   : 0B             $ ; Positivize values LT 0 or LE 0 (0 : LT 0, 1 : LE 0)
   }

; save the default parameters structure in the file def_file
save, par, FILENAME=info.def_file

end
