; $Id: rti_gen_default.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    rti_gen_default
;
; PURPOSE:
;    rti_gen_default generates the default parameter structure
;    for the RTI module and save it in the right location.
;    the user doesn't need to use rti_gen_default, it is used
;    only for developing and upgrading purposes.
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    rti_gen_default
; 
; MODIFICATION HISTORY:
;    program written: March 2007,
;            Andrea La Camera (DISI) [lacamera@disi.unige.it] 
;    modifications  : ...
;
;-
;
pro rti_gen_default
  
; obtain module infos
  info = rti_info()
  
; generate module description structure.
  module = gen_def_module(info.mod_name, info.ver)
  
  angle_val=make_array(360,/float,value=!VALUES.F_NAN)
  par = $
     {  $
     rti,                   $   ; structure named rti
     module : module,       $   ; module description structure
     method : 0B,           $   ; 0B = Pixels-Space Rotation,
                                ; 1B = Frequencies-Space Rotation 
     zp : 8,                $   ; zero-padding before FFT (in Freq.-Space Rot.)
     angle : angle_val,     $   ; array of angles 
     out : 1                $   ; output: original size (out=1), double size (out=2)
     }
  
; save the default parameter structure in the file def_file
  save, par, FILENAME=info.def_file
  
end
