; $Id: rti_info.pro v.7.2 last revision 2017/10/24 Andrea La Camera $ 
;+
; NAME:
;    rti_info
;
; PURPOSE:
;    rti_info is the routine that returns the basic information for the
;    module rti (see rti.pro's header --or file airy_help.html-- for details
;    about the module itself).
;
; CATEGORY:
;    module's utility routine
;
; CALLING SEQUENCE:
;    info_structure = rti_info()
; 
; OUTPUTS:
;    info_structure: structure containing the module's basic information.
;
; ROUTINE MODIFICATION HISTORY:
;    program written:  March 2007,
;					 Andrea La Camera (DISI) [lacamera@disi.unige.it]
;    modifications  : ...
;
;-
;
function rti_info

pack_name= 'AIRY_Software_Package_7.2' ; Software Package name
help_file= 'airy_help.html'
help     = !caos_env.modules+pack_name+!caos_env.delim+'help'+$
           !caos_env.delim+help_file   ; help-file address for Soft.Pack.AIRY
ver      = fix(7)                      ; version number of the Software Package used

mod_name = 'rti'                       ; module name
descr    = 'RoTate Image'              ; short module description
def_file = mk_par_name(mod_name, $
     PACK_NAME=pack_name, /DEFAULT)    ; default parameter file
                                       ; default parameters file name
init     = 0B                          ; initialization structure not needed
time     = 0B                          ; time integration/delay not needed
inp_type = 'gen_t'                     ; generic input type
inp_opt  = [1B]                        ; input needed
out_type = 'gen_t'                     ; generic output type
out_opt  = [1B]                        ; output needed

; convert in low case format and eliminate all the blanks
mod_name = strlowcase(strcompress(mod_name, /REMOVE_ALL))
inp_type = strlowcase(strcompress(inp_type, /REMOVE_ALL))
out_type = strlowcase(strcompress(out_type, /REMOVE_ALL))

; control the module name length
if strlen(mod_name) ne !caos_env.module_len then $
   message, 'the module name must have '+strtrim(!caos_env.module_len)+' chars'

; resulting info structure
info = $
   {   $
   pack_name: pack_name,$
   help     : help,     $
   ver      : ver,      $
   mod_name : mod_name, $
   descr    : descr,    $
   def_file : def_file, $
   inp_type : inp_type, $
   inp_opt  : inp_opt,  $
   out_type : out_type, $
   init     : init,     $
   time     : time      $
   }

; back to calling program
return, info
end
