; $Id: rti_fft_rotate.pro $
;+
; NAME:
;    rti_fft_rotate
;
; PURPOSE:
;
; CATEGORY:
;   library routines
;
; CALLING SEQUENCE:
; 	result = rti_fft_rotate(img, angle, [zero_padding=value])
;
; INPUTS:
; 	img   : the image to be rotate
; 	angle : angle (degrees) of rotation CLOCKWISE ROTATION
;
; OPTIONAL INPUTS:
; 	zero-padding : the zero-padding value (default=8)
;
; KEYWORD PARAMETERS:
; 	none
;
; OUTPUTS:
;	result : the rotated image
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
; 	none
;
; SIDE EFFECTS:
; 	none
;
; RESTRICTIONS:
; 	only 2D images, for cubes it's necessary call FFTROTATE several times
;
; MODIFICATION HISTORY:
;
;-
;
function rti_fft_rotate, img, angle, zero_padding=zp

on_error,2


N=(size(img))[1]
if (size(img))[0] EQ 3 then begin
	message, 'ONLY 2D IMAGES, first image will be rotated', /info
	img=img[*,*,0]
endif

flux = total(img,/double)
max_init = max(img)

;zero-padding
if keyword_set(zp) EQ 0 then zp=8
NN=zp*N
;print, 'zero padding = ', NN
;print, memory()/(1024.*1024.)
img_zp=make_array(NN,NN, /double)
img_zp[(NN-N)/2.,(NN-N)/2.]=img
;print, 'MEM (MB)=', memory()/(1024.*1024.)
	;FT
	img_zp = shift(img_zp,NN/2., NN/2.)
;print, 'MEM (MB)=',memory()/(1024.*1024.)
	img_zp = FFT(img_zp, -1, /double)*NN*NN
;print, 'MEM (MB)=',memory()/(1024.*1024.)
	img_zp = SHIFT(img_zp, NN/2., NN/2.)
;print, 'MEM (MB)=',memory()/(1024.*1024.)

	;ROT(FT)
	img_zp = rot(img_zp, angle, /interp, missing=complex(0.,0.))
;print, 'MEM (MB)=',memory()/(1024.*1024.)
	;FT^-1
	img_zp = FFT(shift(img_zp,NN/2.,NN/2.), +1, /double)/(NN*NN)
;print, 'MEM (MB)=',memory()/(1024.*1024.)
	img_zp = SHIFT(img_zp, NN/2., NN/2.)
;print, 'MEM (MB)=',memory()/(1024.*1024.)
	dummy = img_zp[NN/2.-N:NN/2.+N-1, NN/2.-N:NN/2.+N-1]
;print, 'MEM (MB)=',memory()/(1024.*1024.)
	dummy = float(dummy)
;print, 'MEM (MB)=',memory()/(1024.*1024.)

	;mask (black/white square)
	quad = make_array(2.*N,2.*N,/float,value=0.)
	quad[N/2.:3.*N/2.-1,N/2.:3.*N/2.-1]=1.
	quad = rot(quad, angle,/interp)  ;!!! WITH INTERP !!!
	quad = fix(quad)				 ;

	dummy = dummy * quad

	;positivization
	index = where(dummy LT 0., count)
	xx = index mod (2*N)
	yy = index / (2*N)
	dummy = dummy > 0

	xi=xx-1 > 0
	xf=xx+1 < (2.*N)-1
	yi=yy-1 > 0
	yf=yy+1 < (2.*N)-1

	;print, '#ZERI = ', count
	for k=0L,count-1 do dummy[xx[k],yy[k]]=total(double(dummy[xi[k]:xf[k],yi[k]:yf[k]]))/9.

	;normalization
	dummy = dummy / total(double(dummy)) * flux

;endelse


return, dummy
end
