; $Id: rti_rot_rotate.pro $
;+
; NAME:
;    rti_rot_rotate
;
; PURPOSE:
;
; CATEGORY:
;   library routines
;
; CALLING SEQUENCE:
; 	result = rti_rot_rotate(img, angle, [crc=value, missing=missing])
;
; INPUTS:
; 	img   : the image to be rotate
; 	angle : angle (degrees) of rotation CLOCKWISE ROTATION
;
; OPTIONAL INPUTS:
;	crc           : congrid-rot-congrid method (value must be an integer > 1):
;                     : congrid to NN=value*N pixel with /INTERP
;                     : zero-padding to 2*NN, rot with /INTERP
;                     : congrid to 2*N pixel
;	              : if crc is not present, only zero-padding & rot
;       missing       : value to be substituted for pixels in the
;                     : output image that map outside the input image. 
;
; OUTPUTS:
;	result : the rotated image
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
; 	none
;
; SIDE EFFECTS:
; 	none
;
; RESTRICTIONS:
; 	only 2D images, for cubes it's necessary call ROTROTATE several times
;
; MODIFICATION HISTORY:
;                   : May 2016,
;                     Andrea La Camera (DIBRIS) [andrea.lacamera@unige.it],
;                    -added missing optional input
;-
;
function rti_rot_rotate, img, angle, crc=crc_value, missing=missing

on_error,2

if keyword_set(missing) EQ 0 then missing=0.

N=(size(img))[1]
if (size(img))[0] EQ 3 then begin
	message, 'ONLY 2D IMAGES, first image will be rotated', /info
	img=img[*,*,0]
endif

flux = total(double(img))

if keyword_set(crc_value) EQ 0 then NN=N else NN=N*crc_value

;congrid
if keyword_set(crc_value) NE 0 then img = congrid(img,NN,NN,/interp)


;zero-padding
imgrot = make_array(2*NN,2*NN,/double, value=missing)
imgrot[NN/2.,NN/2.]=img
imgrot = rot(imgrot,angle,/interp, missing=missing)

;congrid
if keyword_set(crc_value) NE 0 then imgrot = congrid(imgrot,2*N,2*N,/interp)

;flux normalization
imgrot = imgrot / total(double(imgrot)) * flux

return, imgrot

end
